SystemOrganization addCategory: #'CollectionsTests-Abstract'!
SystemOrganization addCategory: #'CollectionsTests-Arrayed'!
SystemOrganization addCategory: #'CollectionsTests-Sequenceable'!
SystemOrganization addCategory: #'CollectionsTests-SkipLists'!
SystemOrganization addCategory: #'CollectionsTests-Stack'!
SystemOrganization addCategory: #'CollectionsTests-Streams'!
SystemOrganization addCategory: #'CollectionsTests-Support'!
SystemOrganization addCategory: #'CollectionsTests-Text'!
SystemOrganization addCategory: #'CollectionsTests-Unordered'!
SystemOrganization addCategory: #'CollectionsTests-Weak'!

Trait named: #TAddForUniquenessTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TAddForUniquenessTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 15:40'!
collection
	^ self explicitRequirement! !

!TAddForUniquenessTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 15:30'!
collectionWithElement
	^ self explicitRequirement! !

!TAddForUniquenessTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 15:40'!
element
	^ self explicitRequirement! !

!TAddForUniquenessTest methodsFor: 'tests - adding uniquely' stamp: 'stephane.ducasse 10/8/2008 15:31'!
testAddAlreadyThereDoesNotCount

	| added oldSize |
	oldSize := self collectionWithElement size.
	self assert: (self collectionWithElement includes: self element).

	added := self collectionWithElement add: self element.
	
	self assert: added = self element.
	self assert: (self collectionWithElement includes: self element).
	self assert: self collectionWithElement size = oldSize.! !

!TAddForUniquenessTest methodsFor: 'tests - adding uniquely' stamp: 'stephane.ducasse 10/8/2008 15:43'!
testAddNewElementIncrementsSize

	| added oldSize |
	oldSize := self collection size.
	self deny: (self collection includes: self element).

	added := self collection add: self element.
	
	self assert: added = self element.
	self assert: (self collection includes: self element).
	self assert: self collection size = (oldSize + 1).! !

!TAddForUniquenessTest methodsFor: 'tests - adding uniquely' stamp: 'stephane.ducasse 10/8/2008 16:33'!
testTAddIfNotPresentWithElementAlreadyIn

	| added oldSize |
	oldSize := self collectionWithElement size.
	self assert: (self collectionWithElement includes: self element).

	added := self collectionWithElement addIfNotPresent: self element.
	
	self assert: added = self element.
	self assert: (self collectionWithElement includes: self element).
	self assert: self collectionWithElement size = oldSize.! !

!TAddForUniquenessTest methodsFor: 'tests - adding uniquely' stamp: 'stephane.ducasse 10/8/2008 16:33'!
testTAddIfNotPresentWithNewElement

	| added oldSize |
	oldSize := self collection size.
	self deny: (self collection includes: self element).

	added := self collection addIfNotPresent: self element.
	
	self assert: added = self element.
	self assert: (self collection includes: self element).
	self assert: self collection size = (oldSize + 1).! !

Trait named: #TAddTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TAddTest methodsFor: 'requirements' stamp: 'damiencassou 1/20/2009 13:32'!
collectionWithElement
	"Returns a collection that already includes what is returned by #element."
	^ self explicitRequirement! !

!TAddTest methodsFor: 'requirements' stamp: 'damiencassou 1/14/2009 10:29'!
element
	"Returns an object that can be added to the collection returned by #collection."
	^ self explicitRequirement! !

!TAddTest methodsFor: 'requirements' stamp: 'damiencassou 1/14/2009 10:29'!
otherCollection
	"Returns a collection that does not include what is returned by #element."
	^ self explicitRequirement! !

!TAddTest methodsFor: 'tests - fixture' stamp: 'damiencassou 1/20/2009 13:32'!
test0FixtureRequirementsOfTAddTest
	self 
		shouldnt: [ self collectionWithElement ]
		raise: Exception.
	self 
		shouldnt: [ self otherCollection ]
		raise: Exception.
	self 
		shouldnt: [ self element ]
		raise: Exception.
	self assert: (self collectionWithElement includes: self element).
	self deny: (self otherCollection includes: self element)! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:25'!
testTAdd
	| added collection |
	collection :=self otherCollection copy.
	added := collection add: self element.
	
	self assert: added == self element.	"test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection includes: self element)	.
	self assert: (self collectionWithElement includes: self element).
	
	! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:27'!
testTAddAll
	| added collection toBeAdded |
	collection := self collectionWithElement .
	toBeAdded := self otherCollection .
	added := collection addAll: toBeAdded .
	self assert: added == toBeAdded .	"test for identiy because #addAll: has not reason to copy its parameter."
	self assert: (collection includesAllOf: toBeAdded )! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:29'!
testTAddIfNotPresentWithElementAlreadyIn

	| added oldSize collection element |
	collection := self collectionWithElement .
	oldSize := collection size.
	element := self element .
	self assert: (collection  includes: element ).
	
	added := collection  addIfNotPresent: element .
	
	self assert: added == element .	"test for identiy because #add: has not reason to copy its parameter."
	self assert: collection  size = oldSize! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:32'!
testTAddIfNotPresentWithNewElement

	| added oldSize collection element |
	collection := self otherCollection .
	oldSize := collection  size.
	element := self element .
	self deny: (collection  includes: element ).
	
	added := collection  addIfNotPresent: element .
	self assert: added == element . "test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection  size = (oldSize + 1)).

	! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:38'!
testTAddTwice
	| added oldSize collection element |
	collection := self collectionWithElement .
	element := self element .
	oldSize := collection  size.
	added := collection 
		add: element ;
		add: element .
	self assert: added == element .	"test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection  includes: element ).
	self assert: collection  size = (oldSize + 2)! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:35'!
testTAddWithOccurences
	| added oldSize collection element |
	collection := self collectionWithElement .
	element := self element .
	oldSize := collection  size.
	added := collection  add: element withOccurrences: 5.
	
	self assert: added == element.	"test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection  includes: element).
	self assert: collection  size = (oldSize + 5)! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:38'!
testTWrite
	| added collection element |
	collection := self otherCollection  .
	element := self element .
	added := collection  write: element .
	
	self assert: added == element .	"test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection  includes: element )	.
	self assert: (collection  includes: element ).
	
	! !

!TAddTest methodsFor: 'tests - adding' stamp: 'delaunay 4/21/2009 15:39'!
testTWriteTwice
	| added oldSize collection element |
	collection := self collectionWithElement .
	element := self element .
	oldSize := collection  size.
	added := collection 
		write: element ;
		write: element .
	self assert: added == element .	"test for identiy because #add: has not reason to copy its parameter."
	self assert: (collection  includes: element ).
	self assert: collection  size = (oldSize + 2)! !

Trait named: #TAsStringCommaAndDelimiterSequenceableTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'requirements' stamp: 'stephane.ducasse 1/19/2009 16:17'!
empty

	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'requirements' stamp: 'delaunay 4/8/2009 11:03'!
nonEmpty

	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:56'!
nonEmpty1Element
" return a collection of size 1 including one element"
	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'stephane.ducasse 1/19/2009 16:17'!
oneItemCol 

	^ self explicitRequirement! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'stephane.ducasse 1/19/2009 16:17'!
oneTwoThreeItemCol 

	^ self explicitRequirement! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/8/2009 11:36'!
test0FixtureAsStringCommaAndDelimiterTest
	
	self shouldnt: [self nonEmpty] raise:Error .
	self deny: self nonEmpty isEmpty.
	
	self shouldnt: [self empty] raise:Error .
	self assert: self empty isEmpty.
	
       self shouldnt: [self nonEmpty1Element ] raise:Error .
	self assert: self nonEmpty1Element size=1.! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/8/2009 11:04'!
testAsCommaStringEmpty

	self assert: self empty asCommaString = ''.
	self assert: self empty asCommaStringAnd = ''.
	

! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/10/2009 11:20'!
testAsCommaStringMore

	"self assert: self oneTwoThreeItemCol asCommaString = '1, 2, 3'.
	self assert: self oneTwoThreeItemCol asCommaStringAnd = '1, 2 and 3'
"

	| result resultAnd index allElementsAsString |
	result:= self nonEmpty asCommaString .
	resultAnd:= self nonEmpty asCommaStringAnd .
"	1 to: self nonEmpty size by:3 do:
		[:i | 
		self assert: (result at: i)=((self nonEmpty at:((i+3-1)/3)) asString asCharacter).
		self assert: (result at:(i+1))=$,.
		self assert: (result at:(i+2))=$ .
		]
	"
	"verifying result  :"
	index := 1.
	(result findBetweenSubStrs: ', ' )do:
		[:each |
		self assert: each= ((self nonEmpty at:index)asString).
		index:=index+1
		].
	
	"verifying esultAnd :"
	allElementsAsString:=(resultAnd findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 )
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString)].
		i=(allElementsAsString size-1)
			ifTrue:[ self assert: (allElementsAsString at:i)=('and')].
		i=(allElementsAsString size)
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:(i-1))asString)].
			].! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/8/2009 11:38'!
testAsCommaStringOne
	
	"self assert: self oneItemCol asCommaString = '1'.
	self assert: self oneItemCol asCommaStringAnd = '1'."

	self assert: self nonEmpty1Element  asCommaString = (self nonEmpty1Element first asString).
	self assert: self nonEmpty1Element  asCommaStringAnd = (self nonEmpty1Element first asString).
	! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'stephane.ducasse 1/16/2009 10:23'!
testAsStringOnDelimiterEmpty

	| delim emptyStream |
	delim := ', '.
	emptyStream := ReadWriteStream on: ''.
	self empty asStringOn: emptyStream delimiter: delim.
	self assert: emptyStream contents = ''.
! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/9/2009 09:43'!
testAsStringOnDelimiterLastEmpty

	| delim emptyStream |
	delim := ', '.
	emptyStream := ReadWriteStream on: ''.
	self empty asStringOn: emptyStream delimiter: delim last:'and'.
	self assert: emptyStream contents = ''.
! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/9/2009 09:39'!
testAsStringOnDelimiterLastMore

	| delim multiItemStream result last allElementsAsString |
	
	delim := ', '.
	last := 'and'.
	result:=''.
	multiItemStream := ReadWriteStream on:result.
	self nonEmpty  asStringOn: multiItemStream delimiter: ', ' last: last.
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 )
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString)].
		i=(allElementsAsString size-1)
			ifTrue:[ self deny: (allElementsAsString at:i)=(last)asString].
		i=(allElementsAsString size)
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:(i-1))asString)].
			].
	
! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/9/2009 09:43'!
testAsStringOnDelimiterLastOne

	| delim oneItemStream result |
	
	delim := ', '.
	result:=''.
	oneItemStream := ReadWriteStream on: result.
	self nonEmpty1Element  asStringOn: oneItemStream delimiter: delim last: 'and'.
	oneItemStream do:[:each | self assert: each = (self nonEmpty1Element first asString)].
	

	! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/8/2009 11:53'!
testAsStringOnDelimiterMore

	| delim multiItemStream result index |
	"delim := ', '.
	multiItemStream := '' readWrite.
	self oneTwoThreeItemCol asStringOn: multiItemStream delimiter: ', '.
	self assert: multiItemStream contents = '1, 2, 3'."
	
	delim := ', '.
	result:=''.
	multiItemStream := ReadWriteStream on:result.
	self nonEmpty  asStringOn: multiItemStream delimiter: ', '.
	
	index:=1.
	(result findBetweenSubStrs: ', ' )do:
		[:each |
		self assert: each= ((self nonEmpty at:index)asString).
		index:=index+1
		].! !

!TAsStringCommaAndDelimiterSequenceableTest methodsFor: 'tests - comma and delimiter' stamp: 'delaunay 4/8/2009 14:43'!
testAsStringOnDelimiterOne

	| delim oneItemStream result |
	"delim := ', '.
	oneItemStream := '' readWrite.
	self oneItemCol asStringOn: oneItemStream delimiter: delim.
	self assert: oneItemStream contents = '1'."
	
	delim := ', '.
	result:=''.
	oneItemStream := ReadWriteStream on: result.
	self nonEmpty1Element  asStringOn: oneItemStream delimiter: delim.
	oneItemStream do:[:each | self assert: each = (self nonEmpty1Element first asString)].
	

	! !

Trait named: #TAsStringCommaAndDelimiterTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TAsStringCommaAndDelimiterTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 09:57'!
empty

	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 09:57'!
nonEmpty

	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 09:57'!
nonEmpty1Element
" return a collection of size 1 including one element"
	^ self explicitRequirement ! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 14:41'!
test0FixtureAsStringCommaAndDelimiterTest
	
	self shouldnt: [self nonEmpty] raise:Error .
	self deny: self nonEmpty isEmpty.
	
	self shouldnt: [self empty] raise:Error .
	self assert: self empty isEmpty.
	
       self shouldnt: [self nonEmpty1Element ] raise:Error .
	self assert: self nonEmpty1Element size=1.! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 14:41'!
testAsCommaStringEmpty

	self assert: self empty asCommaString = ''.
	self assert: self empty asCommaStringAnd = ''.
	
! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/21/2009 09:56'!
testAsCommaStringMore

	| result resultAnd index allElementsAsString tmp |
	result:= self nonEmpty asCommaString .
	resultAnd:= self nonEmpty asCommaStringAnd .
	tmp := self nonEmpty collect: [ :each | each asString].

	"verifying result  :"
	index := 1.
	allElementsAsString := (result findBetweenSubStrs: ', ' ).
	allElementsAsString do:
		[:each |
		self assert: (tmp occurrencesOf: each)=(allElementsAsString occurrencesOf: each).
		].
	
	"verifying esultAnd :"
	allElementsAsString:=(resultAnd findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 ) | i= allElementsAsString size
			ifTrue: [self assert: (tmp occurrencesOf:(allElementsAsString at:i))=(allElementsAsString  occurrencesOf:(allElementsAsString at:i))].
		i=(allElementsAsString size-1)
			ifTrue:[ self assert: (allElementsAsString at:i)=('and')].
			].! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 14:59'!
testAsCommaStringOne
	
	self nonEmpty1Element do: 
		[:each | 
		self assert: each asString =self nonEmpty1Element  asCommaString.
		self assert: each asString=self nonEmpty1Element  asCommaStringAnd.].

	! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 14:42'!
testAsStringOnDelimiterEmpty

	| delim emptyStream |
	delim := ', '.
	emptyStream := ReadWriteStream on: ''.
	self empty asStringOn: emptyStream delimiter: delim.
	self assert: emptyStream contents = ''.
! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 14:42'!
testAsStringOnDelimiterLastEmpty

	| delim emptyStream |
	delim := ', '.
	emptyStream := ReadWriteStream on: ''.
	self empty asStringOn: emptyStream delimiter: delim last:'and'.
	self assert: emptyStream contents = ''.
! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 15:16'!
testAsStringOnDelimiterLastMore

	| delim multiItemStream result last allElementsAsString tmp |
	
	delim := ', '.
	last := 'and'.
	result:=''.
	tmp := self nonEmpty collect: [:each | each asString].
	multiItemStream := ReadWriteStream on:result.
	self nonEmpty  asStringOn: multiItemStream delimiter: ', ' last: last.
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 ) | i= allElementsAsString size
			ifTrue: [self assert: (tmp occurrencesOf:(allElementsAsString at:i))=(allElementsAsString 			occurrencesOf:(allElementsAsString at:i))].
		i=(allElementsAsString size-1)
			ifTrue:[ self assert: (allElementsAsString at:i)=('and')].
			].
! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 15:09'!
testAsStringOnDelimiterLastOne

	| delim oneItemStream result |
	
	delim := ', '.
	result:=''.
	oneItemStream := ReadWriteStream on: result.
	self nonEmpty1Element  asStringOn: oneItemStream delimiter: delim last: 'and'.
	oneItemStream  do:
		[:each1 |
		self nonEmpty1Element do: [:each2 |self assert: each1 = (each2 asString) ]
		 ].
	

! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 15:12'!
testAsStringOnDelimiterMore

	| delim multiItemStream result allElementsAsString tmp |
	
	
	delim := ', '.
	result:=''.
	tmp:= self nonEmpty collect:[:each | each asString].
	multiItemStream := ReadWriteStream on:result.
	self nonEmpty  asStringOn: multiItemStream delimiter: ', '.
	
	allElementsAsString := (result findBetweenSubStrs: ', ' ).
	allElementsAsString do:
		[:each |
		self assert: (tmp occurrencesOf: each)=(allElementsAsString occurrencesOf: each).
		].! !

!TAsStringCommaAndDelimiterTest methodsFor: 'tests - as string comma delimiter sequenceable' stamp: 'delaunay 4/14/2009 15:06'!
testAsStringOnDelimiterOne

	| delim oneItemStream result |

	delim := ', '.
	result:=''.
	oneItemStream := ReadWriteStream on: result.
	self nonEmpty1Element  asStringOn: oneItemStream delimiter: delim.
	oneItemStream  do:
		[:each1 |
		self nonEmpty1Element do: [:each2 |self assert: each1 = (each2 asString) ]
		 ].
	
! !

Trait named: #TBeginsEndsWith
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TBeginsEndsWith methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 15:04'!
empty
	self explicitRequirement.! !

!TBeginsEndsWith methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 14:58'!
nonEmpty
	self explicitRequirement.! !

!TBeginsEndsWith methodsFor: 'tests - fixture' stamp: 'delaunay 4/9/2009 15:09'!
test0FixtureBeginsEndsWithTest

	self shouldnt: [self nonEmpty ] raise: Error.
	self deny: self nonEmpty isEmpty.
	self assert: self nonEmpty size>1.
	
	self shouldnt: [self empty ] raise: Error.
	self assert: self empty isEmpty.! !

!TBeginsEndsWith methodsFor: 'tests - begins ends with' stamp: 'delaunay 4/9/2009 15:03'!
testsBeginsWith
	
	self assert: (self nonEmpty beginsWith:(self nonEmpty copyUpTo: self nonEmpty size)).
	self assert: (self nonEmpty beginsWith:(self nonEmpty )).
	self deny: (self nonEmpty beginsWith:(self nonEmpty copyWith:self nonEmpty first)).! !

!TBeginsEndsWith methodsFor: 'tests - begins ends with' stamp: 'delaunay 4/9/2009 15:11'!
testsBeginsWithEmpty
	
	self deny: (self nonEmpty beginsWith:(self empty)).
	self deny: (self empty beginsWith:(self nonEmpty )).
! !

!TBeginsEndsWith methodsFor: 'tests - begins ends with' stamp: 'delaunay 4/9/2009 15:05'!
testsEndsWith
	
	self assert: (self nonEmpty endsWith:(self nonEmpty copyWithoutFirst)).
	self assert: (self nonEmpty endsWith:(self nonEmpty )).
	self deny: (self nonEmpty endsWith:(self nonEmpty copyWith:self nonEmpty first)).! !

!TBeginsEndsWith methodsFor: 'tests - begins ends with' stamp: 'delaunay 4/9/2009 15:11'!
testsEndsWithEmpty
	
	self deny: (self nonEmpty endsWith:(self empty )).
	self deny: (self empty  endsWith:(self nonEmpty )).
	! !

Trait named: #TCloneTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCloneTest methodsFor: 'helper' stamp: 'stephane.ducasse 1/12/2009 14:57'!
empty

	^ self explicitRequirement! !

!TCloneTest methodsFor: 'helper' stamp: 'stephane.ducasse 1/12/2009 14:57'!
nonEmpty

	^ self explicitRequirement! !

!TCloneTest methodsFor: 'tests - copy - clone' stamp: 'stephane.ducasse 11/21/2008 15:43'!
testCopyCreatesNewObject
	"self debug: #testCopyCreatesNewObject"
	
	| copy | 
	copy := self nonEmpty copy.
	self deny: self nonEmpty == copy.
	! !

!TCloneTest methodsFor: 'tests - copy - clone' stamp: 'stephane.ducasse 11/21/2008 15:57'!
testCopyEmpty
	"self debug: #testCopyEmpty"
	
	| copy | 
	copy := self empty copy.
	self assert: copy isEmpty.! !

!TCloneTest methodsFor: 'tests - copy - clone' stamp: 'stephane.ducasse 11/21/2008 15:57'!
testCopyNonEmpty
	"self debug: #testCopyNonEmpty"
	
	| copy | 
	copy := self nonEmpty copy.
	self deny: copy isEmpty.
	self assert: copy size = self nonEmpty size.
	self nonEmpty do: 
		[:each | copy includes: each]! !

Trait named: #TConcatanationTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TConcatanationTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 11:49'!
empty
	self explicitRequirement.! !

!TConcatanationTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 11:49'!
nonEmpty
	self explicitRequirement.! !

!TConcatanationTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 11:52'!
test0FixtureConcatenationTest
	self shouldnt: [self nonEmpty ] raise: Error.
	self deny: self nonEmpty isEmpty.
	self shouldnt: [self empty ] raise: Error.
	self assert: self empty isEmpty.! !

!TConcatanationTest methodsFor: 'tests - concatenation' stamp: 'delaunay 4/14/2009 11:53'!
testConcatenation! !

!TConcatanationTest methodsFor: 'tests - concatenation' stamp: 'delaunay 4/14/2009 11:57'!
testConcatenationWithEmpty
	
	| result |
	result := self nonEmpty , self empty .
	
	self assert: result = self nonEmpty . 
	
! !

Trait named: #TConvertAsSetForMultiplinessTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TConvertAsSetForMultiplinessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:03'!
collectionWithFloatAndCopy
	"return a collection of type 'self collectionWIithoutEqualsElements clas' containing no elements equals ( with identity equality)
	but  2 elements only equals with classic equality"
	| result collection |
	
	collection := OrderedCollection  withAll: self floatCollectionWithoutEqualElements .
	collection add: (collection first copy).
	result:= self floatCollectionWithoutEqualElements class withAll: collection .
	^ result .! !

!TConvertAsSetForMultiplinessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:04'!
collectionWithFloatAndDuplicate
	"return a collection of type : 'self collectionWIithoutEqualsElements class containing two elements equals ( with identity equality)"
	| result collection element |
	
	collection := OrderedCollection  withAll: self floatCollectionWithoutEqualElements .
	element:= collection first.
	collection  add: element.
	result:= self floatCollectionWithoutEqualElements class withAll: collection .
	^ result.! !

!TConvertAsSetForMultiplinessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:07'!
floatCollectionWithEqualElements
" return a collection of float including equal elements (classic equality)"
	^ self explicitRequirement! !

!TConvertAsSetForMultiplinessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:08'!
floatCollectionWithoutEqualElements
" return a collection that does niot incllude equal elements ( classic equality )"
	^ self explicitRequirement! !

!TConvertAsSetForMultiplinessTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/20/2009 15:05'!
test0FixtureTConvertAsSetForMultiplinessTest
	"a collection ofFloat with equal elements:"
	| res |
	self shouldnt: [ self floatCollectionWithEqualElements ]raise: Error.
	self shouldnt: [ self floatCollectionWithEqualElements do: [ :each | self assert: each class=Float ] ]raise: Error.
	
	res := true.
	self floatCollectionWithEqualElements 
		detect: [ :each | (self floatCollectionWithEqualElements occurrencesOf: each) > 1 ]
		ifNone: [ res := false ].
	self assert: res = true.
	
	"a collection of Float without equal elements:"
	
	self shouldnt: [ self floatCollectionWithoutEqualElements ]raise: Error.
	self shouldnt: [ self floatCollectionWithoutEqualElements do: [ :each | self assert: each class=Float ] ]raise: Error.
	
	res := true.
	self floatCollectionWithoutEqualElements 
		detect: [ :each | (self floatCollectionWithoutEqualElements occurrencesOf: each) > 1 ]
		ifNone: [ res := false ].
	self assert: res = false.
	

! !

!TConvertAsSetForMultiplinessTest methodsFor: 'tests - as set tests' stamp: 'delaunay 4/16/2009 13:58'!
testAsIdentitySetWithIdentityEqualsElements
	
	| result |
	result := self collectionWithFloatAndDuplicate   asIdentitySet  .
	" Only one element should have been removed as two elements are equals with Identity equality"
	
	self assert: result size= (self collectionWithFloatAndDuplicate size -1). 
	
	self collectionWithFloatAndDuplicate    do:
	[ :each | 
	(self collectionWithFloatAndDuplicate occurrencesOf: each )>1
		"the two elements equals only with classic equality shouldn't 'have been removed"
		ifTrue: [self assert: (result asOrderedCollection occurrencesOf: each )= 1 ]
		" the other elements are still here"
		ifFalse: [ self assert: (result asOrderedCollection occurrencesOf: each )= 1 ].
	].
	
	self assert: result class =IdentitySet.! !

!TConvertAsSetForMultiplinessTest methodsFor: 'tests - as set tests' stamp: 'delaunay 4/16/2009 13:50'!
testAsIdentitySetWithoutIdentityEqualsElements
	
	| result |
	result := self collectionWithFloatAndCopy asIdentitySet.
	" no elements should have been removed as no elements are equels with Identity equality"
	self assert: result size= self collectionWithFloatAndCopy size.
	
	
	self collectionWithFloatAndCopy   do: 
		[ :each | 
		(self collectionWithFloatAndCopy occurrencesOf: each) = (result asOrderedCollection occurrencesOf: each)
		].
	
	self assert: result class = IdentitySet.! !

!TConvertAsSetForMultiplinessTest methodsFor: 'tests - as set tests' stamp: 'delaunay 4/20/2009 15:05'!
testAsSetWithEqualsElements
	| result |
	result := self floatCollectionWithEqualElements  asSet  .
	
	self floatCollectionWithEqualElements   do:[ :each | self assert: (result occurrencesOf: each )= 1 ].
	self assert: result class = Set.
	! !

Trait named: #TConvertTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TConvertTest methodsFor: 'tests - converting' stamp: 'delaunay 3/27/2009 09:43'!
assertNoDuplicates: aCollection whenConvertedTo: aClass 
	| result |
	result := self collectionWithEqualElements asIdentitySet.
	self assert: (result class includesBehavior: IdentitySet).
	self collectionWithEqualElements do: [ :initial | self assert: (result occurrencesOf: initial) = 1 ]! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:36'!
assertNonDuplicatedContents: aCollection whenConvertedTo: aClass 
	| result |
	result := aCollection perform: ('as' , aClass name) asSymbol.
	self assert: (result class includesBehavior: aClass).
	result do: 
		[ :each | 
		self assert: (aCollection occurrencesOf: each) = (result occurrencesOf: each) ].
	^ result! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:37'!
assertSameContents: aCollection whenConvertedTo: aClass 
	| result |
	result := self assertNonDuplicatedContents: aCollection whenConvertedTo: aClass.
	self assert: result size = aCollection size! !

!TConvertTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:59'!
collectionWithoutEqualElements
" return a collection without equal elements"
	^ self explicitRequirement! !

!TConvertTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:00'!
integerCollectionWithoutEqualElements
" return a collection of integer without equal elements"
	^ self explicitRequirement! !

!TConvertTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/20/2009 10:28'!
test0FixtureTConvertTest
	"a collection of number without equal elements:"
	| res |
	self shouldnt: [ self collectionWithoutEqualElements ]raise: Error.
		
	res := true.
	self collectionWithoutEqualElements 
		detect: [ :each | (self collectionWithoutEqualElements occurrencesOf: each) > 1 ]
		ifNone: [ res := false ].
	self assert: res = false.
	
	self shouldnt: [self integerCollectionWithoutEqualElements ] raise: Error.
	self integerCollectionWithoutEqualElements  do: [ :each | self assert: each class = SmallInteger] .
	res := true.
	self integerCollectionWithoutEqualElements  
		detect: [ :each | (self integerCollectionWithoutEqualElements  occurrencesOf: each) > 1 ]
		ifNone: [ res := false ].
	self assert: res = false.
	! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 14:55'!
testAsArray
	"self debug: #testAsArray3"
	self 
		assertSameContents: self collectionWithoutEqualElements
		whenConvertedTo: Array! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:49'!
testAsBag

	self assertSameContents: self collectionWithoutEqualElements whenConvertedTo: Bag! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'delaunay 4/20/2009 10:28'!
testAsByteArray

	self assertSameContents: self integerCollectionWithoutEqualElements  whenConvertedTo: ByteArray! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'delaunay 4/15/2009 15:33'!
testAsIdentitySet
	"test with a collection without equal elements :"
	self 
		assertSameContents: self collectionWithoutEqualElements
		whenConvertedTo: IdentitySet.
! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:42'!
testAsOrderedCollection
	
	self assertSameContents: self collectionWithoutEqualElements whenConvertedTo: OrderedCollection! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'delaunay 4/15/2009 15:36'!
testAsSet
	| |
	"test with a collection without equal elements :"
	self assertSameContents: self collectionWithoutEqualElements whenConvertedTo: Set.
	! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/25/2009 16:56'!
testAsSortedArray
	| result |
	result := self collectionWithoutEqualElements asSortedArray.
	self assert: (result class includesBehavior: Array).
	self assert: result isSorted.
	self assert: result size = self collectionWithoutEqualElements size! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:45'!
testAsSortedCollection

	self assertSameContents: self collectionWithoutEqualElements whenConvertedTo: SortedCollection! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'delaunay 3/27/2009 10:08'!
testAsSortedCollectionWithSortBlock
	| result tmp |
	result := self collectionWithoutEqualElements asSortedCollection: [:a :b | a > b].
	self assert: (result class includesBehavior: SortedCollection).
	result do: 
		[ :each | 
		self assert: (self collectionWithoutEqualElements  occurrencesOf: each) = (result occurrencesOf: each) ].
	self assert: result size = self collectionWithoutEqualElements size.
	tmp:=result at: 1.
	result do: [:each| self assert: tmp>=each. tmp:=each].
	
	
	! !

!TConvertTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 11:06'!
testTopoligicallySortedUsing
	| result tmp |
	result := self collectionWithoutEqualElements topologicallySortedUsing: [:a :b | a > b].
	self assert: (result class includesBehavior: SortedCollection).
	result includesAnyOf: self collectionWithoutEqualElements.
	self assert: result size = self collectionWithoutEqualElements size.
	tmp:=result at: 1.
	result do: [:each| self assert: tmp>=each. tmp:=each].
	
	
	! !

Trait named: #TCopyPartOfSequenceable
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopyPartOfSequenceable methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:31'!
collectionWithoutEqualsElements

" return a collection not including equal elements "
	self explicitRequirement! !

!TCopyPartOfSequenceable methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 15:47'!
empty
	self explicitRequirement! !

!TCopyPartOfSequenceable methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:32'!
indexInForCollectionWithoutDuplicates
" return an index between 'collectionWithoutEqualsElements'  bounds"
	self explicitRequirement! !

!TCopyPartOfSequenceable methodsFor: 'tests - fixture' stamp: 'delaunay 4/16/2009 15:47'!
test0FixtureCopyPartOfSequenceableTest
	
	self shouldnt: [self collectionWithoutEqualsElements ] raise: Error.
	self collectionWithoutEqualsElements do:
		[:each | self assert: (self collectionWithoutEqualsElements occurrencesOf: each)=1].
	
	self shouldnt: [self indexInForCollectionWithoutDuplicates ] raise: Error.
	self assert: self indexInForCollectionWithoutDuplicates >0 & self indexInForCollectionWithoutDuplicates < self collectionWithoutEqualsElements size.
	
	self shouldnt: [self empty] raise: Error.
	self assert: self empty isEmpty .! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:55'!
testCopyAfter
	| result index |
	index:= self indexInForCollectionWithoutDuplicates .
	result := self collectionWithoutEqualsElements  copyAfter: (self collectionWithoutEqualsElements at:index ).
	
	"verifying content: "
	(1) to: result size do: 
		[:i |
		self assert: (self collectionWithoutEqualsElements  at:(i + index ))=(result at: (i))].

	"verify size: "
	self assert: result size = (self collectionWithoutEqualsElements  size - index).! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:18'!
testCopyAfterEmpty
	| result |
	result := self empty copyAfter: self collectionWithoutEqualsElements first.
	self assert: result isEmpty.
	! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:26'!
testCopyAfterLast
	| result index collectionWithDuplicates element |
	index:= self indexInForCollectionWithoutDuplicates .
	element:= self collectionWithoutEqualsElements at:index.
	collectionWithDuplicates:= self collectionWithoutEqualsElements copyWithFirst: (element). 
	
	result := collectionWithDuplicates   copyAfterLast: element.
	
	"verifying content: "
	1 to: result  size do: 
		[:i |
		self assert: (collectionWithDuplicates  at:(i + index +1))=(result at: (i))].

	"verify size: "
	self assert: result size = (collectionWithDuplicates size - (index +1)).! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:27'!
testCopyAfterLastEmpty
	| result |
	result := self empty copyAfterLast: self collectionWithoutEqualsElements first.
	self assert: result isEmpty.! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:27'!
testCopyEmptyMethod
	| result |
	result := self collectionWithoutEqualsElements  copyEmpty .
	self assert: result isEmpty .
	self assert: result class= self nonEmpty class.! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 16:11'!
testCopyFromTo
	| result  index |
	index:=self indexInForCollectionWithoutDuplicates .
	result := self collectionWithoutEqualsElements  copyFrom: index  to: self collectionWithoutEqualsElements size .
	
	"verify content of 'result' : "
	1 to: result size do:
		[:i | 
		self assert: (result at:i)=(self collectionWithoutEqualsElements at: (i + index - 1))].
	
	"verify size of 'result' : "
	self assert: result size = (self collectionWithoutEqualsElements size - index + 1).! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 16:00'!
testCopyUpTo
	| result index |
	index:= self indexInForCollectionWithoutDuplicates .
	result := self collectionWithoutEqualsElements  copyUpTo: (self collectionWithoutEqualsElements at:index).
	
	"verify content of 'result' :"
	1 to: result size do: [:i| self assert: (self collectionWithoutEqualsElements  at:i)=(result at:i)].
	
	"verify size of 'result' :"
	self assert: result size = (index-1).
	! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:28'!
testCopyUpToEmpty
	| result |
	result := self empty copyUpTo: self collectionWithoutEqualsElements first.
	self assert: result isEmpty.
	! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:39'!
testCopyUpToLast
	| result index collectionWithDuplicates element |
	index:= self indexInForCollectionWithoutDuplicates .
	element:= self collectionWithoutEqualsElements at:index.
	collectionWithDuplicates:= self collectionWithoutEqualsElements copyWithFirst: (element). 
	
	result := collectionWithDuplicates   copyUpToLast: element.
	
	"verify content of 'result' :"
	1 to: result size do: 
		[:i| 
		self assert: (collectionWithDuplicates at:i)=(result at:i)].
	
	"verify size of 'result' :"
	self assert: result size = (index).! !

!TCopyPartOfSequenceable methodsFor: 'tests - copying part of sequenceable' stamp: 'delaunay 4/16/2009 15:29'!
testCopyUpToLastEmpty
	| result |
	result := self empty copyUpToLast: self collectionWithoutEqualsElements first.
	self assert: result isEmpty.! !

Trait named: #TCopyPartOfSequenceableForMultipliness
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopyPartOfSequenceableForMultipliness methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:56'!
collectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals .
(others elements of the collection are not equal to those elements)"
	self explicitRequirement! !

!TCopyPartOfSequenceableForMultipliness methodsFor: 'tests - fixture' stamp: 'delaunay 4/22/2009 10:56'!
test0FixtureCopyPartOfForMultipliness

self shouldnt: [self collectionWithSameAtEndAndBegining  ] raise: Error.

self assert: self collectionWithSameAtEndAndBegining  first = self collectionWithSameAtEndAndBegining  last.

self assert: self collectionWithSameAtEndAndBegining  size > 1.

1 to: self collectionWithSameAtEndAndBegining  size do:
	[:i |
	(i > 1 ) & (i < self collectionWithSameAtEndAndBegining  size)
		ifTrue: [ self deny: (self collectionWithSameAtEndAndBegining  at:i) = (self collectionWithSameAtEndAndBegining  first)].
	]! !

!TCopyPartOfSequenceableForMultipliness methodsFor: 'tests - copying part of sequenceable for multipliness' stamp: 'delaunay 4/22/2009 11:06'!
testCopyAfterLastWithDuplicate
	| result element  collection |
	collection := self collectionWithSameAtEndAndBegining .
	element := collection  first.
	
	" collectionWithSameAtEndAndBegining first and last elements are equals.
	'copyAfter:' should copy after the last occurence of element :"
	result := collection   copyAfterLast: (element ).
	
	"verifying content: "
	self assert: result isEmpty.

! !

!TCopyPartOfSequenceableForMultipliness methodsFor: 'tests - copying part of sequenceable for multipliness' stamp: 'delaunay 4/22/2009 11:03'!
testCopyAfterWithDuplicate
	| result element  collection |
	collection := self collectionWithSameAtEndAndBegining .
	element := collection  last.
	
	" collectionWithSameAtEndAndBegining first and last elements are equals.
	'copyAfter:' should copy after the first occurence :"
	result := collection   copyAfter: (element ).
	
	"verifying content: "
	1 to: result size do: 
		[:i |
		self assert: (collection  at:(i + 1 )) = (result at: (i))
		].

	"verify size: "
	self assert: result size = (collection size - 1).! !

!TCopyPartOfSequenceableForMultipliness methodsFor: 'tests - copying part of sequenceable for multipliness' stamp: 'delaunay 4/22/2009 11:14'!
testCopyUpToLastWithDuplicate
	| result element  collection |
	collection := self collectionWithSameAtEndAndBegining .
	element := collection  first.
	
	" collectionWithSameAtEndAndBegining first and last elements are equals.
	'copyUpToLast:' should copy until the last occurence :"
	result := collection   copyUpToLast: (element ).
	
	"verifying content: "
	1 to: result size do:
		[:i |
		self assert: (result at: i ) = ( collection at: i )  
		].
	
	self assert: result size = (collection size - 1).

! !

!TCopyPartOfSequenceableForMultipliness methodsFor: 'tests - copying part of sequenceable for multipliness' stamp: 'delaunay 4/22/2009 11:08'!
testCopyUpToWithDuplicate
	| result element  collection |
	collection := self collectionWithSameAtEndAndBegining .
	element := collection  last.
	
	" collectionWithSameAtEndAndBegining first and last elements are equals.
	'copyUpTo:' should copy until the first occurence :"
	result := collection   copyUpTo: (element ).
	
	"verifying content: "
	self assert: result isEmpty.

! !

Trait named: #TCopyPreservingIdentityTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopyPreservingIdentityTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:08'!
nonEmpty
	self explicitRequirement! !

!TCopyPreservingIdentityTest methodsFor: 'tests - copy' stamp: 'stephane.ducasse 11/21/2008 15:46'!
testCopyReturnsIdentity
	"self debug: #testCopyReturnsIdentity"
	
	| copy | 
	copy := self nonEmpty copy.
	self assert: self nonEmpty == copy.
	! !

Trait named: #TCopySequenceableSameContents
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopySequenceableSameContents methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 14:44'!
empty
	self explicitRequirement! !

!TCopySequenceableSameContents methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:36'!
integerCollection
" return a collection only including SmallInteger elements"
	self explicitRequirement! !

!TCopySequenceableSameContents methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 14:44'!
nonEmpty
	self explicitRequirement! !

!TCopySequenceableSameContents methodsFor: 'tests - fixture' stamp: 'delaunay 4/16/2009 14:47'!
test0FixtureCopySameContentsTest

	self shouldnt: [self nonEmpty ] raise: Error.
	self deny: self nonEmpty isEmpty.
	
	self shouldnt: [self empty  ] raise: Error.
	self assert: self empty isEmpty.
	
	self shouldnt: [self integerCollection  ] raise: Error.
	self integerCollection do: [:each | self assert: each class=SmallInteger].! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/20/2009 14:38'!
testReverse
	| result |
	result := self nonEmpty reverse .
	
	"verify content of 'result: '"
	1 to: result size do:
		[:i | self assert: ((result at:i)=(self nonEmpty at:(self nonEmpty size-i+1)))].
	"verify size of 'result' :"
	self assert: result size=self nonEmpty size.! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/21/2009 16:10'!
testReversed
	| result |
	result := self nonEmpty reversed .
	
	"verify content of 'result: '"
	1 to:  result size do:
		[:i | self assert: ((result at:i)=(self nonEmpty at:(self nonEmpty size-i+1)))].
	"verify size of 'result' :"
	self assert: result size=self nonEmpty size.! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/16/2009 14:40'!
testShallowCopy
	| result |
	result := self nonEmpty shallowCopy .
	
	"verify content of 'result: '"
	1 to: self nonEmpty size do:
		[:i | self assert: ((result at:i)=(self nonEmpty at:i))].
	"verify size of 'result' :"
	self assert: result size=self nonEmpty size.! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/16/2009 14:39'!
testShallowCopyEmpty
	| result |
	result := self empty shallowCopy .
	self assert: result isEmpty .! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/16/2009 14:40'!
testShuffled
	| result |
	result := self nonEmpty shuffled .
	
	"verify content of 'result: '"
	result do: [:each | self assert: (self nonEmpty occurrencesOf: each)=(result occurrencesOf: each)].
	"verify size of 'result' :"
	self assert: result size=self nonEmpty size.! !

!TCopySequenceableSameContents methodsFor: 'tests - copying same contents' stamp: 'delaunay 4/17/2009 15:10'!
testSortBy

	| result tmp |
	result := self integerCollection sortBy: [:a :b | a <b]  .
	
	"verify content of 'result' : "
	result do: [: each | (self integerCollection occurrencesOf: each )= (result occurrencesOf: each)].
	tmp:=result first. 
	result do: [:each | self assert: each>=tmp. tmp:=each].

	"verify size of 'result' :"
	self assert: result size = (self integerCollection size ).! !

Trait named: #TCopySequenceableWithOrWithoutSpecificElements
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:43'!
indexInNonEmpty
" return an index between bounds of 'nonEmpty' "

	self explicitRequirement! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 16:31'!
nonEmpty
	self explicitRequirement! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - fixture' stamp: 'delaunay 4/16/2009 16:34'!
test0FixtureCopyWithOrWithoutSpecificElementsTest

	self shouldnt: [self nonEmpty ] raise: Error.
	self deny: self nonEmpty 	isEmpty .
		
	self shouldnt: [self indexInNonEmpty ] raise: Error.
	self assert: self indexInNonEmpty > 0 & self indexInNonEmpty < self nonEmpty size.! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:56'!
testCopyWithFirst

	| index element result |
	index:= self indexInNonEmpty .
	element:= self nonEmpty at: index.
	
	result := self nonEmpty copyWithFirst: element.	
	
	self assert: result size = (self nonEmpty size + 1).
	self assert: result first = element .
	
	2 to: result size do:
	[ :i |
	self assert: (result at: i) = ( self nonEmpty at: ( i - 1 ))].! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:57'!
testCopyWithSequenceable

	| result index element |
	index := self indexInNonEmpty .
	element := self nonEmpty at: index.
	result := self nonEmpty copyWith: (element ).
	
	self assert: result size = (self nonEmpty size + 1).
	self assert: result last = element .
	
	1 to: (result size - 1) do:
	[ :i |
	self assert: (result at: i) = ( self nonEmpty at: ( i  ))].! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:49'!
testCopyWithoutFirst

	| result |
	result := self nonEmpty copyWithoutFirst.
	
	self assert: result size = (self nonEmpty size - 1).
	
	1 to: result size do:
		[:i |
		self assert: (result at: i)= (self nonEmpty at: (i + 1))].! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:50'!
testCopyWithoutIndex
	| result index |
	index := self indexInNonEmpty .
	result := self nonEmpty copyWithoutIndex: index .
	
	"verify content of 'result:'"
	1 to: result size do:
		[:i | 
		i<( index ) ifTrue: [self assert: ((result at:i )= (self nonEmpty at:i))].
		i>=( index ) ifTrue: [self assert: (result at:i )= (self nonEmpty at:(i+1))]].
	
	"verify size of result : "
	self assert: result size=(self nonEmpty size -1).! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:53'!
testForceToPaddingStartWith

	| result element |
	element := self nonEmpty at: self indexInNonEmpty .
	result := self nonEmpty forceTo: (self nonEmpty size+2) paddingStartWith: ( element ).
	
	"verify content of 'result' : "
	1 to: 2   do:
		[:i | self assert: ( element ) = ( result at:(i) ) ].
	
	3 to: result size do:
		[:i | self assert: ( result at:i ) = ( self nonEmpty at:(i-2) ) ].

	"verify size of 'result' :"
	self assert: result size = (self nonEmpty size + 2).! !

!TCopySequenceableWithOrWithoutSpecificElements methodsFor: 'tests - copying with or without' stamp: 'delaunay 4/16/2009 16:54'!
testForceToPaddingWith

	| result element |
	element := self nonEmpty at: self indexInNonEmpty .
	result := self nonEmpty forceTo: (self nonEmpty size+2) paddingWith: ( element ).
	
	"verify content of 'result' : "
	1 to: self nonEmpty  size do:
		[:i | self assert: ( self nonEmpty at: i ) = ( result at:(i) ). ].
	
	(result size - 1) to: result size do:
		[:i | self assert: ( result at:i ) = ( element ) ].

	"verify size of 'result' :"
	self assert: result size = (self nonEmpty size + 2).! !

Trait named: #TCopySequenceableWithReplacement
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopySequenceableWithReplacement methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:51'!
collectionWith1TimeSubcollection
" return a collection including 'oldSubCollection'  only one time "
	self explicitRequirement! !

!TCopySequenceableWithReplacement methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:51'!
collectionWith2TimeSubcollection
" return a collection including 'oldSubCollection'  two or many time "
	self explicitRequirement! !

!TCopySequenceableWithReplacement methodsFor: 'tests - copying with replacement' stamp: 'delaunay 4/17/2009 10:43'!
firstIndexesOf: subCollection in: collection
" return an OrderedCollection with the first indexes of the occurrences of subCollection in  collection "
	| tmp result currentIndex |
	tmp:= collection.
	result:= OrderedCollection new.
	currentIndex := 1.
	
	[tmp isEmpty ]whileFalse:
		[
		(tmp beginsWith: subCollection)
			ifTrue: [ 	
				result add: currentIndex.
				1 to: subCollection size do: 
					[:i | 
					tmp := tmp copyWithoutFirst.
					currentIndex := currentIndex + 1]
				]
			ifFalse: [
				tmp := tmp copyWithoutFirst.
				currentIndex := currentIndex +1.
				]
		 ].
	
	^ result.
	! !

!TCopySequenceableWithReplacement methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 10:30'!
howMany: subCollection in: collection
" return an integer representing how many time 'subCollection'  appears in 'collection'  "
	| tmp nTime |
	tmp:= collection.
	nTime:= 0.
	
	[tmp isEmpty ]whileFalse:
		[
		(tmp beginsWith: subCollection)
			ifTrue: [ 	
				nTime := nTime + 1.
				1 to: subCollection size do: [:i | tmp := tmp copyWithoutFirst.]
				]
			ifFalse: [tmp := tmp copyWithoutFirst.]
		 ].
	
	^ nTime.
	! !

!TCopySequenceableWithReplacement methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:52'!
oldSubCollection
" return a subCollection included in collectionWith1TimeSubcollection .
ex :   subCollection := #( 2 3 4) and collectionWith1TimeSubcollection := #(1 2 3 4 5)"
	self explicitRequirement! !

!TCopySequenceableWithReplacement methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:52'!
replacementCollection
" return a collection that will be used to replace 'oldSubcollection' in ' collectionWith1TimeSubcollection'  " 
	self explicitRequirement! !

!TCopySequenceableWithReplacement methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 14:58'!
test0FixtureCopyWithReplacementTest

	self shouldnt: [self replacementCollection   ]raise: Error.
	self shouldnt: [self oldSubCollection]  raise: Error.
	
	self shouldnt: [self collectionWith1TimeSubcollection ]raise: Error.
	self assert: (self howMany: self oldSubCollection  in: self collectionWith1TimeSubcollection  ) = 1.
	
	! !

!TCopySequenceableWithReplacement methodsFor: 'tests - copying with replacement' stamp: 'delaunay 4/17/2009 11:19'!
testCopyReplaceAllWith1Occurence
	| result  firstIndexesOfOccurrence index endPartIndexResult endPartIndexCollection |
	
	result := self collectionWith1TimeSubcollection  copyReplaceAll: self oldSubCollection with: self replacementCollection .
	
	"detecting indexes of olSubCollection"
	firstIndexesOfOccurrence  := self firstIndexesOf: self oldSubCollection in: self collectionWith1TimeSubcollection .
	index:= firstIndexesOfOccurrence at: 1.
	
	"verify content of 'result' : "
	"first part of 'result'' : '"

	1 to: (index -1) do: 
		[
		:i |  
		self assert: (self collectionWith1TimeSubcollection  at:i)=(result at: i)
		].

	" middle part containing replacementCollection : "
	
	index to: (index + self replacementCollection size-1) do: 
		[
		:i |
		self assert: ( result at: i )=(self replacementCollection at: ( i - index + 1 ))
		].
	
	" end part :"
	
	endPartIndexResult :=  index + self replacementCollection  size .
	endPartIndexCollection :=   index + self oldSubCollection size  .
	
	1 to: (result size - endPartIndexResult - 1 ) do:
		[ 
		:i |
		self assert: (result at: ( endPartIndexResult + i - 1 ) ) = (self collectionWith1TimeSubcollection  at: ( endPartIndexCollection + i - 1 ) ).
		].
	
	
	! !

!TCopySequenceableWithReplacement methodsFor: 'tests - copying with replacement' stamp: 'delaunay 4/20/2009 14:27'!
testCopyReplaceAllWithManyOccurence
	| result  firstIndexesOfOccurrence resultBetweenPartIndex collectionBetweenPartIndex |
	" testing fixture here as this method may be not used for collection that can't contain equals element :"
	self shouldnt: [self collectionWith2TimeSubcollection ]raise: Error.
	self assert: (self howMany: self oldSubCollection  in: self collectionWith2TimeSubcollection  ) = 2.
	
	" test :"
	result := self collectionWith2TimeSubcollection  copyReplaceAll: self oldSubCollection with: self replacementCollection .
	
	"detecting indexes of olSubCollection"
	firstIndexesOfOccurrence  := self firstIndexesOf: self oldSubCollection in: self collectionWith2TimeSubcollection .
	
	" verifying that replacementCollection has been put in places of oldSubCollections "
	firstIndexesOfOccurrence do: [
		:each |
		each to: self replacementCollection size do:
			[ :i | self assert: (result at: i) = ( self replacementCollection at: ( i - each + 1 ) ) ].
		].
	
	" verifying that the 'between' parts correspond to the initial collection : "
	1 to: firstIndexesOfOccurrence size do: [
		:i |
		i = 1 
			" specific comportement for the begining of the collection :"
			ifTrue: [ 
				1 to: ((firstIndexesOfOccurrence at: i) - 1 )  do: 
					[ :j | 
					self assert: (result at: i) = (self collectionWith2TimeSubcollection at: i)  ]  
				]
			" between parts till the end : "
			ifFalse: [
				resultBetweenPartIndex := (firstIndexesOfOccurrence at: (i -1)) + self replacementCollection size.
				collectionBetweenPartIndex := (firstIndexesOfOccurrence at: (i -1)) + self oldSubCollection  size.
				
				1 to: ( firstIndexesOfOccurrence at: i) - collectionBetweenPartIndex - 1  do:
					[ :j |
					self assert: (result at: (resultBetweenPartIndex + i - 1)) = (self collectionWith2TimeSubcollection  at: (collectionBetweenPartIndex +i - 1))  ]	
				]
	].

	"final part :"
	1 to:  (self collectionWith2TimeSubcollection size - (firstIndexesOfOccurrence last + self oldSubCollection size ) ) do:
		[
		:i |
		self assert: ( result at:(firstIndexesOfOccurrence last + self replacementCollection  size -1) + i ) = ( self collectionWith2TimeSubcollection at:(firstIndexesOfOccurrence last + self oldSubCollection size -1) + i ) .
		]! !

!TCopySequenceableWithReplacement methodsFor: 'tests - copying with replacement' stamp: 'delaunay 4/17/2009 13:48'!
testCopyReplaceFromToWith
	| result  indexOfSubcollection lastIndexOfOldSubcollection lastIndexOfReplacementCollection |
	
	indexOfSubcollection := (self firstIndexesOf: self oldSubCollection  in:  self collectionWith1TimeSubcollection) at: 1. 
	lastIndexOfOldSubcollection := indexOfSubcollection + self oldSubCollection size -1.
	lastIndexOfReplacementCollection := indexOfSubcollection + self replacementCollection  size -1.
	
	result := self collectionWith1TimeSubcollection  copyReplaceFrom: indexOfSubcollection  to: lastIndexOfOldSubcollection   with: self replacementCollection .
	
	"verify content of 'result' : "
	"first part of 'result'  "
	
	1 to: (indexOfSubcollection  - 1) do: 
		[ 
		:i | 
		self assert: (self collectionWith1TimeSubcollection  at:i) = (result at: i)
		].
	
	" middle part containing replacementCollection : "
	
	(indexOfSubcollection ) to: ( lastIndexOfReplacementCollection  ) do: 
		[
		:i |
		self assert: (result at: i)=(self replacementCollection at: (i - indexOfSubcollection +1))
		].
	
	" end part :"
	1 to: (result size - lastIndexOfReplacementCollection   ) do:
		[ 
		:i |
		self assert: (result at: ( lastIndexOfReplacementCollection  + i  ) ) = (self collectionWith1TimeSubcollection  at: ( lastIndexOfOldSubcollection  + i  ) ).
		].
	
	
	

	
	! !

!TCopySequenceableWithReplacement methodsFor: 'tests - copying with replacement' stamp: 'delaunay 4/17/2009 13:57'!
testCopyReplaceFromToWithInsertion
	| result  indexOfSubcollection |
	
	indexOfSubcollection := (self firstIndexesOf: self oldSubCollection  in:  self collectionWith1TimeSubcollection) at: 1. 
	
	result := self collectionWith1TimeSubcollection  copyReplaceFrom: indexOfSubcollection  to: ( indexOfSubcollection - 1 ) with: self replacementCollection .
	
	"verify content of 'result' : "
	"first part of 'result'' : '"
	
	1 to: (indexOfSubcollection -1) do: 
		[
		:i | 
		self assert: (self collectionWith1TimeSubcollection  at:i)=(result at: i)
		].
	
	" middle part containing replacementCollection : "
	indexOfSubcollection  to: (indexOfSubcollection  + self replacementCollection size-1) do: 
		[
		:i |
		self assert: ( result at: i )=(self replacementCollection at: ( i - indexOfSubcollection +1 ))
		].
	
	" end part :"
	(indexOfSubcollection  + self replacementCollection size) to: (result size) do:
		[:i|
		self assert: (result at: i)=(self collectionWith1TimeSubcollection  at: (i-self replacementCollection size))].
	
	" verify size: "	
	self assert: result size=(self collectionWith1TimeSubcollection  size + self replacementCollection size).
	

	

	
	! !

Trait named: #TCopyTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCopyTest commentStamp: 'stephane.ducasse 1/16/2009 19:07' prior: 0!
Tests whether a copied object contains the same elements than its source.
Note that the order is not garanteed -- see TEqualityTest for that. 

!

!TCopyTest methodsFor: 'requirements' stamp: 'delaunay 3/30/2009 10:37'!
collectionNotIncluded
	^ self explicitRequirement! !

!TCopyTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:21'!
collectionWithElementsToRemove
" return a collection of elements included in 'nonEmpty'  "
	^ self explicitRequirement! !

!TCopyTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ self explicitRequirement! !

!TCopyTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:08'!
empty
	^ self explicitRequirement! !

!TCopyTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: [ self collectionWithElementsToRemove ]
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: [ self elementToAdd ]
		raise: Error.
	self 
		shouldnt: [ self collectionNotIncluded ]
		raise: Error.
	self collectionNotIncluded do: [ :each | self deny: (self nonEmpty includes: each) ]! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'stephane.ducasse 11/21/2008 17:17'!
testCopyNonEmptyWithout
	"self debug: #testCopyNonEmptyWithout"
	
	| res anElementOfTheCollection |
	anElementOfTheCollection :=  self nonEmpty anyOne.
	res := (self nonEmpty copyWithout: anElementOfTheCollection).
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self deny: (res includes: anElementOfTheCollection).
	self nonEmpty do:
		[:each | (each = anElementOfTheCollection) 
					ifFalse: [self assert: (res includes: each)]].
	
! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:37'!
testCopyNonEmptyWithoutAllNotIncluded
	"self debug: #testCopyNonEmptyWithoutAllNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self collectionNotIncluded.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!TCopyTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

Trait named: #TCreationWithTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 18:24'!
collectionClass 
	
	^ self explicitRequirement! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:18'!
testOfSize
	"self debug: #testOfSize"
	
	| aCol |
	aCol := self collectionClass ofSize: 3.
	self assert: (aCol size = 3).
! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 18:23'!
testWith
	"self debug: #testWith"
	
	| aCol |
	aCol := self collectionClass with: $a.
	self assert: (aCol includes: $a).! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:15'!
testWithAll
	"self debug: #testWithAll"
	
	| aCol |
	aCol := self collectionClass withAll: #($a $b $c).
	self assert: (aCol includes: $a).
	self assert: (aCol includes: $b).
	self assert: (aCol includes: $c).
	self assert: (aCol size = 3)! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:14'!
testWithWith
	"self debug: #testWithWith"
	
	| aCol |
	aCol := self collectionClass with: $a with: $b.
	self assert: (aCol includes: $a).
	self assert: (aCol includes: $b).! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:16'!
testWithWithWith
	"self debug: #testWithWithWith"
	
	| aCol |
	aCol := self collectionClass with: $a with: $b with: $c.
	self assert: (aCol includes: $a).
	self assert: (aCol includes: $b).
	self assert: (aCol includes: $c).! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:15'!
testWithWithWithWith
	"self debug: #testWithWithWithWith"
	
	| aCol |
	aCol := self collectionClass with: $a with: $b with: $c with: $d.
	self assert: (aCol includes: $a).
	self assert: (aCol includes: $b).
	self assert: (aCol includes: $c).
	self assert: (aCol includes: $d).! !

!TCreationWithTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:15'!
testWithWithWithWithWith
	"self debug: #testWithWithWithWithWith"
	
	| aCol |
	aCol := self collectionClass with: $a with: $b with: $c with: $d with: $e.
	self assert: (aCol includes: $a).
	self assert: (aCol includes: $b).
	self assert: (aCol includes: $c).
	self assert: (aCol includes: $d).
	self assert: (aCol includes: $e).! !

Trait named: #TDictionaryAccessingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryAccessingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:45'!
emptyDict
	self explicitRequirement! !

!TDictionaryAccessingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:46'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryAccessingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:46'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:44'!
testAddAssociation
	"self run:#testAddAssociation"
	"self debug:#testAddAssociation"
	
	| dict |
	dict := self newEmptyDict.
	dict at: #a put: 1.
	dict at: #b put: 2.
	self assert: (dict at: #a) = 1.
	self assert: (dict at: #b) = 2.
	
	dict at: #a put: 10.
	dict at: #c put: 2.
	
	self assert: (dict at: #a) = 10.
	self assert: (dict at: #b) = 2.
	self assert: (dict at: #c) = 2
	
	! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'damienpollet 1/30/2009 17:46'!
testAssociationDeclareAt
	"This test is related to ImageSegment. See implementor of associationDeclareAt:"
	| t |
	t := false.
	self nonEmptyDict associationAt: #zork ifAbsent: [t := true].
	self assert: t.

	"Create a new association #zork->false" 
	self nonEmptyDict associationDeclareAt: #zork.

	t := true.
	self nonEmptyDict associationAt: #zork ifAbsent: [t := false].
	self assert: t.
	
	self deny: (self nonEmptyDict at: #zork).
	
	self assert: (self nonEmptyDict associationDeclareAt: #c) = (#c -> self elementTwiceIn)! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'damienpollet 1/30/2009 17:52'!
testAt
	
	self assert: (self nonEmptyDict at: #a) = self elementTwiceIn.
	self assert: (self nonEmptyDict at: #b) = 30! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:46'!
testAtError
	"self run: #testAtError"
	
	| dict |
	dict := self emptyDict.
	dict at: #a put: 666.
	self shouldnt: [ dict at: #a ] raise: Error.
	self should: [ dict at: #b ] raise: Error.
	
	! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:47'!
testAtIfAbsent
	"self run: #testAtIfAbsent"
	
	| dict |
	dict := self emptyDict.
	dict at: #a put: 666.
	
	self assert: (dict at: #a ifAbsent: [nil]) = 666.
	
	self assert: (dict at: #b ifAbsent: [nil]) isNil.

	! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'damienpollet 1/30/2009 17:53'!
testAtIfPresent
	"self run: #testAtIfAbsent"
	
	| t |
	t := false.
	self nonEmptyDict at: #b ifPresent: [:x | t := (x = 30)].
	self assert: t.
	
	self assert: (self nonEmptyDict at: #a ifPresent: [:x | 'ABCDEF']) =  'ABCDEF'.

	self assert: (self nonEmptyDict at: #absent ifPresent: [:x | Error signal]) isNil

	! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'stephane.ducasse 1/12/2009 17:20'!
testAtPutDict
	"self run: #testAtPutDict"
	"self debug: #testAtPutDict"
	
	| adictionary |
	adictionary := self emptyDict.
	adictionary at: #a put: 3.
	self assert: (adictionary at: #a) = 3.
	adictionary at: #a put: 3.
	adictionary at: #a put: 4.
	self assert: (adictionary at: #a) = 4.
	adictionary at: nil put: 666.
	self assert: (adictionary at: nil) = 666! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:47'!
testAtPutNil
	"self run: #testAtPut"
	"self debug: #testAtPut"
	
	| dict |
	dict := self emptyDict.
	dict at: nil put: 1.
	self assert: (dict at: nil) = 1.
	dict at: #a put: nil.
	self assert: (dict at: #a) isNil.
	dict at: nil put: nil.
	self assert: (dict at: nil) isNil.
	
	
	! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'damienpollet 1/30/2009 18:00'!
testKeyAtIdentityValue
	self assert: (self nonEmptyDict keyAtIdentityValue: self element) == #b.
	self assert: (self nonEmptyDict keyAtValue: self element) == #b.

	self assert: (self nonEmptyDict keyAtIdentityValue: self element ifAbsent: [Error signal]) == #b.
	self assert: (self nonEmptyDict keyAtValue: self element ifAbsent: [Error signal]) == #b.

	self should: [self nonEmptyDict keyAtIdentityValue: 1000 ifAbsent: [Error signal]] raise: Error.
	self should: [self nonEmptyDict keyAtValue: 1000 ifAbsent: [Error signal]] raise: Error.
! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:47'!
testKeyAtValue
	"self run: #testKeyAtValue"
	"self debug: #testKeyAtValue"
	
	| dict |
	dict := self emptyDict.
	dict at: #a put: 1.
	dict at: #b put: 2.
	dict at: #c put: 1.
	
	self assert: (dict keyAtValue: 2) = #b.
	self assert: (dict keyAtValue: 1) = #c.
	"ugly may be a bug, why not having a set #a and #c"
	
	self should: [dict keyAtValue: 0] raise: Error
	
	
! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:47'!
testKeys
	self assert: self nonEmptyDict keys asSortedCollection asArray = #(a b c d)! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'AlexandreBergel 1/7/2009 11:47'!
testKeysSortedSafely
	self assert: self nonEmptyDict keysSortedSafely asArray = #(a b c d)! !

!TDictionaryAccessingTest methodsFor: 'test - accessing' stamp: 'damienpollet 1/30/2009 17:59'!
testValues
	self assert: self nonEmptyDict values asSortedCollection asArray =  #(-2 1 1 30)! !

Trait named: #TDictionaryAddingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryAddingTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 14:32'!
emptyDict
	self explicitRequirement! !

!TDictionaryAddingTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 15:08'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryAddingTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 14:32'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryAddingTest methodsFor: 'test - adding' stamp: 'AlexandreBergel 1/6/2009 15:05'!
testAdd
	
	| dict |
	dict := self emptyDict.
	dict add: #a -> 1.
	dict add: #b -> 2.
	self assert: (dict at: #a) = 1.
	self assert: (dict at: #b) = 2! !

!TDictionaryAddingTest methodsFor: 'test - adding' stamp: 'AlexandreBergel 1/6/2009 15:07'!
testAddAll
	
	| dict1 dict2 |
	dict1 := self emptyDict copy.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict2 := self emptyDict copy.
	dict2 at: #a put: 3 ; at: #c put: 4.
	dict1 addAll: dict2.
	self assert: (dict1 at: #a) = 3.
	self assert: (dict1 at: #b) = 2.
	self assert: (dict1 at: #c) = 4.! !

!TDictionaryAddingTest methodsFor: 'test - adding' stamp: 'damienpollet 1/30/2009 17:46'!
testAssociationAt
	| t |
	t := false.
	self nonEmptyDict associationAt: #zork ifAbsent: [t := true].
	self assert: t.
	
	self assert: (self nonEmptyDict associationAt: #c) = (#c -> self elementTwiceIn).
	
	self shouldnt: [self nonEmptyDict associationAt: #c ifAbsent: [Error signal]] raise: Error.
	self assert: (self nonEmptyDict associationAt: #c ifAbsent: [Error signal]) = (#c -> self elementTwiceIn)! !

!TDictionaryAddingTest methodsFor: 'test - adding' stamp: 'AlexandreBergel 1/6/2009 15:09'!
testDeclareFrom
	| newDict v |
	newDict := self newEmptyDict at: #zork put: 'abc'; yourself.
	
	"if the key already exist, nothing changes"
	v := self nonEmptyDict at: #a.
	self nonEmptyDict declare: #a from: newDict.
	self assert: (self nonEmptyDict at: #a) = v.
	
	"if the key does not exist, then it gets removed from newDict and is added to the receiver"
	self nonEmptyDict declare: #zork from: newDict.
	self assert: (self nonEmptyDict at: #zork) = 'abc'.
	self assert: (newDict size = 0)! !

Trait named: #TDictionaryComparingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryComparingTest methodsFor: 'test - comparing' stamp: 'damienpollet 1/30/2009 17:54'!
testEquality
	| nonEmptyDict2 |
	nonEmptyDict2 := self emptyDict.
	nonEmptyDict2
		at: #a put: 1;
		at: #b put: 30; 
		at: #c put: 1;
		at: #d put: -2.
	self assert: (self nonEmptyDict = nonEmptyDict2)! !

Trait named: #TDictionaryCopyingTest
	uses: TCloneTest
	category: 'CollectionsTests-Unordered'!

TDictionaryCopyingTest classTrait
	uses: TCloneTest classTrait!

!TDictionaryCopyingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:55'!
emptyDict
	self explicitRequirement! !

!TDictionaryCopyingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:55'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryCopyingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:55'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryCopyingTest methodsFor: 'test - copying' stamp: 'adrian_lienhard 2/21/2009 13:45'!
testComma
	| dict1 dict2 dict3 |
	dict1 := self emptyDict.
	dict1 at: #a put:1 ; at: #b put: 2. 
	dict2 := self emptyDict.
	dict2 at: #a put: 3 ; at: #c put: 4.
	dict3 := dict1, dict2.
	self assert: (dict3 at: #a) = 3.
	self assert: (dict3 at: #b) = 2.
	self assert: (dict3 at: #c) = 4.! !

!TDictionaryCopyingTest methodsFor: 'test - copying' stamp: 'adrian_lienhard 2/21/2009 13:46'!
testDictionaryConcatenation
	"self run: #testDictionaryConcatenation"
	
	| dict1 dict2 dict3 |
	dict1 := self emptyDict.
	dict1 at: #a put: 'Nicolas' ; at: #b put: 'Damien'. 
	
	dict2 := self emptyDict.
	dict2 at: #a put: 'Christophe' ; at: #c put: 'Anthony'.
	dict3 := dict1, dict2.
	
	self assert: (dict3 at: #a) = 'Christophe'.
	self assert: (dict3 at: #b) = 'Damien'.
	self assert: (dict3 at: #c) = 'Anthony'.
	

	! !

Trait named: #TDictionaryEnumeratingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryEnumeratingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:58'!
emptyDict
	self explicitRequirement! !

!TDictionaryEnumeratingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:58'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryEnumeratingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 11:58'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testAssociationsDo
	| t |
	t := 0.
	self nonEmptyDict associationsDo: [:assoc | t := t + assoc value].
	self assert: t = (self nonEmptyDict values inject: 0 into: [:sum :el | sum + el]).! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'damienpollet 1/30/2009 17:52'!
testAssociationsSelect
	| t assoc |
	t := self nonEmptyDict at: #a.
	assoc := self nonEmptyDict associationsSelect: [:a | a value > t].
	self assert: assoc size = 1.! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testCollect
	| values |
	values := self nonEmptyDict collect: [:assoc | assoc value].
	self assert: values asSortedCollection asArray = self nonEmptyDict values asSortedCollection asArray.! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'damienpollet 1/30/2009 17:54'!
testDo
	| t |
	t := OrderedCollection new.
	self nonEmptyDict do: [:assoc | t add: assoc].
	self assert: t asArray = {30 . 1 . -2 . 1}! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'damienpollet 1/30/2009 17:57'!
testKeysAndValuesDo
	| t |
	t := OrderedCollection new.
	self nonEmptyDict keysAndValuesDo: [:key :value | t add: key -> value].
	self assert: (t asSortedCollection: [:a :b | b key < a key]) asArray = {#d -> -2 . #c -> 1 . #b -> 30 . #a -> 1}! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testKeysDo
	| t |
	t := OrderedCollection new.
	self nonEmptyDict keysDo: [:key | t add: key ].
	self assert: (t asSortedCollection) asArray = {#a . #b . #c . #d}! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testReject
	"Ensure that Dictionary>>reject: answers a dictionary not something else"
	| dict expected result |
	dict := Dictionary newFromPairs:{
		#first.		1.
		#second.	2.
		#third.		3.
		#fourth.	4.
		#fifth.		5.
	}.
	result := dict reject:[:each| each odd].
	expected := Dictionary newFromPairs:{
		#second.	2.
		#fourth.	4.
	}.
	self assert: result = expected.! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testSelect
	| values |
	values := self nonEmptyDict select: [:value | true].
	self assert: values asSortedCollection asArray = self nonEmptyDict values asSortedCollection asArray.! !

!TDictionaryEnumeratingTest methodsFor: 'test - enumerating' stamp: 'AlexandreBergel 1/7/2009 11:58'!
testValuesDo
	| t |
	t := 0.
	self nonEmptyDict valuesDo: [:value | t := t + value ].
	self assert: (t )  = (self nonEmptyDict values inject: 0 into: [:sum :el | sum + el])! !

Trait named: #TDictionaryImplementationTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryImplementationTest methodsFor: 'test - implementation' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testAtNil
	"(self run: #testAtNil)"
	"nil is a valid key in squeak. In VW nil is not a valid key"
	"Ansi 1.9 p, 168
    		5.7.2.5 Message: at: key put: newElement
    		Synopsis
    			Store newElement at key in the receiver. Answer newElement.
    		Definition: <abstractDictionary>
    		If lookup succeeds for key, then newElement replaces the element previously stored at key.
    		Otherwise, the newElement is stored at the new key. In either case, subsequent successful
    		lookups for key will answer newElement.  Answer newElement.

    		The result is undefined if the key is nil.

		This clearly indicates that different smalltalks where doing different assumptions."
	
	
	
	| dict1  |
	dict1 := self emptyDict.
	self shouldnt: [ dict1 at: nil put: #none] raise: Error.
	self assert: (dict1 at: nil) = #none. 
	! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testFindElementOrNil
	"Set>>findElementOrNil: takes an associaiton (when called on a dictionary) an integer, the index of the first
	position at is either equal to the assocation or which is nil"
	| assoc indexForG |
	assoc := #g -> 100.
	self assert: (self nonEmptyDict findElementOrNil: assoc key) = (self nonEmptyDict array indexOf: nil).
	
	indexForG := (#g hash \\ self emptyDict array size) + 1.
	self assert: (self emptyDict findElementOrNil: assoc key) = ((self emptyDict array indexOf: nil) max: indexForG).! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testKeyAt
	self assert: (self nonEmptyDict keyAt: 1) == #b.
	self assert: (self nonEmptyDict keyAt: 2) == #c.
	self assert: (self nonEmptyDict keyAt: 3) == #d.
! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/14/2009 15:15'!
testNew
	| d |
	d := self classToBeTested new: 10.
	self assert: d size = 0.
	
	"Why 14? Mysterious"
	self assert: d capacity = 14! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testPseudo
	"(self run: #testPseudo)"
	"true and false are valid keys"
	
	| dict1  |
	dict1 := self emptyDict.
	self shouldnt: [dict1 at: true put: #true] raise: Error.
	self assert: (dict1 at: true) = #true.
		
	self shouldnt: [dict1 at: false put: #false] raise: Error.
	self assert: (dict1 at: false) = #false.! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testPseudoVariablesAreValidKeys
	"(self run: #testPseudoVariablesAreValidKeys)"
	"true and false are valid keys"
	
	| dict1  |
	dict1 := self emptyDict.
	self shouldnt: [dict1 at: true put: #true] raise: Error.
	self assert: (dict1 at: true) = #true.
		
	self shouldnt: [dict1 at: false put: #false] raise: Error.
	self assert: (dict1 at: false) = #false.! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testScanFor
	"Set>>scanFor: return an integer "
	| assoc indexForG |
	assoc := #g -> 100.
	self assert: (self nonEmptyDict scanFor: assoc) = (self nonEmptyDict array indexOf: nil).

	indexForG := (#g hash \\ self emptyDict array size) + 1.
	self assert: (self emptyDict scanFor: assoc) = ((self emptyDict array indexOf: nil) max: indexForG).
! !

!TDictionaryImplementationTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/7/2009 19:29'!
testSpecies
	self assert: self nonEmptyDict species == self nonEmptyDict class.
	self assert: self emptyDict species == self emptyDict class! !

Trait named: #TDictionaryPrintingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryPrintingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:32'!
emptyDict
	self explicitRequirement! !

!TDictionaryPrintingTest methodsFor: 'test - printing' stamp: 'AlexandreBergel 1/7/2009 19:31'!
flattenOnStream
	"This test is related to Morphic (see the implementer of writeDictionary:"
	| str |
	"I have no idea how to make this work"
	str := PropertyListEncoder new.
	nonEmptyDict flattenOnStream: str.
	self assert: (str contents = '')! !

!TDictionaryPrintingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:32'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryPrintingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:32'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryPrintingTest methodsFor: 'test - printing' stamp: 'damienpollet 1/30/2009 17:57'!
testPrintElementsOn
	| str |
	str := WriteStream on: String new.
	self nonEmptyDict printElementsOn: str.
	self assert: (str contents = '(#a->1 #b->30 #c->1 #d->-2 )')! !

!TDictionaryPrintingTest methodsFor: 'test - printing' stamp: 'damienpollet 1/30/2009 17:59'!
testStoreOn
	| str |
	str := WriteStream on: String new.
	self nonEmptyDict storeOn: str.
	self assert: str contents = '((Dictionary new) add: (#b->30); add: (#c->1); add: (#d->-2); add: (#a->1); yourself)'! !

Trait named: #TDictionaryRemovingTest
	uses: {}
	category: 'CollectionsTests-Unordered'!

!TDictionaryRemovingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:43'!
emptyDict
	self explicitRequirement! !

!TDictionaryRemovingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:43'!
newEmptyDict
	self explicitRequirement! !

!TDictionaryRemovingTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/7/2009 19:43'!
nonEmptyDict
	self explicitRequirement! !

!TDictionaryRemovingTest methodsFor: 'test - removing' stamp: 'damienpollet 1/30/2009 17:58'!
removedValue
	
	^ self element! !

!TDictionaryRemovingTest methodsFor: 'test - removing' stamp: 'AlexandreBergel 1/7/2009 19:41'!
testKeysAndValuesRemove
	| t |
	t := self nonEmptyDict size.
	self nonEmptyDict keysAndValuesRemove: [:key :value | key == #a].
	self assert: (self nonEmptyDict size = (t - 1))! !

!TDictionaryRemovingTest methodsFor: 'test - removing' stamp: 'stephane.ducasse 1/8/2009 14:36'!
testRemove

	self should: [self nonEmptyDict remove: nil] raise: Error.
	self should: [self nonEmptyDict remove: nil ifAbsent: ['What ever here']] raise: Error.! !

!TDictionaryRemovingTest methodsFor: 'test - removing' stamp: 'damienpollet 1/30/2009 17:58'!
testRemoveKey
	"self debug: #testRemoveKey"

	| t |
	t := self nonEmptyDict size.
	self assert: (self nonEmptyDict at: #b) = self removedValue.
	self nonEmptyDict removeKey: #b.
	self assert: (self nonEmptyDict size = (t - 1)).
	self should: [ (self nonEmptyDict at: #b)] raise: Error.
	
	t := false.
	self assert: (self nonEmptyDict at: #b ifAbsent: [t := true. 189]) = 189.
	self assert: t.! !

!TDictionaryRemovingTest methodsFor: 'test - removing' stamp: 'AlexandreBergel 1/7/2009 19:41'!
testRemoveKeyIfAbsent
	| t |
	t := self nonEmptyDict size.
	self nonEmptyDict removeKey: #a ifAbsent: [Error signal].
	self assert: (self nonEmptyDict size = (t - 1)).
	
	t := false.
	self assert: (self nonEmptyDict at: #a ifAbsent: [t := true. 189]) = 189.
	self assert: t.
	
	"======"
	self should: [self nonEmptyDict removeKey: #zork ifAbsent: [Error signal]] raise: Error.
! !

Trait named: #TEmptySequenceableTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TEmptySequenceableTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/5/2008 13:06'!
accessValuePutIn
	"return access the element put in the non-empty collection"
	
	^ self perform: self selectorToAccessValuePutIn! !

!TEmptySequenceableTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/5/2008 13:06'!
accessValuePutInOn: s

	"return access the element put in the non-empty collection"
	
	^ s perform: self selectorToAccessValuePutIn! !

!TEmptySequenceableTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:12'!
empty
	
	^ self explicitRequirement! !

!TEmptySequenceableTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:12'!
nonEmpty
	
	^ self explicitRequirement! !

!TEmptySequenceableTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/5/2008 13:05'!
selectorToAccessValuePutIn
	"return the selector of the method that should be invoked to access an element"
	
	^ self explicitRequirement! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 12:57'!
testSequenceAbleIfEmptyifNotEmptyDo
	"self debug: #testSequenceAbleIfEmptyifNotEmptyDo"
	
	self assert: (self nonEmpty 
					ifEmpty: [false]
					ifNotEmptyDo: [:s | (self accessValuePutInOn: s) = self valuePutIn]).! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 12:59'!
testSequenceIfEmptyifNotEmptyDo
	"self debug #testSequenceIfEmptyifNotEmptyDo"
	
	self assert: (self nonEmpty 
					ifEmpty: [false]
					ifNotEmptyDo: [:s | (self accessValuePutInOn: s) = self valuePutIn]).! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 13:02'!
testSequenceIfNotEmpty

	self assert: (self nonEmpty 
					ifNotEmpty: [:s | self accessValuePutInOn: s]) = self valuePutIn! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 13:03'!
testSequenceIfNotEmptyDo

	self empty ifNotEmptyDo: [:s | self assert: false].
	self assert: (self nonEmpty ifNotEmptyDo: [:s | self accessValuePutInOn: s]) = self valuePutIn
! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 13:04'!
testSequenceIfNotEmptyDoifNotEmpty

	self assert: (self nonEmpty 
					ifNotEmptyDo: [:s | (self accessValuePutInOn: s) = self valuePutIn] 
					ifEmpty: [false])! !

!TEmptySequenceableTest methodsFor: 'tests - sequence isempty' stamp: 'stephane.ducasse 10/5/2008 13:05'!
testSequenceIfNotEmptyifEmpty

	self assert: (self nonEmpty ifEmpty: [false] ifNotEmpty: [:s | (self accessValuePutInOn: s) = self valuePutIn])! !

!TEmptySequenceableTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/5/2008 13:06'!
valuePutIn
	"the value that we will put in the non empty collection"
	
	^ #x! !

Trait named: #TEmptyTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TEmptyTest commentStamp: 'stephane.ducasse 10/5/2008 12:25' prior: 0!
I group a set of tests testing whether a collection is empty.
I do not do any assymption on the elements being added since it would
bind me to know how to access it. 
!

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 2/14/2009 17:25'!
empty
	
	^ self explicitRequirement! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 2/14/2009 17:25'!
nonEmpty
	
	^ self explicitRequirement! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'delaunay 3/27/2009 11:42'!
testIfEmpty

	self nonEmpty ifEmpty: [ self assert: false] .
	self empty ifEmpty: [ self assert: true] .
	

	! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 12:23'!
testIfEmptyifNotEmpty

	self assert: (self empty ifEmpty: [true] ifNotEmpty: [false]).
	self assert: (self nonEmpty ifEmpty: [false] ifNotEmpty: [true]).
	! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:00'!
testIfEmptyifNotEmptyDo
	"self debug #testIfEmptyifNotEmptyDo"
	
	self assert: (self empty ifEmpty: [true] ifNotEmptyDo: [:s | false]).
	self assert: (self nonEmpty ifEmpty: [false] ifNotEmptyDo: [:s | true]).
	self assert: (self nonEmpty 
					ifEmpty: [false]
					ifNotEmptyDo: [:s | s]) == self nonEmpty.! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:03'!
testIfNotEmpty

	self empty ifNotEmpty: [self assert: false].
	self nonEmpty ifNotEmpty: [self assert: true].
	self assert: (self nonEmpty ifNotEmpty: [:s | s ]) = self nonEmpty
	! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:03'!
testIfNotEmptyDo

	self empty ifNotEmptyDo: [:s | self assert: false].
	self assert: (self nonEmpty ifNotEmptyDo: [:s | s]) == self nonEmpty
! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:04'!
testIfNotEmptyDoifNotEmpty

	self assert: (self empty ifNotEmptyDo: [:s | false] ifEmpty: [true]).
	self assert: (self nonEmpty 
					ifNotEmptyDo: [:s | s] 
					ifEmpty: [false]) == self nonEmpty! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'delaunay 4/8/2009 15:26'!
testIfNotEmptyifEmpty

	self assert: (self empty ifNotEmpty: [false] ifEmpty: [true]).
	self assert: (self nonEmpty ifNotEmpty: [true] ifEmpty: [false]).
	! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 11:59'!
testIsEmpty

	self assert: (self empty isEmpty).
	self deny: (self nonEmpty isEmpty).! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 12:00'!
testIsEmptyOrNil

	self assert: (self empty isEmptyOrNil).
	self deny: (self nonEmpty isEmptyOrNil).! !

!TEmptyTest methodsFor: 'tests - empty' stamp: 'delaunay 4/2/2009 11:17'!
testNotEmpty

	self assert: (self nonEmpty  notEmpty).
	self deny: (self empty notEmpty).! !

Trait named: #TGrowableTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TGrowableTest methodsFor: 'tests - growable' stamp: 'stephane.ducasse 1/19/2009 16:18'!
element

	^ self explicitRequirement ! !

!TGrowableTest methodsFor: 'tests - growable' stamp: 'stephane.ducasse 1/19/2009 16:20'!
elementNotIn
	
	^ self explicitRequirement! !

!TGrowableTest methodsFor: 'tests - growable' stamp: 'stephane.ducasse 1/19/2009 16:20'!
empty 

	^ self explicitRequirement! !

!TGrowableTest methodsFor: 'tests - growable' stamp: 'damiencassou 1/20/2009 10:41'!
nonEmpty 

	^ self explicitRequirement! !

!TGrowableTest methodsFor: 'tests - fixture' stamp: 'cyrille.delaunay 3/20/2009 14:24'!
test0FixtureRequirementsOfTGrowableTest
	self shouldnt: [self empty] raise: Exception.
	self shouldnt: [self nonEmpty] raise: Exception.
	self shouldnt: [self element] raise: Exception.
	self shouldnt: [self elementNotIn] raise: Exception.
	self assert: self empty isEmpty.
	self deny: self nonEmpty isEmpty.
	self assert: (self nonEmpty includes: self element).
	self deny: (self nonEmpty includes: self elementNotIn).! !

!TGrowableTest methodsFor: 'tests - growable' stamp: 'cyrille.delaunay 3/20/2009 09:47'!
testAddEmptyGrows
	"self debug: #testAddEmptyGrows"
	
	| oldSize |
	oldSize := self empty size.
	self empty add: self element.
	self assert: (self empty size) = (oldSize + 1).! !

!TGrowableTest methodsFor: 'tests - growable' stamp: 'stephane.ducasse 1/12/2009 17:04'!
testAddNonEmptyGrowsWhenNewElement
	"self debug: #testAddNonEmptyGrowsWhenNewElement"
	
	| oldSize |
	oldSize := self nonEmpty size.
	self deny: (self nonEmpty includes: self elementNotIn).
	self nonEmpty add: self elementNotIn.
	self assert: (self nonEmpty size) > oldSize.! !

Trait named: #TIdentityAddTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIdentityAddTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:22'!
collection
	^ self explicitRequirement! !

!TIdentityAddTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:22'!
element
	^ self explicitRequirement! !

!TIdentityAddTest methodsFor: 'tests - identity adding' stamp: 'damienpollet 1/9/2009 17:47'!
equalNotIdenticalElement
	^ self element copy! !

!TIdentityAddTest methodsFor: 'tests - identity adding' stamp: 'damienpollet 1/9/2009 18:17'!
testIdentityAdd
	| added oldSize |
	oldSize := self collection size.
	self collection add: self element.
	self deny: (self collection includes: self equalNotIdenticalElement).
	
	added := self collection add: self equalNotIdenticalElement.
	self assert: added == self equalNotIdenticalElement.
	self assert: (self collection includes: self equalNotIdenticalElement)! !

Trait named: #TIncludesTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIncludesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:47'!
anotherElementNotIn
" return an element different of 'elementNotIn'  not included in 'nonEmpty' "
	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:50'!
collectionOfFloat
" return a collection included only Float elements '"
	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:49'!
elementInForIncludesTest
" return an element included in nonEmpty "
	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:46'!
elementNotIn
"return an element not included in 'nonEmpty' "

	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'requirements' stamp: 'stephane.ducasse 11/21/2008 14:11'!
empty
	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'requirements' stamp: 'stephane.ducasse 11/21/2008 14:11'!
nonEmpty
	^ self explicitRequirement! !

!TIncludesTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/20/2009 10:53'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
		
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
		
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: (each class= Float) ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:17'!
testIncludesAllOfAllThere
	"self debug: #testIncludesAllOfAllThere'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: { self nonEmpty anyOne }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty).! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'delaunay 4/20/2009 10:52'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self nonEmpty ).
	self deny: (self nonEmpty includesAllOf: { self elementNotIn. self anotherElementNotIn })! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:22'!
testIncludesAnyOfAllThere
	"self debug: #testIncludesAnyOfAllThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self assert: (self nonEmpty includesAnyOf: { self nonEmpty anyOne }).
	self assert: (self nonEmpty includesAnyOf: self nonEmpty).! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:26'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { self elementNotIn. self anotherElementNotIn })! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:05'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	
	self deny: (self nonEmpty includes: self elementNotIn).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotIn)! !

!TIncludesTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:05'!
testIncludesElementIsThere
	"self debug: #testIncludesElementIsThere"
	
	self assert: (self nonEmpty includes: self nonEmpty anyOne).! !

Trait named: #TIndexAccess
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIndexAccess methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 11:12'!
accessCollection
" return a collection of size 5 without equal elements"
	self explicitRequirement! !

!TIndexAccess methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:29'!
collectionOfFloat
" return a collection only includiing Float elements "
	self explicitRequirement! !

!TIndexAccess methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:29'!
elementInCollectionOfFloat
" return an element included in 'collectionOfFloat' "
	self explicitRequirement! !

!TIndexAccess methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 12:01'!
elementInForIndexAccessing
" return an element included in 'accessCollection' "
	self explicitRequirement! !

!TIndexAccess methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:32'!
elementNotInForIndexAccessing
" return an element not included in 'accessCollection' "
	self explicitRequirement! !

!TIndexAccess methodsFor: 'tests - fixture' stamp: 'delaunay 4/21/2009 11:14'!
test0FixtureIndexAccessTest
	| res |
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self deny: self accessCollection isEmpty.
	res := true.
	self accessCollection  
		detect: [ :each | (self accessCollection  occurrencesOf: each) > 1 ]
		ifNone: [ res := false ].
	self assert: res = false.
	
	
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self elementNotInForIndexAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForIndexAccessing).
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each class = Float ].
	self 
		shouldnt: [ self elementInCollectionOfFloat ]
		raise: Error.
	self assert: (self collectionOfFloat includes: self elementInCollectionOfFloat)! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:41'!
testIdentityIndexOf
	"self debug: #testIdentityIndexOf"
	"| collection element |
	element := self elementInCollectionOfFloat copy.
	self deny: self elementInCollectionOfFloat == element.
	collection := self collectionOfFloat copyWith: element.
	self assert: (collection identityIndexOf: element) = collection size"
	
	| collection element |
	element := self accessCollection first.
	collection := self accessCollection . 
	self assert: (collection identityIndexOf: element) = (collection indexOf: element)! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:44'!
testIdentityIndexOfIAbsent
	"self debug: #testIdentityIndexOfIfAbsent"
	"| collection element |
	element := self elementInCollectionOfFloat copy.
	self deny: self elementInCollectionOfFloat == element.
	collection := self collectionOfFloat copyWith: element.
	self assert: (collection 
			identityIndexOf: element
			ifAbsent: [ 0 ]) = collection size.
	self assert: (self collectionOfFloat
			identityIndexOf: element
			ifAbsent: [ 55 ]) = 55"
			
	| collection element |
	element := self accessCollection first.
	collection := self accessCollection .
	self assert: (collection identityIndexOf: element ifAbsent: [ 0 ]) = 1.
	self assert: (collection identityIndexOf: self elementNotInForIndexAccessing ifAbsent: [ 55 ]) = 55! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/10/2009 11:59'!
testIndexOf
	"self debug: #testIndexOf"
	
	"self assert: (self accessCollection indexOf: 2) = 2.
	self assert: (self accessCollection indexOf: 99) = 0"
	| tmp index |
	tmp:=self accessCollection size.
	self accessCollection reverseDo:
		[:each | 
		each =self elementInForIndexAccessing 
			ifTrue:[index:=tmp].
		tmp:=tmp-1].
	
	self assert: (self accessCollection indexOf: self elementInForIndexAccessing )	= index.! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/10/2009 14:27'!
testIndexOfIfAbsent
	"self debug: #testIndexOfIfAbsent"
	
	self assert: (self accessCollection indexOf: (self accessCollection first)  ifAbsent: [33]) = 1.
	self assert: (self accessCollection indexOf: self elementNotInForIndexAccessing  ifAbsent: [33]) = 33.
	! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:15'!
testIndexOfStartingAt
	"self debug: #testLastIndexOf"
	| element |
	element := self accessCollection first.
	self assert: (self accessCollection  indexOf: (element ) startingAt: 2 ifAbsent:[99]) = 99.
	self assert: (self accessCollection  indexOf: (element ) startingAt: 1 ifAbsent:[99]) = 1.
	self assert: (self accessCollection  indexOf: (self elementNotInForIndexAccessing ) startingAt: 1 ifAbsent:[99]) = 99.
! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:33'!
testIndexOfSubCollectionStartingAt
	"self debug: #testIndexOfIfAbsent"
"	| subcollection index |
	subcollection:= self accessCollection copyWithoutFirst.
	index:= self accessCollection indexOfSubCollection: subcollection startingAt:1.
	self assert: index=2.
	
	subcollection:= (self accessCollection copyEmpty )copyWith: self elementNotInForIndexAccessing  .
	index:= self accessCollection indexOfSubCollection: subcollection  startingAt:1.
	self assert: index=0."
	| subcollection index |
	subcollection:= self accessCollection .
	index:= self accessCollection indexOfSubCollection: subcollection startingAt:1.
	self assert: index=1.
	
	index:= self accessCollection indexOfSubCollection: subcollection  startingAt:2.
	self assert: index=0.! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:35'!
testIndexOfSubCollectionStartingAtIfAbsent
	"self debug: #testIndexOfIfAbsent"
	"| index absent subcollection |
	absent:=false. 
	subcollection:= (self accessCollection copyEmpty )copyWith: self elementNotInForIndexAccessing  .
	index:= self accessCollection indexOfSubCollection: subcollection  startingAt:1 ifAbsent: [absent := true].
	self assert: absent=true.
	
	absent:=false.
	subcollection:= self accessCollection copyWithoutFirst.
	index:= self accessCollection indexOfSubCollection: subcollection startingAt:1 ifAbsent: [absent := true].
	self assert: absent=false."
	
	| index absent subcollection |
	absent:=false. 
	subcollection:= (self accessCollection ) .
	index:= self accessCollection indexOfSubCollection: subcollection  startingAt:1 ifAbsent: [absent := true].
	self assert: absent=false.
	
	absent:=false.
	index:= self accessCollection indexOfSubCollection: subcollection startingAt:2 ifAbsent: [absent := true].
	self assert: absent=true.! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:23'!
testLastIndexOf
	"self debug: #testLastIndexOf"
"	| collection |
	
	collection := self accessCollection copyWith: (self accessCollection at: 1).
	self assert: (collection lastIndexOf: (self accessCollection at:1)) = collection size.
	self assert: (collection lastIndexOf: (self elementNotInForIndexAccessing )) = 0."
	| element |
	element  := self accessCollection first.
	self assert: (self accessCollection lastIndexOf: (element)) = 1.
	self assert: (self accessCollection  lastIndexOf: (self elementNotInForIndexAccessing )) = 0.! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:24'!
testLastIndexOfIfAbsent
	"self debug: #testIndexOfIfAbsent"
	
	"| collection |
	collection := self accessCollection copyWith: (self accessCollection at: 1).
	
	self assert: (collection lastIndexOf: (self accessCollection at:1)ifAbsent: [99]) = collection size.
	self assert: ( collection lastIndexOf: (self elementNotInForIndexAccessing )ifAbsent: [99]) = 99."
	
	| element |
	element  := self accessCollection first.
	self assert: (self accessCollection lastIndexOf: (element) ifAbsent: [99]) = 1.
	self assert: (self accessCollection  lastIndexOf: (self elementNotInForIndexAccessing ) ifAbsent: [99]) = 99.
	! !

!TIndexAccess methodsFor: 'tests - index access' stamp: 'delaunay 4/21/2009 11:54'!
testLastIndexOfStartingAt
	"self debug: #testLastIndexOf"
"	| collection |
	collection := (self accessCollection copyWith: (self elementNotInForIndexAccessing ))copyWith: self elementNotInForIndexAccessing .
	self assert: (collection lastIndexOf: (self elementNotInForIndexAccessing ) startingAt: (collection size) ifAbsent:[99]) = collection size.
	self assert: (collection lastIndexOf: (self elementNotInForIndexAccessing ) startingAt: (collection size-1) ifAbsent:[99]) = (collection size-1).
	self assert: (collection lastIndexOf: (self elementNotInForIndexAccessing ) startingAt:(collection size -2) ifAbsent:[99]) = 99."
	| element |
	element := self accessCollection last.
	self assert: (self accessCollection  lastIndexOf: (element ) startingAt: (self accessCollection size) ifAbsent:[99]) = self accessCollection size.
	self assert: (self accessCollection  lastIndexOf: (element) startingAt: (self accessCollection size - 1) ifAbsent:[99]) = 99.
	self assert: (self accessCollection  lastIndexOf: (self elementNotInForIndexAccessing ) startingAt:(self accessCollection size) ifAbsent:[99]) = 99.! !

Trait named: #TIndexAccessForMultipliness
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIndexAccessForMultipliness methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 13:24'!
floatCollectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals only with classic equality (they are not the same object).
(others elements of the collection are not equal to those elements)"
	self explicitRequirement! !

!TIndexAccessForMultipliness methodsFor: 'tests - fixture' stamp: 'delaunay 4/21/2009 13:42'!
test0FixtureIndexAccessFotMultipliness

self shouldnt: [self floatCollectionWithSameAtEndAndBegining ] raise: Error.

self assert: self floatCollectionWithSameAtEndAndBegining first = self floatCollectionWithSameAtEndAndBegining last.
self deny: self floatCollectionWithSameAtEndAndBegining first == self floatCollectionWithSameAtEndAndBegining last.
self assert: self floatCollectionWithSameAtEndAndBegining size > 1.

1 to: self floatCollectionWithSameAtEndAndBegining size do:
	[:i |
	(i > 1 ) & (i < self floatCollectionWithSameAtEndAndBegining size)
		ifTrue: [ self deny: (self floatCollectionWithSameAtEndAndBegining at:i) = (self floatCollectionWithSameAtEndAndBegining first)].
	]! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testIdentityIndexOfDuplicate
	"self debug: #testIdentityIndexOf"
	| collection element |
	
	
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals but are not the same object"
	self assert: (collection identityIndexOf: element) = collection size! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:43'!
testIdentityIndexOfIAbsentDuplicate
	"self debug: #testIdentityIndexOfIfAbsent"
	| collection element elementCopy |
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	elementCopy := element copy.
	
	self deny: self elementInCollectionOfFloat == element.
	
	self assert: (collection  identityIndexOf: element ifAbsent: [ 0 ]) = collection size.
	self assert: (collection  identityIndexOf: elementCopy ifAbsent: [ 55 ]) = 55
			
	! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testIndexOfDuplicate
	"self debug: #testIndexOf"
	
	| collection element |
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'indexOf: should return the position of the first occurrence :'"
	self assert: (collection indexOf: element ) = 1.! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testIndexOfIfAbsentDuplicate
	"self debug: #testIndexOfIfAbsent"
	
	| collection element |
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'indexOf:ifAbsent: should return the position of the first occurrence :'"
	self assert: (collection indexOf: element  ifAbsent: [55]) = 1.
	

	! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testIndexOfStartingAtDuplicate
	"self debug: #testLastIndexOf"
	| collection element |
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'indexOf:ifAbsent:startingAt: should return the position of the first occurrence :'"
	self assert: (collection indexOf: element  startingAt: 1 ifAbsent: [55] ) = 1.
	self assert: (collection indexOf: element  startingAt: 2 ifAbsent: [55] ) = collection size.
	
	! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testLastIndexOfDuplicate
	"self debug: #testLastIndexOf"
	| collection element |
	
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection first.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'lastIndexOf: should return the position of the last occurrence :'"
	self assert: (collection lastIndexOf: (element)) = collection size.

! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:38'!
testLastIndexOfIfAbsentDuplicate
	"self debug: #testIndexOfIfAbsent"
	
	"self debug: #testLastIndexOf"
	| collection element |
	
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection first.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'lastIndexOf: should return the position of the last occurrence :'"
	self assert: (collection lastIndexOf: (element) ifAbsent:[55] ) = collection size.

	! !

!TIndexAccessForMultipliness methodsFor: 'tests - index accessing for multipliness' stamp: 'delaunay 4/21/2009 13:58'!
testLastIndexOfStartingAtDuplicate
	"self debug: #testLastIndexOf"
| collection element |
	collection := self floatCollectionWithSameAtEndAndBegining .
	element := collection last.
	
	" floatCollectionWithSameAtEndAndBegining first and last elements are equals 
	'lastIndexOf:ifAbsent:startingAt: should return the position of the last occurrence :'"
	self assert: (collection lastIndexOf: element  startingAt: collection size ifAbsent: [55] ) = collection size.
	self assert: (collection lastIndexOf: element  startingAt: (collection size -1) ifAbsent: [55] ) = 1.! !

Trait named: #TIterateSequenceableTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIterateSequenceableTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 10:33'!
collection
	self explicitRequirement! !

!TIterateSequenceableTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 10:33'!
firstEven
	"Returns the first even number of #collection"
	self explicitRequirement! !

!TIterateSequenceableTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 10:33'!
firstOdd
	"Returns the first odd number of #collection"
	self explicitRequirement! !

!TIterateSequenceableTest methodsFor: 'tests - fixture' stamp: 'damiencassou 1/27/2009 10:32'!
test0IterateSequenceableTest
	self shouldnt: [self collection] raise: Error.
	self shouldnt: [self collection do: [:each | self assert: each isNumber]] raise: Error.
	self shouldnt: [self firstEven] raise: Error.
	self shouldnt: [self firstOdd] raise: Error.! !

!TIterateSequenceableTest methodsFor: 'tests - iterate sequenceable' stamp: 'damiencassou 1/27/2009 10:32'!
testDetectSequenced

	| res |
	res := self collection detect: [:each | each even].
	self assert: res  = self firstEven.
	res := self collection detect: [:each | each odd].
	self assert: res  = self firstOdd.! !

!TIterateSequenceableTest methodsFor: 'tests - iterate sequenceable' stamp: 'stephane.ducasse 1/16/2009 21:24'!
testDoWithIndex

	self flag: #todo.
	! !

Trait named: #TIterateSequencedReadableTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIterateSequencedReadableTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 14:05'!
empty
	^self explicitRequirement.! !

!TIterateSequencedReadableTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 14:05'!
nonEmpty
	^self explicitRequirement.! !

!TIterateSequencedReadableTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:24'!
nonEmptyMoreThan1Element
" return a collection of integer with more than one element" 
	^self explicitRequirement.! !

!TIterateSequencedReadableTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/2/2009 15:38'!
test0FixtureIterateSequencedReadableTest

	self shouldnt: self nonEmpty raise: Error.
	self deny: self nonEmpty isEmpty.
	
	self shouldnt: self nonEmptyMoreThan1Element  raise: Error.
	self assert: self nonEmptyMoreThan1Element  size > 1.
	self nonEmptyMoreThan1Element do:
		[:each | self assert: each class= SmallInteger].
	
	self shouldnt: self empty raise: Error.
	self assert: self empty isEmpty .! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 10:39'!
testAllButFirstDo
	
	| result |
	result:= OrderedCollection  new.
	
	self nonEmptyMoreThan1Element  allButFirstDo: [:each | result add: each].
	
	1 to: (result size) do:
		[:i|
		self assert: (self nonEmptyMoreThan1Element  at:(i +1))=(result at:i)].
	
	self assert: result size=(self nonEmptyMoreThan1Element  size-1).! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 10:40'!
testAllButLastDo
	
	| result |
	result:= OrderedCollection  new.
	
	self nonEmptyMoreThan1Element  allButLastDo: [:each | result add: each].
	
	1 to: (result size) do:
		[:i|
		self assert: (self nonEmptyMoreThan1Element  at:(i ))=(result at:i)].
	
	self assert: result size=(self nonEmptyMoreThan1Element  size-1).! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/8/2009 10:39'!
testCollectFromTo
	
	| result |
	result:=self nonEmptyMoreThan1Element 
		collect: [ :each | each ]
		from: 1
		to: (self nonEmptyMoreThan1Element size - 1).
		
	1 to: result size
		do: [ :i | self assert: (self nonEmptyMoreThan1Element at: i) = (result at: i) ].
	self assert: result size = (self nonEmptyMoreThan1Element size - 1)! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 13:58'!
testDo! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 14:39'!
testFindFirst

	| element result |
	element := self nonEmpty  at:1.
	 result:=self nonEmpty findFirst: [:each | each =element].
	
	self assert: result=1. ! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:29'!
testFindFirstNotIn

	| result |
	
	 result:=self empty findFirst: [:each | true].
	
	self assert: result=0. ! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 14:24'!
testFindLast

	| element result |
	element := self nonEmpty at:self nonEmpty size.
	 result:=self nonEmpty findLast: [:each | each =element].
	
	self assert: result=self nonEmpty size. ! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:30'!
testFindLastNotIn

	| result |
	
	 result:=self empty findFirst: [:each | true].
	
	self assert: result=0. ! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 14:40'!
testFromToDo
	
	| result |
	result:= OrderedCollection  new.
	
	self nonEmptyMoreThan1Element  from: 1 to: (self nonEmptyMoreThan1Element  size -1) do: [:each | result add: each].
	
	1 to: (self nonEmptyMoreThan1Element  size -1) do:
		[:i|
		self assert: (self nonEmptyMoreThan1Element  at:i )=(result at:i)].
	self assert: result size=(self nonEmptyMoreThan1Element  size-1).! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 14:53'!
testKeysAndValuesDo
	| result |
	result:= OrderedCollection new.
	
	self nonEmptyMoreThan1Element  keysAndValuesDo: 
		[:i :value|
		result add: (value+i)].
	
	1 to: result size do:
		[:i|
		self assert: (result at:i)=((self nonEmptyMoreThan1Element at:i)+i)]! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:35'!
testKeysAndValuesDoEmpty
	| result |
	result:= OrderedCollection new.
	
	self empty  keysAndValuesDo: 
		[:i :value|
		result add: (value+i)].
	
	self assert: result isEmpty .! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 15:19'!
testPairsCollect
	
	| index result |
	index:=1.
	
	result:=self nonEmptyMoreThan1Element  pairsCollect: 
		[:each1 :each2 | 
		each1+each2.
		].
	result do: 
		[:each | self assert: each=(((self nonEmptyMoreThan1Element at:index)+(self nonEmptyMoreThan1Element at:(index+1)))).
		index:=index+2].
	
	self nonEmptyMoreThan1Element size odd
		ifTrue:[self assert: index=self nonEmptyMoreThan1Element size]
		ifFalse:[self assert: index=(self nonEmptyMoreThan1Element size+1)].! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 15:08'!
testPairsDo
	
	| index |
	index:=1.
	
	self nonEmptyMoreThan1Element  pairsDo: 
		[:each1 :each2 | 
		self assert:(self nonEmptyMoreThan1Element at:index)=each1.
		self assert:(self nonEmptyMoreThan1Element at:(index+1))=each2.
		index:=index+2].
	
	self nonEmptyMoreThan1Element size odd
		ifTrue:[self assert: index=self nonEmptyMoreThan1Element size]
		ifFalse:[self assert: index=(self nonEmptyMoreThan1Element size+1)].
	
	
	! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:01'!
testReverseDo
	| result |
	result:= OrderedCollection new.
	self nonEmpty reverseDo: [: each | result add: each].
	
	1 to: result size do:
		[:i|
		self assert: (result at: i)=(self nonEmpty at:(self nonEmpty size-i+1))].! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:36'!
testReverseDoEmpty
	| result |
	result:= OrderedCollection new.
	self empty reverseDo: [: each | result add: each].
	
	self assert: result isEmpty .! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 15:35'!
testReverseWithDo
	
	| secondCollection result |
	result:= OrderedCollection new.
	secondCollection:= self nonEmptyMoreThan1Element  copy.
	
	self nonEmptyMoreThan1Element  reverseWith: secondCollection do:
		[:a :b |
		result add: (a + b)].
	
	1 to: result size do:
		[:i|
		self assert: (result at:i)=((self nonEmptyMoreThan1Element  at:(self nonEmptyMoreThan1Element size-i+1))*2)].
	! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/15/2009 14:51'!
testWithCollect
	
	| result newCollection |
"	copy:= self nonEmptyMoreThan1Element  copy.
	result:=self nonEmptyMoreThan1Element  with: copy collect: [:a :b | a+b].
	
	self assert: (result = (self nonEmptyMoreThan1Element   collect: [:each | each * 2])).
"	
	newCollection := self nonEmptyMoreThan1Element copy.
	"1 to: newCollection size do: [:i | newCollection at: i put: i]."
	result:=self nonEmptyMoreThan1Element  with: newCollection collect: [:a :b | a+b].
	
	1 to: result size do:[: i | self assert: (result at:i)= ((self nonEmptyMoreThan1Element at: i) *2)]! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/6/2009 17:12'!
testWithCollectError
	self should: [self nonEmptyMoreThan1Element with: self empty collect:[:a :b | ]] raise: Error.! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/2/2009 15:14'!
testWithDo
	
	| secondCollection result |
	result:= OrderedCollection new.
	secondCollection:= self nonEmptyMoreThan1Element  copy.
	
	self nonEmptyMoreThan1Element  with: secondCollection do:
		[:a :b |
		result add: (a + b)].
	
	1 to: result size do:
		[:i|
		self assert: (result at:i)=((self nonEmptyMoreThan1Element  at:i)*2)].
	! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 10:36'!
testWithDoError
	
	self should: [self nonEmptyMoreThan1Element with: self empty do:[:a :b | ]] raise: Error.! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/7/2009 14:36'!
testWithIndexCollect
	
	| result |
	
	result:=self nonEmptyMoreThan1Element  withIndexCollect: [:each :i | each+i].
	
	1 to: result size do:[: i | self assert: (result at:i)= ((self nonEmptyMoreThan1Element at: i) + i)]! !

!TIterateSequencedReadableTest methodsFor: 'tests - iterate on sequenced reable collections' stamp: 'delaunay 4/15/2009 14:52'!
testWithIndexDo
	
	| result |
	result:=Array new: self nonEmptyMoreThan1Element size.
	self nonEmptyMoreThan1Element  withIndexDo: [:each :i | result at:i put:(each+i)].
	
	1 to: result size do:[: i | self assert: (result at:i)= ((self nonEmptyMoreThan1Element at: i) + i)]! !

Trait named: #TIterateTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TIterateTest commentStamp: 'stephane.ducasse 10/6/2008 17:49' prior: 0!
I'm testing that high-order iterators are working.
Note however that I do not pay attention to the order of the elements.
Hence, my doTest is designed to not checking that. 
TIterateSequenceableTest is about this distinction and as such will not be applicable to test
Set, Bag, Dictionary. 


!

!TIterateTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/5/2008 15:48'!
collection

	^ self explicitRequirement! !

!TIterateTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 20:26'!
doWithoutNumber

	^ 3! !

!TIterateTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 15:52'!
elementNotIn
	^ self explicitRequirement! !

!TIterateTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:04'!
elementTwiceIn
	^ self explicitRequirement! !

!TIterateTest methodsFor: 'requirements' stamp: 'marcus.denker 2/20/2009 16:29'!
expectedElementByDetect
	"Returns the first even element of #collection"
	^ self explicitRequirement
	
	! !

!TIterateTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 17:15'!
expectedSizeAfterReject
	"Number of even elements in #collection"
	^ self explicitRequirement! !

!TIterateTest methodsFor: 'requirements' stamp: 'damiencassou 1/20/2009 13:40'!
result
	"Returns a collection of the classes of elements in #collection"
	 ^ self explicitRequirement! !

!TIterateTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 19:40'!
selectedNumber

	^ 3
	! !

!TIterateTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 17:38'!
speciesClass
	
	^ self explicitRequirement! !

!TIterateTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/20/2009 16:59'!
test0FixtureTIterateTest
	self shouldnt: [self collection] raise: Error.
	self shouldnt: [self collection do: [:each | self assert: each isNumber]] raise: Error.
	self shouldnt: [self empty] raise: Error.
	self assert: self empty isEmpty.
	self shouldnt: [self result ] raise: Error.
	self result do: [:each | self assert: each = SmallInteger ].
	
	
	self shouldnt: [self expectedSizeAfterReject] raise: Error.
	self shouldnt: [self expectedElementByDetect] raise: Error.
	self deny: (self collection includes: 0).
	self assert: (self collection anySatisfy: [:each | each even])! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 17:17'!
testAllSatisfy

	self assert: ( self collection allSatisfy: [:each | each isNumber] ).
	self deny: ( self collection allSatisfy: [:each | each positive] ).! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 17:28'!
testAllSatisfyEmpty

	self assert: ( self empty allSatisfy: [:each | false]).
	
! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 17:31'!
testAnySastify

	self assert: ( self collection anySatisfy: [:each | each = -2]).
	self deny: (self collection anySatisfy: [:each | each isString]).! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/14/2009 14:06'!
testBasicCollect

	| res |
	res := self collection collect: [:each | each class].
	self assert: res = self result. ! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/14/2009 14:06'!
testBasicCollectEmpty

	| res |
	res := self empty collect: [:each | each class].
	self assert: res isEmpty
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 1/16/2009 18:58'!
testCollectOnEmpty
	self assert: (self empty collect: [:e | self fail]) isEmpty! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 1/16/2009 19:00'!
testCollectThenSelectOnEmpty

	self assert: (self empty collect: [:e | self fail] thenSelect: [:e | self fail]) isEmpty! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'cyrille.delaunay 3/20/2009 11:16'!
testDetect

	| res |
	res := self collection detect: [:each | each even].
	self assert: (res  = self expectedElementByDetect).

	
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'luc.fabresse 11/29/2008 23:23'!
testDetectIfNone

	| res |
	res := self collection detect: [:each | each isZero] ifNone: [100].
	self assert: res  = 100.
	res := self collection detect: [:each | each even] ifNone: [100].
	self assert: res  = self expectedElementByDetect.

	
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/20/2009 09:58'!
testDo2
	"dc: Bad test, it assumes that a new instance of #speciesClass allows addition with #add:. This is not the case of Interval for which species is Array."
	| res |
	"res := self speciesClass new.  
	self collection do: [:each | res add: each class].
	self assert: res = self result. "
	
	res := OrderedCollection new.  
	self collection do: [:each | res add: each class].
	self assert: res size= self collection size.
	res do: [ :each | self assert: each = SmallInteger]. ! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 14:10'!
testDoSeparatedBy
	| string expectedString beforeFirst |
	string := ''.
	self collection 
		do: [ :each | string := string , each asString ]
		separatedBy: [ string := string , '|' ].
		
	expectedString := ''.
	beforeFirst := true.
	self collection do: 
		[ :each | 
		beforeFirst = true 
			ifTrue: [ beforeFirst := false ]
			ifFalse: [ expectedString := expectedString , '|' ].
		expectedString := expectedString , each asString ].
	self assert: expectedString = string! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/20/2009 10:02'!
testDoWithout
	"self debug: #testDoWithout"
	
	"| res element |
	res := OrderedCollection new.  
	element:=self elementTwiceIn .
	self collection do: [:each | res add: each] without: element  .
	self assert: res size = (self collection size - (self collection occurrencesOf: element))."
	
	| res element |
	res := OrderedCollection new.  
	element:=self collection anyOne .
	self collection do: [:each | res add: each class] without: element  .
	self assert: res size = (self collection size - (self collection occurrencesOf: element)).
	res do: [ :each | self assert: each = SmallInteger ].! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/15/2009 13:45'!
testDoWithoutNoDuplicates
	"self debug: #testDoWithoutNoDuplicates"
	| res element |
	res := OrderedCollection new.  
	element:=self collection anyOne.
	self collection do: [:each | res add: each] without:element .
	self assert: res size = (self collection size - (self collection occurrencesOf: element)).! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 11:07'!
testInjectInto
	|result expectedResult|
	result:= self collection 
		inject: 0
		into: [:inj :ele | inj + ele].
		
	expectedResult:=0.
	self collection do: [:each | expectedResult := expectedResult + each ].
	
	self assert: expectedResult = result .! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/6/2009 14:35'!
testNoneSatisfy

	self assert: ( self collection noneSatisfy: [:each | each isNumber not] ).
	self deny: ( self collection noneSatisfy: [:each | each positive] ).! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 17:35'!
testNoneSatisfyEmpty

	self assert: ( self empty noneSatisfy: [:each | false]).
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'luc.fabresse 11/29/2008 23:08'!
testReject

	| res |
	res := self collection reject: [:each | each odd].
	self assert: res size = self expectedSizeAfterReject
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 16:41'!
testRejectEmpty

	| res |
	res := self empty reject: [:each | each odd].
	self assert: res size = self empty size
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'damienpollet 1/29/2009 19:04'!
testRejectNoReject

	| res |
	res := self collection reject: [:each | each = self elementNotIn].
	self assert: res size = self collection size
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 10/6/2008 19:30'!
testSelect

	| res |
	res := self collection select: [:each | each odd].
	self assert: res size = self selectedNumber.
	! !

!TIterateTest methodsFor: 'tests - iterate' stamp: 'stephane.ducasse 1/16/2009 18:57'!
testSelectOnEmpty

	self assert: (self empty select: [:e | self fail]) isEmpty
	
! !

Trait named: #TOccurrencesTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TOccurrencesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:16'!
elementInForOccurrences
	self explicitRequirement! !

!TOccurrencesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:16'!
elementNotInForOccurrences
	self explicitRequirement! !

!TOccurrencesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:16'!
empty
	self explicitRequirement! !

!TOccurrencesTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:16'!
nonEmpty
	self explicitRequirement! !

!TOccurrencesTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/2/2009 11:53'!
test0FixtureOccurrencesTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: self elementInForOccurrences
		raise: Error.
	self assert: (self nonEmpty includes: self elementInForOccurrences).
	self 
		shouldnt: self elementNotInForOccurrences
		raise: Error.
	self deny: (self nonEmpty includes: self elementNotInForOccurrences)! !

!TOccurrencesTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:52'!
testOccurrencesOf
	| result expected |
	result := self nonEmpty occurrencesOf: self elementInForOccurrences.
	expected := 0.
	self nonEmpty do: [ :each | self elementInForOccurrences = each ifTrue: [ expected := expected + 1 ] ].
	self assert: result = expected! !

!TOccurrencesTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:52'!
testOccurrencesOfEmpty
	| result |
	result := self empty occurrencesOf: self elementInForOccurrences.
	self assert: result = 0! !

!TOccurrencesTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:53'!
testOccurrencesOfNotIn
	| result |
	result := self empty occurrencesOf: self elementNotInForOccurrences.
	self assert: result = 0! !

Trait named: #TPrintOnSequencedTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TPrintOnSequencedTest methodsFor: 'requirements' stamp: 'delaunay 4/10/2009 09:46'!
nonEmpty
 self explicitRequirement! !

!TPrintOnSequencedTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/10/2009 09:47'!
test0FixturePrintTest

	self shouldnt: [self nonEmpty ] raise: Error.! !

!TPrintOnSequencedTest methodsFor: 'tests - printing' stamp: 'delaunay 4/10/2009 10:52'!
testPrintElementsOn

	| aStream result allElementsAsString |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printElementsOn: aStream .
	allElementsAsString:=(result findBetweenSubStrs: ' ' ).
	1 to: allElementsAsString size do:
		[:i | 
		self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString).
			].! !

!TPrintOnSequencedTest methodsFor: 'tests - printing' stamp: 'delaunay 4/10/2009 15:13'!
testPrintNameOn

	| aStream result |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printNameOn: aStream .
	Transcript show: result asString.
	self nonEmpty class name first isVowel
		ifTrue:[ self assert: aStream contents =('an ',self nonEmpty class name ) ]
		ifFalse:[self assert: aStream contents =('a ',self nonEmpty class name)].! !

!TPrintOnSequencedTest methodsFor: 'tests - printing' stamp: 'delaunay 4/10/2009 15:22'!
testPrintOn
	| aStream result allElementsAsString |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printOn: aStream .
	allElementsAsString:=(result findBetweenSubStrs: ' ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i=1
			ifTrue:[
			self accessCollection class name first isVowel 
				ifTrue:[self assert: (allElementsAsString at:i)='an' ]
				ifFalse:[self assert: (allElementsAsString at:i)='a'].].
		i=2
			ifTrue:[self assert: (allElementsAsString at:i)=self accessCollection class name].
		i>2
			ifTrue:[self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString).].	
			].! !

!TPrintOnSequencedTest methodsFor: 'tests - printing' stamp: 'delaunay 4/10/2009 10:29'!
testPrintOnDelimiter
	| aStream result allElementsAsString |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printOn: aStream delimiter: ', ' .
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString).
			].! !

!TPrintOnSequencedTest methodsFor: 'tests - printing' stamp: 'delaunay 4/10/2009 10:28'!
testPrintOnDelimiterLast

	| aStream result allElementsAsString |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printOn: aStream delimiter: ', ' last: 'and'.
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 )
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:i)asString)].
		i=(allElementsAsString size-1)
			ifTrue:[ self deny: (allElementsAsString at:i)=('and')asString].
		i=(allElementsAsString size)
			ifTrue: [self assert: (allElementsAsString at:i)=((self nonEmpty at:(i-1))asString)].
			].! !

Trait named: #TPrintTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TPrintTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 16:30'!
nonEmpty
 self explicitRequirement! !

!TPrintTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 16:30'!
test0FixturePrintTest

	self shouldnt: [self nonEmpty ] raise: Error.! !

!TPrintTest methodsFor: 'tests - printing' stamp: 'delaunay 4/14/2009 16:42'!
testPrintElementsOn

	| aStream result allElementsAsString tmp |
	result:=''.
	aStream:= ReadWriteStream on: result.
	tmp:= self nonEmpty collect: [:each | each asString].
	
	self nonEmpty printElementsOn: aStream .
	allElementsAsString:=(result findBetweenSubStrs: ' ' ).
	1 to: allElementsAsString size do:
		[:i | 
		self assert: (tmp occurrencesOf:(allElementsAsString at:i))=(allElementsAsString  occurrencesOf:(allElementsAsString at:i)).
			].! !

!TPrintTest methodsFor: 'tests - printing' stamp: 'delaunay 4/14/2009 16:31'!
testPrintNameOn

	| aStream result |
	result:=''.
	aStream:= ReadWriteStream on: result.
	
	self nonEmpty printNameOn: aStream .
	Transcript show: result asString.
	self nonEmpty class name first isVowel
		ifTrue:[ self assert: aStream contents =('an ',self nonEmpty class name ) ]
		ifFalse:[self assert: aStream contents =('a ',self nonEmpty class name)].! !

!TPrintTest methodsFor: 'tests - printing' stamp: 'delaunay 4/14/2009 16:34'!
testPrintOn
	| aStream result allElementsAsString tmp |
	result:=''.
	aStream:= ReadWriteStream on: result.
	tmp:= self nonEmpty collect: [:each | each asString].
	
	self nonEmpty printOn: aStream .
	allElementsAsString:=(result findBetweenSubStrs: ' ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i=1
			ifTrue:[
			self accessCollection class name first isVowel 
				ifTrue:[self assert: (allElementsAsString at:i)='an' ]
				ifFalse:[self assert: (allElementsAsString at:i)='a'].].
		i=2
			ifTrue:[self assert: (allElementsAsString at:i)=self accessCollection class name].
		i>2
			ifTrue:[self assert: (tmp occurrencesOf:(allElementsAsString at:i))=(allElementsAsString  occurrencesOf:(allElementsAsString at:i)).].	
			].! !

!TPrintTest methodsFor: 'tests - printing' stamp: 'delaunay 4/14/2009 16:44'!
testPrintOnDelimiter
	| aStream result allElementsAsString tmp |
	result:=''.
	aStream:= ReadWriteStream on: result.
	tmp:= self nonEmpty collect: [:each | each asString].
	self nonEmpty printOn: aStream delimiter: ', ' .
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		self assert: (tmp occurrencesOf:(allElementsAsString at:i))=(allElementsAsString  occurrencesOf:(allElementsAsString at:i))
			].! !

!TPrintTest methodsFor: 'tests - printing' stamp: 'delaunay 4/14/2009 16:46'!
testPrintOnDelimiterLast

	| aStream result allElementsAsString tmp |
	result:=''.
	aStream:= ReadWriteStream on: result.
	tmp:= self nonEmpty collect:[ :each | each asString].
	
	self nonEmpty printOn: aStream delimiter: ', ' last: 'and'.
	
	allElementsAsString:=(result findBetweenSubStrs: ', ' ).
	1 to: allElementsAsString size do:
		[:i | 
		i<(allElementsAsString size-1 )
			ifTrue: [self assert: (tmp occurrencesOf: (allElementsAsString at:i))=(allElementsAsString  occurrencesOf: (allElementsAsString at:i))].
		i=(allElementsAsString size-1)
			ifTrue:[ self deny: (allElementsAsString at:i)=('and')asString].
		i=(allElementsAsString size)
			ifTrue: [self assert: (tmp occurrencesOf: (allElementsAsString at:i))=(allElementsAsString  occurrencesOf: (allElementsAsString at:i))].
			].! !

Trait named: #TPutTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TPutTest methodsFor: 'requirements' stamp: 'stephane.ducasse 1/12/2009 17:12'!
aValue

	^ self explicitRequirement! !

!TPutTest methodsFor: 'requirements' stamp: 'stephane.ducasse 1/12/2009 17:12'!
anIndex

	^ self explicitRequirement! !

!TPutTest methodsFor: 'requirements' stamp: 'stephane.ducasse 1/12/2009 17:17'!
anotherValue

	^ self explicitRequirement! !

!TPutTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:11'!
indexArray
" return a Collection including indexes between bounds of 'nonEmpty' "

	self explicitRequirement.! !

!TPutTest methodsFor: 'requirements' stamp: 'stephane.ducasse 1/12/2009 16:56'!
nonEmpty

	^ self explicitRequirement! !

!TPutTest methodsFor: 'tests - fixture' stamp: 'delaunay 3/31/2009 15:43'!
test0FixturePutTest
	self shouldnt: self aValue raise: Error.
	self shouldnt: self anotherValue raise: Error.
	
	self shouldnt: self indexArray  raise: Error.
	self indexArray do: [
		:each| 
		self assert: each class = SmallInteger. 
		self assert: (each>=1 & each<= self nonEmpty size).].
	
	self shouldnt: self valueArray   raise: Error.
	
	self assert: self indexArray size = self valueArray size.
	
	self shouldnt: self anIndex   raise: Error.
	self assert: (self anIndex >=1 & self anIndex <= self nonEmpty size).
	
	self shouldnt: self empty raise: Error.
	self assert: self empty isEmpty .
	
	self shouldnt: self nonEmpty  raise: Error.
	self deny: self nonEmpty  isEmpty.! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 4/7/2009 11:57'!
testAtAllIndexesPut
	
	self nonEmpty atAllPut: self aValue..
	self nonEmpty do:[ :each| self assert: each = self aValue].
	! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 3/31/2009 16:53'!
testAtAllPut
	| |
	self nonEmpty atAll: self indexArray put: self aValue..
	
	self indexArray do:
		[:i | self assert: (self nonEmpty at: i)=self aValue ].
	! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 4/6/2009 10:54'!
testAtAllPutAll
	self nonEmpty atAll: self indexArray putAll: self valueArray .
	
	1 to: self indexArray size do:
		[:i |
		self assert: (self nonEmpty at:(self indexArray at: i))= (self valueArray at:i) ]! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 3/31/2009 16:04'!
testAtLastPut
	| result |
	result := self nonEmpty atLast: self anIndex  put: self aValue.
	
	self assert: (self nonEmpty at: (self nonEmpty size +1 - self anIndex)) = self aValue .! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 3/27/2009 13:45'!
testAtPut
	"self debug: #testAtPut"
	
	self nonEmpty at: self anIndex put: self aValue.
	self assert: (self nonEmpty at: self anIndex) = self aValue.
	! !

!TPutTest methodsFor: 'tests - at put' stamp: 'stephane.ducasse 1/12/2009 17:23'!
testAtPutOutOfBounds
	"self debug: #testAtPutOutOfBounds"
	
	self should: [self empty at: self anIndex put: self aValue] raise: Error
	! !

!TPutTest methodsFor: 'tests - at put' stamp: 'stephane.ducasse 1/12/2009 17:17'!
testAtPutTwoValues
	"self debug: #testAtPutTwoValues"
	
	self nonEmpty at: self anIndex put: self aValue.
	self nonEmpty at: self anIndex put: self anotherValue.
	self assert: (self nonEmpty at: self anIndex) = self anotherValue.! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 4/8/2009 10:19'!
testAtWrapPut
	"self debug: #testAtWrapPut"
	
	self nonEmpty atWrap: 0 put: self aValue.
	self assert: (self nonEmpty at:(self nonEmpty size))=self aValue.
	
	self nonEmpty atWrap: (self nonEmpty size+1) put: self aValue.
	self assert: (self nonEmpty at:(1))=self aValue.
	
	self nonEmpty atWrap: (self anIndex ) put: self aValue.
	self assert: (self nonEmpty at: self anIndex) = self aValue.
	
	self nonEmpty atWrap: (self nonEmpty size+self anIndex ) put: self anotherValue .
	self assert: (self nonEmpty at:(self anIndex ))=self anotherValue .! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 4/7/2009 12:05'!
testFromToPut
	
	| collection |
	collection := self nonEmpty copy.
	collection from: 1 to: self anIndex  put: self aValue..
	1 to: self anIndex do:
		[:i | self assert: (collection at: i)= self aValue].
	(self anIndex+1) to: collection size do:
		[:i | self assert: (collection at:i)= (self nonEmpty at:i)].
	! !

!TPutTest methodsFor: 'tests - at put' stamp: 'delaunay 4/8/2009 15:44'!
testSwapWith
	"self debug: #testSwapWith"
	| result |
	result:= self nonEmpty copy .
	result swap: self anIndex with: 1.
	self assert: (result at: self anIndex) = (self nonEmpty at:1).
	self assert: (result at: 1) = (self nonEmpty at:self anIndex).
	! !

!TPutTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:12'!
valueArray
" return a collection of values to be put in 'nonEmpty'  at indexes in 'indexArray' "
	self explicitRequirement.! !

Trait named: #TRemoveByIndexTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TRemoveByIndexTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:07'!
collectionWith5Elements
" return a collection of size 5 including 5 elements"
self explicitRequirement! !

!TRemoveByIndexTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:37'!
empty
" return an empty collection"
self explicitRequirement! !

!TRemoveByIndexTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/22/2009 10:07'!
test0FixtureRemoveByIndexTest

self shouldnt: [self collectionWith5Elements  ] raise: Error.
self assert: self collectionWith5Elements  size = 5.! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:24'!
testRemoveAt

| collection element result oldSize |
collection := self collectionWith5Elements .
element := collection at: 3.
oldSize := collection size.

result := collection removeAt: 3.
self assert: result = element .
self assert: collection size = (oldSize - 1).! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:38'!
testRemoveAtNotPresent

| |
self should: [self empty removeAt: 2] raise: Error.! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:24'!
testRemoveFirst

| collection element result oldSize |
collection := self collectionWith5Elements .
element := collection first.
oldSize := collection size.

result := collection removeFirst.
self assert: result = element .
self assert: collection size = (oldSize - 1).! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:24'!
testRemoveFirstNElements

| collection elements result oldSize |
collection := self collectionWith5Elements .
elements := {collection first. collection at:2}.
oldSize := collection size.

result := collection removeFirst: 2.
self assert: result = elements .
self assert: collection size = (oldSize - 2).! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:40'!
testRemoveFirstNElementsNotPresent

self should: [self empty removeFirst: 2] raise: Error.! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:40'!
testRemoveFirstNotPresent

self should: [self empty removeFirst] raise: Error.! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:24'!
testRemoveLast

| collection element result oldSize |
collection := self collectionWith5Elements .
element := collection last.
oldSize := collection size.

result := collection removeLast.
self assert: result = element .
self assert: collection size = (oldSize - 1).! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:37'!
testRemoveLastNElements

| collection  result oldSize elements |
collection := self collectionWith5Elements .
elements := {  (collection at: (4)). collection last. }.
oldSize := collection size.


result := (collection removeLast: 2).
self assert: result = elements.
self assert: collection size = (oldSize - 2).! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:41'!
testRemoveLastNElementsNElements

self should: [self empty removeLast: 2] raise: Error.! !

!TRemoveByIndexTest methodsFor: 'tests - removing by index' stamp: 'delaunay 4/22/2009 10:40'!
testRemoveLastNotPresent

self should: [self empty removeLast] raise: Error.! !

Trait named: #TRemoveTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

Trait named: #TRemoveForMultiplenessTest
	uses: TRemoveTest
	category: 'CollectionsTests-Abstract'!

TRemoveForMultiplenessTest classTrait
	uses: TRemoveTest classTrait!

!TRemoveForMultiplenessTest methodsFor: 'test - remove' stamp: 'adrian_lienhard 2/21/2009 13:46'!
elementTwiceIn
	^ self explicitRequirement! !

!TRemoveForMultiplenessTest methodsFor: 'test - remove' stamp: 'adrian_lienhard 2/21/2009 13:46'!
testRemoveElementThatExistsTwice
	"self debug: #testRemoveElementThatDoesExistsTwice"

	| size |
	size := self nonEmpty size.
	self assert: (self nonEmpty includes: self elementTwiceIn).
	self nonEmpty remove: self elementTwiceIn.
	self assert: size - 1 = self nonEmpty size.
	
	self assert: (self nonEmpty includes: self elementTwiceIn).
	self nonEmpty remove: self elementTwiceIn.
	self assert: size - 2 = self nonEmpty size! !

!TRemoveTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 15:41'!
elementNotIn
	self explicitRequirement! !

!TRemoveTest methodsFor: 'requirements' stamp: 'stephane.ducasse 11/29/2008 14:32'!
empty
	self explicitRequirement! !

!TRemoveTest methodsFor: 'requirements' stamp: 'stephane.ducasse 11/29/2008 14:32'!
nonEmpty
	self explicitRequirement! !

!TRemoveTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 15:42'!
test0FixtureTRemoveTest
	self shouldnt: [self empty] raise: Error.
	self shouldnt: [self nonEmpty] raise: Error.
	self shouldnt: [self elementNotIn ] raise: Error.
	self assert: self empty isEmpty.
	self deny: self nonEmpty isEmpty.
	self deny: (self nonEmpty includes: self elementNotIn ).! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'delaunay 4/14/2009 15:50'!
testRemoveAll
	"self debug: #testRemoveElementThatExists"
	
	| el res subCollection |
	el := self nonEmpty anyOne.
	subCollection := (self nonEmpty copyWithout: el) .
	
	self shouldnt: [res := self nonEmpty removeAll: subCollection ] raise: Error.
	self assert: self nonEmpty size=1.
	self nonEmpty do: [:each | self assert: each = el].! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'delaunay 4/14/2009 15:49'!
testRemoveAllError
	"self debug: #testRemoveElementThatExists"
	
	| el res subCollection |
	el := self elementNotIn .
	subCollection := self nonEmpty copyWith: el.
	
	self should: [res := self nonEmpty removeAll: subCollection ] raise: Error.
! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'delaunay 4/14/2009 15:58'!
testRemoveAllFoundIn
	"self debug: #testRemoveElementThatExists"
	
	| el res subCollection |
	el := self nonEmpty anyOne.
	subCollection := (self nonEmpty copyWithout: el) copyWith: self elementNotIn .
	
	self shouldnt: [res := self nonEmpty removeAllFoundIn: subCollection ] raise: Error.
	self assert: self nonEmpty size=1.
	self nonEmpty do: [:each | self assert: each = el].! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'delaunay 4/14/2009 15:53'!
testRemoveAllSuchThat
	"self debug: #testRemoveElementThatExists"
	
	| el subCollection |
	el := self nonEmpty anyOne.
	subCollection := (self nonEmpty copyWithout: el) .
	
	self nonEmpty removeAllSuchThat: [:each | subCollection includes: each].
	self assert: self nonEmpty size=1.
	self nonEmpty do: [:each | self assert: each = el].! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'stephane.ducasse 11/29/2008 14:34'!
testRemoveElementFromEmpty
	"self debug: #testRemoveElementFromEmpty"
	
	self should: [self empty remove: self nonEmpty anyOne] raise: Error! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'stephane.ducasse 11/29/2008 14:32'!
testRemoveElementReallyRemovesElement
	"self debug: #testRemoveElementReallyRemovesElement"

	| size |
	size :=  self nonEmpty size.
	self nonEmpty remove: self nonEmpty anyOne.
	self assert: (size -1) = self nonEmpty size! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'stephane.ducasse 11/29/2008 14:30'!
testRemoveElementThatExists
	"self debug: #testRemoveElementThatExists"
	
	| el res |
	el := self nonEmpty anyOne.
	self shouldnt: [res := self nonEmpty remove: el ] raise: Error.
	self assert: res == el! !

!TRemoveTest methodsFor: 'tests - remove' stamp: 'delaunay 4/14/2009 15:43'!
testRemoveIfAbsent
	"self debug: #testRemoveElementThatExists"
	
	| el res |
	el := self elementNotIn .
	
	self shouldnt: [res := self nonEmpty remove: el  ifAbsent: [33]] raise: Error.
	self assert: res == 33.! !

Trait named: #TReplacementSequencedOldTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:35'!
elementInForReplacement
" return an element included in 'nonEmpty' "
^ self nonEmpty anyOne.! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:38'!
firstIndex
" return an index between 'nonEmpty' bounds that is < to 'second index' "
	^self explicitRequirement! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:36'!
newElement
"return an element that will be put in the collection in place of another"
	self explicitRequirement! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/6/2009 09:49'!
nonEmpty
	^self explicitRequirement! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:19'!
replacementCollection
" return a collection that will be put into 'nonEmpty' "
	^ self replacementCollectionSameSize, self 	replacementCollectionSameSize   ! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:39'!
replacementCollectionSameSize
" return a collection of size (secondIndex - firstIndex + 1)"
	^self explicitRequirement! !

!TReplacementSequencedOldTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:38'!
secondIndex
" return an index between 'nonEmpty' bounds that is > to 'second index' "
	^self explicitRequirement! !

!TReplacementSequencedOldTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/6/2009 10:34'!
testOFixtureReplacementSequencedTest

	self shouldnt: self nonEmpty   raise: Error.
	self deny: self nonEmpty isEmpty.
	
	self shouldnt: self elementInForReplacement   raise: Error.
	self assert: (self nonEmpty includes: self elementInForReplacement ) .
	
	self shouldnt: self newElement raise: Error.
	
	self shouldnt: self firstIndex  raise: Error.
	self assert: (self firstIndex >= 1 & self firstIndex <= self nonEmpty size).
	
	self shouldnt: self secondIndex   raise: Error.
	self assert: (self secondIndex >= 1 & self secondIndex <= self nonEmpty size).
	
	self assert: self firstIndex <=self secondIndex .
	
	self shouldnt: self replacementCollection   raise: Error.
	
	self shouldnt: self replacementCollectionSameSize    raise: Error.
	self assert: (self secondIndex  - self firstIndex +1)= self replacementCollectionSameSize size
	! !

!TReplacementSequencedOldTest methodsFor: 'tests - replacing' stamp: 'delaunay 4/6/2009 10:12'!
testReplaceAllWith
	| result  |
	result := self nonEmpty copy.
	result replaceAll: self elementInForReplacement  with: self newElement  .
	
	1 to: self nonEmpty size do:
		[:each |
		(self nonEmpty at: each)=self elementInForReplacement 
			ifTrue: [self assert: (result at: each)=self newElement ].
			].! !

!TReplacementSequencedOldTest methodsFor: 'tests - replacing' stamp: 'delaunay 4/6/2009 10:32'!
testReplaceFromToWith
	| result  |
	result := self nonEmpty copy.
	result replaceFrom: self firstIndex to: self secondIndex with: self replacementCollectionSameSize  .
	
	"verify content of 'result' : "
	"first part of 'result'' : '"
	1 to: (self firstIndex-1) do: [:i| self assert: (self nonEmpty at:i)=(result at: i)].
	" middle part containing replacementCollection : "
	(self firstIndex) to: (self firstIndex + self replacementCollectionSameSize  size-1) do: 
		[:i|
		self assert: (result at: i)=(self replacementCollectionSameSize  at: (i-self firstIndex+1))].
	" end part :"
	(self firstIndex + self replacementCollectionSameSize  size) to: (result size) do:
		[:i|
		self assert: (result at: i)=(self nonEmpty at: (self secondIndex +1-(self firstIndex + self replacementCollectionSameSize  size)+i))].
	
	! !

!TReplacementSequencedOldTest methodsFor: 'tests - replacing' stamp: 'delaunay 4/20/2009 16:22'!
testReplaceFromToWithStartingAt
	| result  repStart |
	result := self nonEmpty copy.
	repStart := self replacementCollection size - (self secondIndex - self firstIndex + 1 ) +1.
	result replaceFrom: self firstIndex to: self secondIndex with: self replacementCollection  startingAt: repStart   .
	
	"verify content of 'result' : "
	"first part of 'result'' : '"
	1 to: (self firstIndex-1) do: [:i| self assert: (self nonEmpty at:i)=(result at: i)].
	
	" middle part containing replacementCollection : "
	(self firstIndex) to: (self replacementCollection  size - repStart +1) do: 
		[:i|
		self assert: (result at: i)=(self replacementCollection  at: (repStart  + (i  -self firstIndex )))].
	" end part :"
	(self firstIndex + self replacementCollection  size) to: (result size) do:
		[:i|
		self assert: (result at: i)=(self nonEmpty at: (self secondIndex +1-(self firstIndex + self replacementCollection  size)+i))].! !

Trait named: #TReplacementSequencedTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TReplacementSequencedTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:36'!
collectionWith1TimeSubcollection
" return a collection including 'oldSubCollection'  only one time "
	self explicitRequirement! !

!TReplacementSequencedTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:37'!
collectionWith2TimeSubcollection
" return a collection including 'oldSubCollection'  two or many time "
	self explicitRequirement! !

!TReplacementSequencedTest methodsFor: 'tests - replacement' stamp: 'delaunay 4/17/2009 15:56'!
firstIndexesOf: subCollection in: collection
" return an OrderedCollection with the first indexes of the occurrences of subCollection in  collection "
	| tmp result currentIndex |
	tmp:= collection.
	result:= OrderedCollection new.
	currentIndex := 1.
	
	[tmp isEmpty ]whileFalse:
		[
		(tmp beginsWith: subCollection)
			ifTrue: [ 	
				result add: currentIndex.
				1 to: subCollection size do: 
					[:i | 
					tmp := tmp copyWithoutFirst.
					currentIndex := currentIndex + 1]
				]
			ifFalse: [
				tmp := tmp copyWithoutFirst.
				currentIndex := currentIndex +1.
				]
		 ].
	
	^ result.
	! !

!TReplacementSequencedTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:55'!
howMany: subCollection in: collection
" return an integer representing how many time 'subCollection'  appears in 'collection'  "
	| tmp nTime |
	tmp:= collection.
	nTime:= 0.
	
	[tmp isEmpty ]whileFalse:
		[
		(tmp beginsWith: subCollection)
			ifTrue: [ 	
				nTime := nTime + 1.
				1 to: subCollection size do: [:i | tmp := tmp copyWithoutFirst.]
				]
			ifFalse: [tmp := tmp copyWithoutFirst.]
		 ].
	
	^ nTime.
	! !

!TReplacementSequencedTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:34'!
oldSubCollection
" return a subCollection included in collectionWith1TimeSubcollection .
ex :   subCollection := #( 2 3 4) and collectionWith1TimeSubcollection := #(1 2 3 4 5)"
	self explicitRequirement! !

!TReplacementSequencedTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:36'!
replacementCollection
" return a collection that will be used to replace 'oldSubcollection' in ' collectionWith1TimeSubcollection'  " 
	self explicitRequirement! !

!TReplacementSequencedTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:54'!
test0FixtureReplacementTest
	
	self shouldnt: [self replacementCollection   ]raise: Error.
	self shouldnt: [self oldSubCollection]  raise: Error.
	
	self shouldnt: [self collectionWith1TimeSubcollection ]raise: Error.
	self assert: (self howMany: self oldSubCollection  in: self collectionWith1TimeSubcollection  ) = 1.
	
	! !

!TReplacementSequencedTest methodsFor: 'tests - replacement' stamp: 'delaunay 4/20/2009 15:32'!
testReplaceAllWith
	| result  |
	result := self nonEmpty copy.
	result replaceAll: self elementInForReplacement  with: self newElement  .
	
	1 to: self nonEmpty size do:
		[:each |
		(self nonEmpty at: each)=self elementInForReplacement 
			ifTrue: [self assert: (result at: each)=self newElement ].
			].! !

!TReplacementSequencedTest methodsFor: 'tests - replacement' stamp: 'delaunay 4/17/2009 15:59'!
testReplaceFromToWith
	| result  indexOfSubcollection lastIndexOfOldSubcollection lastIndexOfReplacementCollection |
	
	indexOfSubcollection := (self firstIndexesOf: self oldSubCollection  in:  self collectionWith1TimeSubcollection) at: 1. 
	lastIndexOfOldSubcollection := indexOfSubcollection + self oldSubCollection size -1.
	lastIndexOfReplacementCollection := indexOfSubcollection + self replacementCollection  size -1.
	
	result := self collectionWith1TimeSubcollection  replaceFrom: indexOfSubcollection  to: lastIndexOfOldSubcollection   with: self replacementCollection .
	
	"verify content of 'result' : "
	"first part of 'result'  "
	
	1 to: (indexOfSubcollection  - 1) do: 
		[ 
		:i | 
		self assert: (self collectionWith1TimeSubcollection  at:i) = (result at: i)
		].
	
	" middle part containing replacementCollection : "
	
	(indexOfSubcollection ) to: ( lastIndexOfReplacementCollection  ) do: 
		[
		:i |
		self assert: (result at: i)=(self replacementCollection at: (i - indexOfSubcollection +1))
		].
	
	" end part :"
	1 to: (result size - lastIndexOfReplacementCollection   ) do:
		[ 
		:i |
		self assert: (result at: ( lastIndexOfReplacementCollection  + i  ) ) = (self collectionWith1TimeSubcollection  at: ( lastIndexOfOldSubcollection  + i  ) ).
		].
	
	
	
	! !

!TReplacementSequencedTest methodsFor: 'tests - replacement' stamp: 'delaunay 4/17/2009 16:00'!
testReplaceFromToWithInsertion
	| result  indexOfSubcollection |
	
	indexOfSubcollection := (self firstIndexesOf: self oldSubCollection  in:  self collectionWith1TimeSubcollection) at: 1. 
	
	result := self collectionWith1TimeSubcollection  replaceFrom: indexOfSubcollection  to: ( indexOfSubcollection - 1 ) with: self replacementCollection .
	
	"verify content of 'result' : "
	"first part of 'result'' : '"
	
	1 to: (indexOfSubcollection -1) do: 
		[
		:i | 
		self assert: (self collectionWith1TimeSubcollection  at:i)=(result at: i)
		].
	
	" middle part containing replacementCollection : "
	indexOfSubcollection  to: (indexOfSubcollection  + self replacementCollection size-1) do: 
		[
		:i |
		self assert: ( result at: i )=(self replacementCollection at: ( i - indexOfSubcollection +1 ))
		].
	
	" end part :"
	(indexOfSubcollection  + self replacementCollection size) to: (result size) do:
		[:i|
		self assert: (result at: i)=(self collectionWith1TimeSubcollection  at: (i-self replacementCollection size))].
	
	" verify size: "	
	self assert: result size=(self collectionWith1TimeSubcollection  size + self replacementCollection size).
	

	

	! !

Trait named: #TSameContents
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSameContents methodsFor: 'requirements' stamp: 'stephane.ducasse 1/19/2009 16:25'!
empty
	^ self explicitRequirement! !

!TSameContents methodsFor: 'requirements' stamp: 'stephane.ducasse 1/19/2009 16:25'!
nonEmpty
	^ self explicitRequirement! !

!TSameContents methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/16/2009 20:51'!
testHasSameContents

	| oneElement |
	oneElement := self nonEmpty any.
	self assert: self empty = self empty.
	self assert: self nonEmpty = self nonEmpty. 
	self assert: (self nonEmpty remove: oneElement ; add: oneElement ; yourself) = self nonEmpty 
	! !

!TSameContents methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/16/2009 20:53'!
testHasSameContentsOfDifferentObjects

	| oneElement |
	oneElement := self anyOne.
	self assert: self empty copy = self empty.
	self assert: self nonEmpty copy = self nonEmpty.
	self assert: (self nonEmpty copy remove: oneElement ; add: oneElement ; yourself) = self nonEmpty ! !

Trait named: #TSequencedConcatanationTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSequencedConcatanationTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 16:27'!
empty
	self explicitRequirement! !

!TSequencedConcatanationTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:11'!
firstCollection
" return a collection that will be the first part of the concatenation"
	self explicitRequirement! !

!TSequencedConcatanationTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:11'!
secondCollection
" return a collection that will be the second part of the concatenation"
	self explicitRequirement! !

!TSequencedConcatanationTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 11:50'!
test0FixtureSequencedConcatenationTest
	self 
		shouldnt: self empty
		raise: Exception.
	self assert: self empty isEmpty.
	self 
		shouldnt: self firstCollection
		raise: Exception.
	self 
		shouldnt: self secondCollection
		raise: Exception! !

!TSequencedConcatanationTest methodsFor: 'tests - concatenation' stamp: 'delaunay 4/2/2009 16:42'!
testConcatenation
	| result index |
	result:= self firstCollection,self secondCollection .
	"first part : "
	index := 1.
	self firstCollection do: 
		[:each | 
		self assert: (self firstCollection at: index)=each.
		index := index+1.].
	"second part : "
	1 to: self secondCollection size do:
		[:i | 
		self assert: (self secondCollection at:i)= (result at:index).
		index:=index+1].
	"size : "
	self assert: result size = (self firstCollection size + self secondCollection size).! !

!TSequencedConcatanationTest methodsFor: 'tests - concatenation' stamp: 'delaunay 4/2/2009 16:48'!
testConcatenationWithEmpty
	| result |
	result:= self empty,self secondCollection .
	
	1 to: self secondCollection size do:
		[:i | 
		self assert: (self secondCollection at:i)= (result at:i).
		].
	"size : "
	self assert: result size = ( self secondCollection size).! !

Trait named: #TSequencedElementAccessTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSequencedElementAccessTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/30/2008 17:59'!
accessCollection

	self explicitRequirement! !

!TSequencedElementAccessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:39'!
elementInForElementAccessing
" return an element inculded in 'accessCollection '"
	self explicitRequirement! !

!TSequencedElementAccessTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:40'!
elementNotInForElementAccessing
" return an element not included in 'accessCollection' "
	self explicitRequirement! !

!TSequencedElementAccessTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:38'!
subCollectionNotIn
" return a collection for which at least one element is not included in 'accessCollection' "
	self explicitRequirement! !

!TSequencedElementAccessTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/20/2009 11:43'!
test0FixtureSequencedElementAccessTest
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self assert: self accessCollection size = 5.
	self 
		shouldnt: [ self subCollectionNotIn ]
		raise: Error.
	self subCollectionNotIn 
		detect: [ :each | (self accessCollection includes: each) not ]
		ifNone: [ self assert: false ].
	self 
		shouldnt: [ self elementNotInForElementAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForElementAccessing).
	self 
		shouldnt: [ self elementInForElementAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForElementAccessing).
	
	
! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/20/2009 11:57'!
testAfter
	"self debug: #testAfter"
	

	self assert: (self accessCollection after: (self accessCollection at: 1)) = (self accessCollection at:2).
	
	self should: [ (self accessCollection after: (self accessCollection at: (self accessCollection size)))] raise: Error.
	self should: [ self accessCollection after: self elementNotInForElementAccessing ] raise: Error.! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/20/2009 11:57'!
testAfterIfAbsent
	"self debug: #testAfterIfAbsent"
	

	self assert: (self accessCollection after: (self accessCollection at: 1) ifAbsent: [33]) = (self accessCollection at:2).
	
	self assert: (self accessCollection after: (self accessCollection at: (self accessCollection size))ifAbsent: [33]) ==33.
	self assert: (self accessCollection after: self elementNotInForElementAccessing  ifAbsent: [33]) = 33.! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/20/2009 11:58'!
testAt
	"self debug: #testAt"
	"
	self assert: (self accessCollection at: 1) = 1.
	self assert: (self accessCollection at: 2) = 2.
	"
	
	| index |
	index:= self accessCollection indexOf: self elementInForElementAccessing .
	self assert: (self accessCollection at:index)=self elementInForElementAccessing .! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 16:52'!
testAtAll
	"self debug: #testAtAll"
	
"	self flag: #theCollectionshouldbe102030intheFixture.
	
	self assert: (self accessCollection atAll: #(2 1)) first = self accessCollection second.
	self assert: (self accessCollection atAll: #(2)) first = self accessCollection second."
	
	| result |
	result:= self accessCollection atAll: #(2 1 2).
	self assert: (result at:1)=(self accessCollection at:2).
	self assert: (result at:2)=(self accessCollection at:1).
	self assert: (result at:3)=(self accessCollection at:2).
	
	self assert: (self accessCollection atAll: #()) = self accessCollection species new.
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/3/2009 11:12'!
testAtIfAbsent
	"self debug: #testAt"
	
	| absent |
	absent := false.
	self accessCollection at: (self accessCollection size +1) ifAbsent:[absent := true].	
	self assert: absent=true.
	
	absent := false.
	self accessCollection at: (self accessCollection size ) ifAbsent:[absent := true].	
	self assert: absent=false.
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/20/2009 11:57'!
testAtLast
	"self debug: #testAtLast"
	
	| index |
	self assert: (self accessCollection atLast: 1) = (self accessCollection last).
	"tmp:=1.
	self do:
		[:each | 
		each =self elementInForIndexAccessing 
			ifTrue:[index:=tmp].
		tmp:=tmp+1]."
	index:=self accessCollection indexOf: self elementInForElementAccessing .
	
	self assert: (self accessCollection atLast: (index)) =(self accessCollection at: (self accessCollection size-index+1)) 
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 16:15'!
testAtLastError
	"self debug: #testAtLast"
	

	self should: [self accessCollection atLast: (self accessCollection size +1)] raise: Error.
	
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 16:12'!
testAtLastIfAbsent
	"self debug: #testAtLastIfAbsent"
	
	| |
	self assert: (self accessCollection atLast: 1 ifAbsent:[nil]) = (self accessCollection last).
	self assert: (self accessCollection atLast: (self accessCollection size+1) ifAbsent:[222]) =222.
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 16:54'!
testAtOutOfBounds
	"self debug: #testAtOutOfBounds"
	

	self should: [self accessCollection at: (self accessCollection size+1)] raise: Error.
	self should: [self accessCollection at: -1 ] raise: Error! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'damiencassou 1/27/2009 16:52'!
testAtPin
	"self debug: #testAtPin"
	
	
	self assert: (self accessCollection atPin: 2)  = self accessCollection second.
	self assert: (self accessCollection atPin: 99)  = self accessCollection last.
	self assert: (self accessCollection atPin: -99)  = self accessCollection first.
! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/14/2009 13:46'!
testAtRandom
	| result |
	result := self nonEmpty atRandom .
	self assert: (self nonEmpty includes: result).! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/20/2009 11:59'!
testAtWrap
	"self debug: #testAt"
	"
	self assert: (self accessCollection at: 1) = 1.
	self assert: (self accessCollection at: 2) = 2.
	"
	| index |
	index := self accessCollection indexOf: self elementInForElementAccessing.
	self assert: (self accessCollection atWrap: index) = self elementInForElementAccessing.
	self assert: (self accessCollection atWrap: index + self accessCollection size) = self elementInForElementAccessing.
	self assert: (self accessCollection atWrap: index - self accessCollection size) = self elementInForElementAccessing.
	self assert: (self accessCollection atWrap: 1 + self accessCollection size) = (self accessCollection at: 1)! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'stephane.ducasse 10/30/2008 18:37'!
testBefore
	"self debug: #testBefore"
	

	self assert: (self accessCollection before: (self accessCollection at: 2)) = (self accessCollection at:1).
	
	self should: [ (self accessCollection before: (self accessCollection at: 1))] raise: Error.
	self should: [ self accessCollection before: 66] raise: Error.! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 10:17'!
testBeforeIfAbsent
	"self debug: #testBefore"
	
	self assert: (self accessCollection before: (self accessCollection at:1) ifAbsent: [99])=99.
	self assert: (self accessCollection before: (self accessCollection at:2) ifAbsent: [99])=(self accessCollection at:1).

	
! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'stephane.ducasse 10/30/2008 18:17'!
testFirstSecondThird
	"self debug: #testFirstSecondThird"
	
	self assert: (self accessCollection first) = (self accessCollection at: 1).
	self assert: (self accessCollection second) = (self accessCollection at: 2).
	self assert: (self accessCollection third) = (self accessCollection at: 3).
	self assert: (self accessCollection fourth) = (self accessCollection at: 4).
! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/7/2009 10:31'!
testLast
	"self debug: #testLast"
	
	self assert: (self accessCollection last) = (self accessCollection at: self accessCollection size).
	! !

!TSequencedElementAccessTest methodsFor: 'tests - element accessing' stamp: 'delaunay 4/15/2009 14:15'!
testMiddle
	"self debug: #testMiddle"
	
	self assert: (self accessCollection middle) = (self accessCollection at:((self accessCollection size)//2+1)).
	! !

Trait named: #TSetAritmetic
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:42'!
anotherElementIn

	^ self collection anyOne! !

!TSetAritmetic methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 16:24'!
anotherElementNotIn
" return an element included  in 'collection' "
	^ self explicitRequirement! !

!TSetAritmetic methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:28'!
collection

	^ self explicitRequirement! !

!TSetAritmetic methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 16:25'!
collectionClass
" return the class to be used to create instances of the class tested"
	^ self explicitRequirement! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 11/21/2008 17:17'!
containsAll: union of: one andOf: another
			
	self assert: (one allSatisfy: [:each | union includes: each]).
	self assert: (another allSatisfy: [:each | union includes: each])! !

!TSetAritmetic methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:28'!
nonEmpty

	^ self explicitRequirement! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:51'!
numberOfSimilarElementsInIntersection

	^ self collection occurrencesOf: self anotherElementIn! !

!TSetAritmetic methodsFor: 'tests - fixture' stamp: 'delaunay 4/21/2009 16:29'!
test0FixtureSetAritmeticTest

	self shouldnt: [ self collection ] raise: Error .
	self deny: self collection  isEmpty .
	
	self shouldnt: [ self nonEmpty  ] raise: Error .
	self deny: self nonEmpty isEmpty .
	
	self shouldnt: [ self anotherElementNotIn  ] raise: Error .
	self deny: (self collection  includes: self anotherElementNotIn ).
	
	self shouldnt: [self collectionClass] raise: Error. ! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 23:04'!
testDifference
	"Answer the set theoretic difference of two collections."
	"self debug: #testDifference"
	
	self assert: (self collection difference: self collection) isEmpty.
	self assert: (self empty difference: self collection) isEmpty.
	self assert: (self collection difference: self empty) = self collection	
! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 23:23'!
testDifferenceWithNonNullIntersection
	"Answer the set theoretic difference of two collections."
	"self debug: #testDifferenceWithNonNullIntersection"
	
	| res overlapping |
	"	#(1 2 3) difference: #(2 4) 
	->  #(1 3)"
	overlapping := (self collectionClass with: self anotherElementNotIn with: self anotherElementIn). 
	res := (self collection difference: overlapping).
	self deny: (res includes: self anotherElementIn).
	overlapping do: [:each |  self deny: (res includes: each)].
	! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 23:17'!
testDifferenceWithSeparateCollection
	"Answer the set theoretic difference of two collections."
	"self debug: #testDifferenceWithSeparateCollection"
	
	| res separateCol |
	separateCol := (self collectionClass with: self anotherElementNotIn). 
	res := (self collection difference: separateCol).
	self deny: (res includes: self anotherElementNotIn).
	self assert: res = self collection.
	res := separateCol difference: self collection.
	self deny: (res includes: self collection anyOne).
	self assert: res = separateCol.
	
	! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:52'!
testIntersectionBasic
	"self debug: #testIntersectionBasic"
	
	| inter |
	inter := self collection intersection: (self collectionClass with: self anotherElementIn).
	self deny: inter isEmpty.
	self assert: (inter includes: self anotherElementIn).! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:46'!
testIntersectionEmpty
	"self debug: #testIntersectionEmpty"
	
	| inter |
	inter := self empty intersection: self empty.
	self assert: inter isEmpty. 
	inter := self empty intersection: self nonEmpty.
	self assert: inter =  self empty.
	! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:43'!
testIntersectionItself
	"self debug: #testIntersectionItself"
	
	self assert: (self collection intersection: self collection) = self collection.
	! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:52'!
testIntersectionTwoSimilarElementsInIntersection
	"self debug: #testIntersectionBasic"
	
	| inter |
	inter := self collection intersection: (self collectionClass with: self anotherElementIn).
	self assert: (self collection  occurrencesOf: self anotherElementIn) = self numberOfSimilarElementsInIntersection.
	self assert: (inter includes: self anotherElementIn).! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 11/21/2008 17:26'!
testUnion
	"self debug: #testUnionOfEmpties"
	
	| union |
	union := self empty union: self nonEmpty.
	self containsAll: union of: self empty andOf: self nonEmpty.
	union := self nonEmpty union: self empty.
	self containsAll: union of: self empty andOf: self nonEmpty.
	union := self collection union: self nonEmpty.
	self containsAll: union of: self collection andOf: self nonEmpty.! !

!TSetAritmetic methodsFor: 'tests - set arithmetic' stamp: 'stephane.ducasse 11/21/2008 17:09'!
testUnionOfEmpties
	"self debug: #testUnionOfEmpties"
	
	self assert:  (self empty union: self empty) isEmpty.
	
	! !

Trait named: #TSizeTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSizeTest methodsFor: 'requirements' stamp: 'AlexandreBergel 1/19/2009 18:28'!
empty

	^ self explicitRequirement! !

!TSizeTest methodsFor: 'requirements' stamp: 'damiencassou 1/20/2009 14:03'!
sizeCollection
	"Answers a collection whose #size is 4"
	^ self explicitRequirement! !

!TSizeTest methodsFor: 'tests - fixture' stamp: 'damiencassou 1/20/2009 14:04'!
test0TSizeTest
	self shouldnt: [self empty] raise: Error.
	self shouldnt: [self sizeCollection] raise: Error.
	self assert: self empty isEmpty.
	self deny: self sizeCollection isEmpty.! !

!TSizeTest methodsFor: 'tests - size capacity' stamp: 'stephane.ducasse 10/6/2008 19:32'!
testSize

	self assert: self empty size = 0.
	self assert: self sizeCollection size = 4! !

Trait named: #TSortTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSortTest methodsFor: 'requirements' stamp: 'delaunay 3/27/2009 14:52'!
sortedInAscendingOrderCollection
	^self explicitRequirement
	! !

!TSortTest methodsFor: 'tests - fixture' stamp: 'delaunay 3/27/2009 15:16'!
test0SortingArrayedTest
	| tmp max |
	" an unsorted collection of number "
	self shouldnt: [ self  unsortedCollection ]raise: Error.
	self  unsortedCollection do:[:each | each isNumber].
	max := self unsortedCollection max.
	self assert: (self unsortedCollection at: 1) <max.
	self assert: (self unsortedCollection last) <max.
 	
	
	" a collection of number sorted in an ascending order"
	self shouldnt: [ self  sortedInAscendingOrderCollection  ]raise: Error.
	self  sortedInAscendingOrderCollection do:[:each | each isNumber].
	tmp:= self sortedInAscendingOrderCollection at:1.
	self sortedInAscendingOrderCollection do:
		[: each | each<= tmp. tmp:=each]
	! !

!TSortTest methodsFor: 'tests - sorting' stamp: 'delaunay 3/27/2009 14:40'!
testIsSorted
	self assert: [ self sortedInAscendingOrderCollection isSorted ].
	self deny: [ self unsortedCollection isSorted ]! !

!TSortTest methodsFor: 'tests - sorting' stamp: 'delaunay 3/27/2009 14:43'!
testIsSortedBy
	self assert: (self sortedInAscendingOrderCollection isSortedBy: [:a :b | a<b]).
	self deny: (self sortedInAscendingOrderCollection isSortedBy: [:a :b | a>b]).
! !

!TSortTest methodsFor: 'tests - sorting' stamp: 'delaunay 3/27/2009 15:22'!
testSort
	| result tmp |
	result := self unsortedCollection sort.
	tmp := result at: 1.
	result do:
		[:each | self assert: each>=tmp. tmp:= each. ].! !

!TSortTest methodsFor: 'tests - sorting' stamp: 'delaunay 3/27/2009 15:22'!
testSortUsingSortBlock
	| result tmp |
	result := self unsortedCollection sort: [:a :b | a>b].
	tmp := result at: 1.
	result do:
		[:each | self assert: each<=tmp. tmp:= each. ].! !

!TSortTest methodsFor: 'requirements' stamp: 'delaunay 3/27/2009 14:52'!
unsortedCollection
	^self explicitRequirement! !

Trait named: #TStructuralEqualityTest
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TStructuralEqualityTest commentStamp: 'stephane.ducasse 1/16/2009 19:08' prior: 0!
Tests whether two objects are equals: they contain the same objects in the same order.!

Trait named: #TSequencedStructuralEqualityTest
	uses: TStructuralEqualityTest
	category: 'CollectionsTests-Abstract'!

TSequencedStructuralEqualityTest classTrait
	uses: TStructuralEqualityTest classTrait!

!TSequencedStructuralEqualityTest methodsFor: 'tests - fixture' stamp: 'damiencassou 1/20/2009 13:59'!
test0TSequencedStructuralEqualityTest

	self shouldnt: [self nonEmpty at: 1] raise: Error. "Ensures #nonEmpty is sequenceable"! !

!TSequencedStructuralEqualityTest methodsFor: 'test - equality' stamp: 'damiencassou 1/20/2009 14:01'!
testEqualSignForSequenceableCollections
	"self debug: #testEqualSign"

	self deny: (self nonEmpty = self nonEmpty asSet). 
	self deny: (self nonEmpty reversed = self nonEmpty).
	self deny: (self nonEmpty = self nonEmpty reversed).! !

!TSequencedStructuralEqualityTest methodsFor: 'test - equality' stamp: 'damiencassou 1/20/2009 13:56'!
testHasEqualElements
	"self debug: #testHasEqualElements"

	self deny: (self empty hasEqualElements: self nonEmpty).
	self deny: (self nonEmpty hasEqualElements: self nonEmpty asSet). 
	self deny: (self nonEmpty reversed hasEqualElements: self nonEmpty).
	self deny: (self nonEmpty hasEqualElements: self nonEmpty reversed).! !

!TSequencedStructuralEqualityTest methodsFor: 'test - equality' stamp: 'damiencassou 1/20/2009 13:56'!
testHasEqualElementsIsTrueForNonIdenticalButEqualCollections
	"self debug: #testHasEqualElementsIsTrueForNonIdenticalButEqualCollections"
		
	self assert: (self empty hasEqualElements: self empty copy). 
	self assert: (self empty copy hasEqualElements: self empty).
	self assert: (self empty copy hasEqualElements: self empty copy).
		
	self assert: (self nonEmpty hasEqualElements: self nonEmpty copy). 
	self assert: (self nonEmpty copy hasEqualElements: self nonEmpty).
	self assert: (self nonEmpty copy hasEqualElements: self nonEmpty copy).! !

!TSequencedStructuralEqualityTest methodsFor: 'test - equality' stamp: 'damiencassou 1/20/2009 13:56'!
testHasEqualElementsOfIdenticalCollectionObjects
	"self debug: #testHasEqualElementsOfIdenticalCollectionObjects"
	
	self assert: (self empty hasEqualElements: self empty). 
	self assert: (self nonEmpty hasEqualElements: self nonEmpty). 
	! !

!TStructuralEqualityTest methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/19/2009 16:27'!
empty
	
	^ self explicitRequirement! !

!TStructuralEqualityTest methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/19/2009 16:27'!
nonEmpty
	
	^ self explicitRequirement! !

!TStructuralEqualityTest methodsFor: 'tests - fixture' stamp: 'damiencassou 1/20/2009 13:57'!
test0TStructuralEqualityTest
	self shouldnt: [self empty] raise: Error.
	self shouldnt: [self nonEmpty] raise: Error.
	self assert: self empty isEmpty.
	self deny: self nonEmpty isEmpty.! !

!TStructuralEqualityTest methodsFor: 'test - equality' stamp: 'damiencassou 1/20/2009 14:02'!
testEqualSign
	"self debug: #testEqualSign"

	self deny: (self empty = self nonEmpty).! !

!TStructuralEqualityTest methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/16/2009 19:25'!
testEqualSignIsTrueForNonIdenticalButEqualCollections
	"self debug: #testEqualSignIsTrueForNonIdenticalButEqualCollections"
		
	self assert: (self empty = self empty copy). 
	self assert: (self empty copy = self empty).
	self assert: (self empty copy = self empty copy).
		
	self assert: (self nonEmpty = self nonEmpty copy). 
	self assert: (self nonEmpty copy = self nonEmpty).
	self assert: (self nonEmpty copy = self nonEmpty copy).! !

!TStructuralEqualityTest methodsFor: 'test - equality' stamp: 'stephane.ducasse 1/16/2009 19:25'!
testEqualSignOfIdenticalCollectionObjects
	"self debug: #testEqualSignOfIdenticalCollectionObjects"
	
	self assert: (self empty = self empty). 
	self assert: (self nonEmpty = self nonEmpty). 
	! !

Trait named: #TSubCollectionAccess
	uses: {}
	category: 'CollectionsTests-Abstract'!

!TSubCollectionAccess methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:26'!
accessCollection
" return a collection of size 5"
	self explicitRequirement! !

!TSubCollectionAccess methodsFor: 'tests - fixture' stamp: 'delaunay 4/10/2009 15:54'!
test0FixtureSubcollectionAccessTest

	self shouldnt: [self accessCollection ] raise: Error.
	self assert: self accessCollection size> 2.! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:51'!
testAllButFirst
	"self debug: #testAllButFirst"
	
	| abf col |
	col := self accessCollection.
	abf := col allButFirst.
	self deny: abf first = col first. 
	self assert: abf size +1  = col size.! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:53'!
testAllButFirstNElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButFirst: 2.
	1 to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i + 2) ].
	self assert: abf size + 2 = col size! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:52'!
testAllButLast
	"self debug: #testAllButLast"
	
	| abf col |
	col := self accessCollection.
	abf := col allButLast.
	self deny: abf last = col last. 
	self assert: abf size + 1= col size.
	! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:53'!
testAllButLastNElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButLast: 2.
	1 
		to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i) ].
	self assert: abf size + 2 = col size! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:55'!
testFirstNElements
	"self debug: #testFirstNElements"
	
	| result |
	result:= self accessCollection first: (self accessCollection size-1).
	1 to: result size do:
		[:i | self assert: (result at:i)=(self accessCollection at:i)].
	
	self assert: result size = (self accessCollection size -1).
	self should: [self accessCollection first: (self accessCollection size+1)] raise: Error.! !

!TSubCollectionAccess methodsFor: 'tests - subcollections access' stamp: 'delaunay 4/10/2009 15:56'!
testLastNElements
	"self debug: #testLastNElements"
	
	| result |
	result:= self accessCollection last: (self accessCollection size-1).
	
	1 to: result size do:
		[:i | self assert: (result at:i)=(self accessCollection at:i+1)].
	
	self assert: result size = (self accessCollection size -1).
	self should: [self accessCollection last: (self accessCollection size+1)] raise: Error.! !

HashAndEqualsTestCase subclass: #TextAlignmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextAlignmentTest methodsFor: 'initialization' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes add: TextAlignment centered;
		 add: TextAlignment justified;
		 add: TextAlignment leftFlush;
		 add: TextAlignment rightFlush ! !

HashAndEqualsTestCase subclass: #TextEmphasisTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextEmphasisTest methodsFor: 'initialization' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes add: TextEmphasis bold;
		 add: TextEmphasis italic;
		 add: TextEmphasis narrow;
		 add: TextEmphasis normal;
		 add: TextEmphasis struckOut;
		 add: TextEmphasis underlined ! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:03'!
testAppendString
	"tests the Text>>prepend: method when appending a String " 
	"self run: # testAppendString"

	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy'.
	expectedResult := 'xxxyyy' asText 
		addAttribute: TextEmphasis bold from: 1 to: 3.
	result := receiver append: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testAppendText
	"tests the Text>>prepend: method when appending a Text " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy' asText
		addAttribute: TextEmphasis italic from: 1 to: 3.		.
	expectedResult := 'xxxyyy' asText 
		addAttribute: TextEmphasis bold from: 1 to: 3;
		addAttribute: TextEmphasis italic from: 4 to: 6.
	result := receiver append: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)

	
	! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testPrependString
	"tests the Text>>prepend: method when prepending a String " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy'.
	expectedResult := 'yyyxxx' asText 
		addAttribute: TextEmphasis bold from: 4 to: 6.
	result := receiver prepend: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)

	! !

!TextEmphasisTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:04'!
testPrependText
	"tests the Text>>prepend: method when prepending Text " 
	| receiver argument result expectedResult |

	receiver := 'xxx' asText  
		addAttribute: TextEmphasis bold from: 1 to: 3.
	argument := 'yyy' asText 
		addAttribute: TextEmphasis italic from: 1 to: 3.
	expectedResult := 'yyyxxx' asText 
		addAttribute: TextEmphasis italic from: 1 to: 3;
		addAttribute: TextEmphasis bold from: 4 to: 6.
	result := receiver prepend: argument.
	self assert: (result == receiver). 
	self assert: (result string = expectedResult string).
	self assert: (result runs = expectedResult runs)! !

HashAndEqualsTestCase subclass: #TextFontChangeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextFontChangeTest methodsFor: 'initialization' stamp: 'mjr 8/20/2003 18:55'!
setUp
	"create the prototypes for testing"
	super setUp.
	prototypes add: TextFontChange defaultFontChange.
	prototypes add: TextFontChange font1.
	prototypes add: TextFontChange font2.
	prototypes add: TextFontChange font3.
	prototypes add: TextFontChange font4.
	prototypes
		add: (TextFontChange fontNumber: 6) ! !

!TextFontChangeTest methodsFor: 'tests' stamp: 'mjr 8/17/2003 20:29'!
testEquality
	"Check that different instances of the same TextFontChange are equal"
	self assert: TextFontChange defaultFontChange = TextFontChange defaultFontChange.
	self assert: TextFontChange font1 = TextFontChange font1.
	self assert: TextFontChange font2 = TextFontChange font2.
	self assert: TextFontChange font3 = TextFontChange font3.
	self assert: TextFontChange font4 = TextFontChange font4.
	self assert: (TextFontChange fontNumber: 6)
			= (TextFontChange fontNumber: 6)! !

!TextFontChangeTest methodsFor: 'tests' stamp: 'stephaneducasse 2/4/2006 20:10'!
testHash
	"test that different instances of the same TextFontChange hash to the 
	same value"
	| hashes hash |
	hashes := OrderedCollection new.
	1
		to: 100
		do: [:i | hashes add: TextFontChange defaultFontChange hash].
	hash := hashes at: 1.
	2
		to: 100
		do: [:i | self assert: (hashes at: i)
					= hash]! !

HashAndEqualsTestCase subclass: #TextFontReferenceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextFontReferenceTest methodsFor: 'tests' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes
		add: (TextFontReference
				toFont: (StrikeFont familyName: 'NewYork' size: 15)) ! !

HashAndEqualsTestCase subclass: #TextKernTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextKernTest methodsFor: 'tests' stamp: 'mjr 8/20/2003 18:55'!
setUp
	super setUp.
	prototypes
		add: (TextKern kern: 1) ! !

ClassTestCase subclass: #AssociationTest
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!AssociationTest methodsFor: 'setup' stamp: 'zz 12/5/2005 18:33'!
setUp

	a := 1 -> 'one'.
	b := 1 -> 'een'.! !

!AssociationTest methodsFor: 'tests' stamp: 'ab 12/29/2008 07:59'!
testComparison
	self assert: ((#a -> 'foo') < (#b -> 'zork'))! !

!AssociationTest methodsFor: 'tests' stamp: 'md 3/8/2004 16:37'!
testEquality
	
	self 
		assert: (a key = b key);
		deny: (a value = b value);
		deny: (a = b)

! !

!AssociationTest methodsFor: 'tests' stamp: 'al 10/13/2008 20:32'!
testHash

	self 
		assert: (a hash = a copy hash);
		assert: (a hash = b hash)! !

!AssociationTest methodsFor: 'tests' stamp: 'ab 12/29/2008 08:01'!
testIsSelfEvaluating
	self assert: (a isSelfEvaluating).
	
	self assert: (a printString = '1->''one''')
! !

ClassTestCase subclass: #CharacterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!CharacterTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Character. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!CharacterTest methodsFor: 'testing-printing' stamp: 'stephane.ducasse 5/25/2008 15:47'!
testHex
	self assert: $a hex = '61'.
	self assert: Character space hex = '20'! !

!CharacterTest methodsFor: 'testing - class methods' stamp: 'sd 6/5/2005 09:25'!
testNew

	self should: [Character new] raise: Error.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:41'!
testPrintString
	self assert: $a printString = '$a'.
	self assert: $5 printString = '$5'.
	self assert: $@ printString = '$@'.

	self assert: Character cr printString = 'Character cr'.
	self assert: Character lf printString = 'Character lf'.
	self assert: Character space printString = 'Character space'.

	self assert: (Character value: 0) printString = 'Character value: 0'.
	self assert: (Character value: 17) printString = 'Character value: 17'.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:41'!
testPrintStringAll
	Character allCharacters do: [ :each |
		self assert: (self class compilerClass 
			evaluate: each printString) = each ].! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:22'!
testStoreString
	self assert: $a storeString = '$a'.
	self assert: $5 storeString = '$5'.
	self assert: $@ storeString = '$@'.

	self assert: Character cr storeString = 'Character cr'.
	self assert: Character lf storeString = 'Character lf'.
	self assert: Character space storeString = 'Character space'.

	self assert: (Character value: 0) storeString = '(Character value: 0)'.
	self assert: (Character value: 17) storeString = '(Character value: 17)'.! !

!CharacterTest methodsFor: 'testing-printing' stamp: 'lr 11/21/2005 17:24'!
testStoreStringAll
	Character allCharacters do: [ :each |
		self assert: (self class compilerClass 
			evaluate: each storeString) = each ].! !

ClassTestCase subclass: #CollectionRootTest
	uses: TIterateTest + TEmptyTest + TSizeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Abstract'!

!CollectionRootTest commentStamp: 'stephane.ducasse 1/12/2009 17:41' prior: 0!
I'm the root of the hierarchy of the collection tests.
!

CollectionRootTest class
	uses: TIterateTest classTrait + TEmptyTest classTrait + TSizeTest classTrait
	instanceVariableNames: ''!

CollectionRootTest subclass: #ArrayTest
	uses: TEmptySequenceableTest + TIterateSequenceableTest + TSequencedElementAccessTest + TCloneTest + TIncludesTest + TCopyTest + TSetAritmetic + TCreationWithTest + TPutTest + TConvertTest - {} + TSortTest + TOccurrencesTest + TIterateSequencedReadableTest + TSequencedConcatanationTest + TReplacementSequencedOldTest + TAsStringCommaAndDelimiterSequenceableTest + TBeginsEndsWith + TPrintOnSequencedTest + TIndexAccess + TSubCollectionAccess + TConvertAsSetForMultiplinessTest + TCopyPartOfSequenceable + TCopySequenceableSameContents + TCopySequenceableWithOrWithoutSpecificElements + TCopySequenceableWithReplacement + TIndexAccessForMultipliness + TCopyPartOfSequenceableForMultipliness
	instanceVariableNames: 'example1 literalArray selfEvaluatingArray otherArray nonSEArray1 nonSEarray2 example2 empty collection result withoutEqualElements withEqualElements withCharacters unsortedCollection sortedInAscendingOrderCollection sizeCollection collectionNotIncluded removedCollection elementInForCopy elementNotInForCopy firstIndex secondIndex replacementCollection indexArray valueArray nonEmptyMoreThan1Element subCollectionNotIn replacementCollectionSameSize oldSubCollection nonEmpty1Element collectionOfCollection collectionOfFloatWithEqualElements floatCollectionWithSameBeginingAnEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!ArrayTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Array. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

ArrayTest class
	uses: TEmptySequenceableTest classTrait + TIterateSequenceableTest classTrait + TSequencedElementAccessTest classTrait + TCloneTest classTrait + TIncludesTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait + TCreationWithTest classTrait + TPutTest classTrait + TConvertTest classTrait + TSortTest classTrait + TOccurrencesTest classTrait + TIterateSequencedReadableTest classTrait + TSequencedConcatanationTest classTrait + TReplacementSequencedOldTest classTrait + TAsStringCommaAndDelimiterSequenceableTest classTrait + TBeginsEndsWith classTrait + TPrintOnSequencedTest classTrait + TIndexAccess classTrait + TSubCollectionAccess classTrait + TConvertAsSetForMultiplinessTest classTrait + TCopyPartOfSequenceable classTrait + TCopySequenceableSameContents classTrait + TCopySequenceableWithOrWithoutSpecificElements classTrait + TCopySequenceableWithReplacement classTrait + TIndexAccessForMultipliness classTrait + TCopyPartOfSequenceableForMultipliness classTrait
	instanceVariableNames: ''!

!ArrayTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:12'!
aValue

	^ 33! !

!ArrayTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/30/2008 19:02'!
accessCollection

	^ example1! !

!ArrayTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:13'!
anIndex

	^ 2! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:15'!
anotherElementNotIn
	^ 42! !

!ArrayTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:19'!
anotherValue

	^ 66! !

!ArrayTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 12:35'!
assertSameContents: aCollection whenConvertedTo: aClass 
	| result |
	result := aCollection perform: ('as' , aClass name) asSymbol.
	self assert: (result class includesBehavior: aClass).
	result do: 
		[ :each | 
		self assert: (aCollection occurrencesOf: each) = (result occurrencesOf: each) ].
	self assert: result size = aCollection size! !

!ArrayTest methodsFor: 'initialization' stamp: 'stephane.ducasse 10/6/2008 16:53'!
collection 

	^ collection ! !

!ArrayTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 22:00'!
collectionClass 
	
	^ Array! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 11:11'!
collectionInForIncluding
	^ self nonEmpty copyWithoutFirst.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/30/2009 10:51'!
collectionNotIncluded
	^ collectionNotIncluded.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	^ collectionOfCollection! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 10:28'!
collectionWith1TimeSubcollection
	^ (self oldSubCollection copyWithoutFirst),self oldSubCollection,(self oldSubCollection copyWithoutFirst). ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 10:29'!
collectionWith2TimeSubcollection
	^ (self oldSubCollection copyWithoutFirst),self oldSubCollection,(self oldSubCollection copyWithoutFirst),self oldSubCollection .! !

!ArrayTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/26/2009 09:58'!
collectionWithCharacters
	^ withCharacters.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:24'!
collectionWithElementsToRemove
	^ removedCollection! !

!ArrayTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/26/2009 09:57'!
collectionWithEqualElements
	^ withEqualElements.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 11:15'!
collectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals .
(others elements of the collection are not equal to those elements)"
	^ floatCollectionWithSameBeginingAnEnd ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:25'!
collectionWithoutEqualElements
	^ withoutEqualElements .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 15:49'!
collectionWithoutEqualsElements
	^ withoutEqualElements ! !

!ArrayTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:27'!
element
	^ 3! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:22'!
elementInCollectionOfFloat
	^ collectionOfCollection atRandom! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 10:28'!
elementInForCopy
	^ elementInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:39'!
elementInForElementAccessing
" return an element inculded in 'accessCollection '"
	^ self accessCollection anyOne! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 13:55'!
elementInForIncludesTest

	^ elementInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/10/2009 14:49'!
elementInForIndexAccess
	^ elementInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/7/2009 16:06'!
elementInForIndexAccessing

	^elementInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 11:54'!
elementInForOccurrences
	^ elementInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/6/2009 10:16'!
elementInForReplacement
	^ elementInForCopy ! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:54'!
elementNotIn

	^ 666.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 10:29'!
elementNotInForCopy
	^ elementNotInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:39'!
elementNotInForElementAccessing
" return an element not included in 'accessCollection' "
	^ elementNotInForCopy ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/7/2009 11:18'!
elementNotInForIndexAccessing

	^elementNotInForCopy ! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:55'!
elementNotInForOccurrences
	^ elementNotInForCopy! !

!ArrayTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ 55! !

!ArrayTest methodsFor: 'initialization' stamp: 'stephane.ducasse 10/5/2008 15:06'!
empty

	^ empty! !

!ArrayTest methodsFor: 'test - iterate' stamp: 'luc.fabresse 11/29/2008 23:10'!
expectedSizeAfterReject
	^1! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 16:41'!
firstCollection
	^example1 ! !

!ArrayTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 10:35'!
firstEven
	"Returns the first even number of #collection"
	^ -2! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 10:29'!
firstIndex
	^ firstIndex ! !

!ArrayTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 10:35'!
firstOdd
	"Returns the first odd number of #collection"
	^ 1! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 12:08'!
floatCollectionWithEqualElements
" return a collection of float including equal elements (classic equality)"
	^ collectionOfFloatWithEqualElements ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:00'!
floatCollectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals only with classic equality (they are not the same object).
(others elements of the collection are not equal to those elements)"
	^ floatCollectionWithSameBeginingAnEnd ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 12:04'!
floatCollectionWithoutEqualElements
" return a collection that does niot include equal elements ( classic equality )"
	^ collectionOfCollection ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 16:00'!
indexArray
	^ indexArray .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 15:59'!
indexInForCollectionWithoutDuplicates
	^ 2.! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/16/2009 16:56'!
indexInNonEmpty
	^ 2 ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 15:53'!
integerCollection
	^example1 .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:28'!
integerCollectionWithoutEqualElements
	^{1. 2. 6. 5.}! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/6/2009 10:16'!
newElement
	^999! !

!ArrayTest methodsFor: 'initialization' stamp: 'stephane.ducasse 10/5/2008 15:06'!
nonEmpty

	^ example1! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/8/2009 11:40'!
nonEmpty1Element

	^ nonEmpty1Element  ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 15:20'!
nonEmptyMoreThan1Element
	^nonEmptyMoreThan1Element .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/6/2009 11:41'!
oldSubCollection
	^oldSubCollection ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 10:29'!
replacementCollection
	^replacementCollection .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/6/2009 10:32'!
replacementCollectionSameSize
	^replacementCollectionSameSize ! !

!ArrayTest methodsFor: 'initialization' stamp: 'stephane.ducasse 10/6/2008 16:54'!
result
	
	^ result! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 16:41'!
secondCollection
	^example2 ! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 10:29'!
secondIndex
	^ secondIndex ! !

!ArrayTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/5/2008 15:12'!
selectorToAccessValuePutIn
	"return the selector of the method that should be invoked to access an element"
	
	^ #second! !

!ArrayTest methodsFor: 'initialization' stamp: 'delaunay 4/21/2009 13:59'!
setUp

	literalArray := #(1 true 3 #four).
	selfEvaluatingArray := { 1. true. (3/4). Color black. (2 to: 4) . 5 }.
	nonSEArray1 := { 1 . Set with: 1 }.
	nonSEarray2 := { Smalltalk associationAt: #Array }.
	example1 := #(1 2 3 4 5) copy.
	indexArray:= {2. 3. 4.}.
	valueArray:={0. 0. 0.}.
	oldSubCollection:= {2. 3. 4.}.
	nonEmptyMoreThan1Element:= example1.
	subCollectionNotIn:= {1. 8. 3.}.
	collectionNotIncluded:= {7. 8. 9.}.
	removedCollection:=  { 2. 4. }.
	
	example2 := {1. 2. 3/4. 4. 5}.
	collection := #(1 -2 3 1). 
	result := {SmallInteger. SmallInteger. SmallInteger. SmallInteger.}.
	empty := #().
	withEqualElements := {1.5. 5.2. 6.1. 2.0. 5.2.} .
	withoutEqualElements := {1.1. 4.4. 6.5. 2.4. 3.1.}.
	withCharacters := {$a. $x. $d. $c. $m.}.
	unsortedCollection := {1. 2.  8. 5. 6. 7.}.
	sortedInAscendingOrderCollection := {1. 2. 3. 4. 5. 6.}.
	elementInForCopy:= 2.
	elementNotInForCopy:= 9.
	firstIndex:= 2.
	secondIndex:= 4.
	replacementCollection:= {4. 3. 2. 1.}.
	replacementCollectionSameSize := {5. 4. 3.}.	
	nonEmpty1Element:={ 5.}.
	collectionOfCollection:={1.5. 5.5. 6.5.}.
	collectionOfFloatWithEqualElements:={1.5. 5.5. 6.5. 1.5}.
	floatCollectionWithSameBeginingAnEnd := {1.5. 5.5. 1.5 copy}.! !

!ArrayTest methodsFor: 'requirements' stamp: 'damienpollet 1/13/2009 16:59'!
sizeCollection

	^ self collection! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 14:08'!
smallerIndex
	^ firstIndex -1! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/27/2009 15:20'!
sortedInAscendingOrderCollection
	^sortedInAscendingOrderCollection .
	! !

!ArrayTest methodsFor: 'test - iterate' stamp: 'stephane.ducasse 10/6/2008 17:39'!
speciesClass
	
	^ Array! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 4/3/2009 11:35'!
subCollectionNotIn

	^subCollectionNotIn ! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: self collectionWithElementsToRemove
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: self elementToAdd
		raise: Error! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:37'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each class = Float ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:36'!
test0FixtureIndexAccessTest
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self deny: self accessCollection isEmpty.
	self 
		shouldnt: [ self elementInForIndexAccess ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccess).
	self 
		shouldnt: [ self elementNotInForIndexAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForIndexAccessing).
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each class=Float ].
	self 
		shouldnt: [ self elementInCollectionOfFloat ]
		raise: Error.
	self assert: (self collectionOfFloat includes: self elementInCollectionOfFloat)! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/2/2009 11:53'!
test0FixtureOccurrencesTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: self elementInForOccurrences
		raise: Error.
	self assert: (self nonEmpty includes: self elementInForOccurrences).
	self 
		shouldnt: self elementNotInForOccurrences
		raise: Error.
	self deny: (self nonEmpty includes: self elementNotInForOccurrences)! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/14/2009 11:50'!
test0FixtureSequencedConcatenationTest
	self 
		shouldnt: self empty
		raise: Exception.
	self assert: self empty isEmpty.
	self 
		shouldnt: self firstCollection
		raise: Exception.
	self 
		shouldnt: self secondCollection
		raise: Exception! !

!ArrayTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:37'!
test0IndexAccessingTest
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self assert: self accessCollection size = 5.
	self 
		shouldnt: [ self subCollectionNotIn ]
		raise: Error.
	self subCollectionNotIn 
		detect: [ :each | (self accessCollection includes: each) not ]
		ifNone: [ self assert: false ].
	self 
		shouldnt: [ self elementNotInForIndexAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForIndexAccessing).
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each class=Float ]! !

!ArrayTest methodsFor: 'test - iterate' stamp: 'damienpollet 1/13/2009 16:28'!
testAnySatisfy

	self assert: ( self collection anySatisfy: [:each | each = -2]).
	self deny: (self collection anySatisfy: [:each | each isString]).! !

!ArrayTest methodsFor: 'tests - converting' stamp: 'cyrille.delaunay 3/26/2009 14:55'!
testAsArray
	"self debug: #testAsArray3"
	self 
		assertSameContents: self collectionWithoutEqualElements
		whenConvertedTo: Array! !

!ArrayTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/10/2009 16:20'!
testAtWrap
	"self debug: #testAt"
	"
	self assert: (self accessCollection at: 1) = 1.
	self assert: (self accessCollection at: 2) = 2.
	"
	| index |
	index := self accessCollection indexOf: self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index + self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index - self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: 1 + self accessCollection size) = (self accessCollection at: 1)! !

!ArrayTest methodsFor: 'tests - accessing' stamp: 'delaunay 4/10/2009 16:19'!
testAtWrap2
	| tabTest |
	tabTest := #(5 6 8 ).
	self assert: (tabTest atWrap: 2) = 6.
	self assert: (tabTest atWrap: 7) = 5.
	self assert: (tabTest atWrap: 5) = 6.
	self assert: (tabTest atWrap: 0) = 8.
	self assert: (tabTest atWrap: 1) = 5.
	self assert: (tabTest atWrap: -2) = 5! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!ArrayTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!ArrayTest methodsFor: 'test - iterate' stamp: 'delaunay 4/14/2009 14:12'!
testDo

	| res |
	res := OrderedCollection new.  
	self collection do: [:each | res add: each class].
	self assert: res asArray = self result.! !

!ArrayTest methodsFor: 'test - iterate' stamp: 'delaunay 4/14/2009 14:13'!
testDo2

	| res |
	res := OrderedCollection new.  
	self collection do: [:each | res add: each class].
	self assert: res asArray = self result. ! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!ArrayTest methodsFor: 'tests - index access' stamp: 'delaunay 4/15/2009 14:22'!
testIdentityIndexOf
	"self debug: #testIdentityIndexOf"
	| collection element |
	element := self elementInCollectionOfFloat copy.
	self deny: self elementInCollectionOfFloat == element.
	collection := self collectionOfFloat copyWith: element.
	self assert: (collection identityIndexOf: element) = collection size! !

!ArrayTest methodsFor: 'tests - index access' stamp: 'delaunay 4/15/2009 14:22'!
testIdentityIndexOfIAbsent
	"self debug: #testIdentityIndexOfIfAbsent"
	| collection element |
	element := self elementInCollectionOfFloat copy.
	self deny: self elementInCollectionOfFloat == element.
	collection := self collectionOfFloat copyWith: element.
	self assert: (collection 
			identityIndexOf: element
			ifAbsent: [ 0 ]) = collection size.
	self assert: (self collectionOfFloat 
			identityIndexOf: element
			ifAbsent: [ 55 ]) = 55! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!ArrayTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:12'!
testIsArray
	
	self assert: example1 isArray! !

!ArrayTest methodsFor: 'testing' stamp: 'apb 4/21/2006 08:59'!
testIsLiteral
	"We work with a copy of literalArray, to avoid corrupting the code."
	| l |
	l := literalArray copy.
	self assert: l isLiteral.
	l at: 1 put: self class.
	self deny: l isLiteral! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:18'!
testIsSelfEvaluating

	self assert: example1 isSelfEvaluating.
	example1 at: 1 put: Bag new.
	self deny: example1 isSelfEvaluating.
	example1 at: 1 put: 1.! !

!ArrayTest methodsFor: 'testing' stamp: 'zz 12/5/2005 17:50'!
testLiteralEqual
	self
		deny: (example1 literalEqual: example1 asIntegerArray)! !

!ArrayTest methodsFor: 'testing' stamp: 'dc 5/24/2007 10:56'!
testNewWithSize
	|array|
	array := Array new: 5.
	self assert: array size = 5.
	1 to: 5 do: [:index | self assert: (array at: index) isNil]! !

!ArrayTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:52'!
testOccurrencesOf
	| result expected |
	result := self nonEmpty occurrencesOf: self elementInForOccurrences.
	expected := 0.
	self nonEmpty do: [ :each | self elementInForOccurrences = each ifTrue: [ expected := expected + 1 ] ].
	self assert: result = expected! !

!ArrayTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:52'!
testOccurrencesOfEmpty
	| result |
	result := self empty occurrencesOf: self elementInForOccurrences.
	self assert: result = 0! !

!ArrayTest methodsFor: 'tests - occurrencesOf' stamp: 'delaunay 4/2/2009 11:53'!
testOccurrencesOfNotIn
	| result |
	result := self empty occurrencesOf: self elementNotInForOccurrences.
	self assert: result = 0! !

!ArrayTest methodsFor: 'testing' stamp: 'stephane.ducasse 10/6/2008 16:53'!
testPremultiply
	self assert: example1 +* #(2 ) = #(2 4 6 8 10 ) ! !

!ArrayTest methodsFor: 'testing' stamp: 'marcus.denker 10/22/2008 10:34'!
testPrinting
	self assert: literalArray printString = '#(1 true 3 #four)'.
	self assert: (literalArray = (Compiler evaluate: literalArray printString)).
	self assert: selfEvaluatingArray printString =  '{1. true. (3/4). Color black. (2 to: 4). 5}'.
	self assert: (selfEvaluatingArray = (Compiler evaluate: selfEvaluatingArray printString)).
	self assert: nonSEArray1 printString =  'an Array(1 a Set(1))'.
	self assert: nonSEarray2 printString =  '{#Array->Array}'
! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/27/2009 15:20'!
unsortedCollection
	^unsortedCollection .! !

!ArrayTest methodsFor: 'requirements' stamp: 'delaunay 3/31/2009 16:00'!
valueArray
	^ valueArray ! !

!ArrayTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/9/2008 18:49'!
valuePutIn
	"the value that we will put in the non empty collection"
	
	^ 2! !

CollectionRootTest subclass: #BagTest
	uses: TAddTest + TIncludesTest + TCloneTest + TCopyTest + TSetAritmetic + TRemoveForMultiplenessTest + TConvertTest + TAsStringCommaAndDelimiterTest + TRemoveForMultiplenessTest + TPrintTest
	instanceVariableNames: 'empty nonEmpty result emptyButAllocatedWith20 otherCollection elementExistsTwice element collectionWithElement collectionIn collectionNotIn collectionOfString elementNotIn collectionWithCharacters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

BagTest class
	uses: TAddTest classTrait + TIncludesTest classTrait + TCloneTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait + TRemoveForMultiplenessTest classTrait + TConvertTest classTrait + TAsStringCommaAndDelimiterTest classTrait + TPrintTest classTrait
	instanceVariableNames: ''!

!BagTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:58'!
anotherElementNotIn
	^ 42! !

!BagTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/6/2008 17:22'!
collection

	^ nonEmpty.
	! !

!BagTest methodsFor: 'test - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 22:46'!
collectionClass

	^ Bag! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 12:07'!
collectionInForIncluding
	 ^ collectionIn ! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 12:08'!
collectionNotIncluded
	^ collectionNotIn ! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	^ collectionOfString! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 14:22'!
collectionWithCharacters
	^ collectionWithCharacters .! !

!BagTest methodsFor: 'requirements' stamp: 'sd 1/28/2009 16:32'!
collectionWithElement
	"Returns a collection that already includes what is returned by #element."
	^ collectionWithElement! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:24'!
collectionWithElementsToRemove
	^ collectionIn! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 14:20'!
collectionWithEqualElements
	^ nonEmpty ! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 14:23'!
collectionWithoutEqualElements
	^ otherCollection  ! !

!BagTest methodsFor: 'test - iterate' stamp: 'damienpollet 1/30/2009 17:36'!
doWithoutNumber

	^ 4! !

!BagTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:32'!
element
	^ super element! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 14:19'!
elementInForIncludesTest

	^ self element ! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 13:40'!
elementNotIn

	^elementNotIn ! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 666! !

!BagTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ 42! !

!BagTest methodsFor: 'test - remove' stamp: 'damienpollet 1/30/2009 17:15'!
elementTwiceIn
	^ super elementTwiceIn! !

!BagTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 15:13'!
empty

	^ empty
	
	! !

!BagTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/6/2008 17:26'!
emptyButAllocatedWith20 
	
		^ emptyButAllocatedWith20! !

!BagTest methodsFor: 'test - iterate' stamp: 'marcus.denker 2/20/2009 16:29'!
expectedElementByDetect

	^ -2
	
	! !

!BagTest methodsFor: 'test - iterate' stamp: 'damienpollet 1/30/2009 17:37'!
expectedSizeAfterReject
	^ 2! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:33'!
integerCollectionWithoutEqualElements
	^ otherCollection ! !

!BagTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 15:14'!
nonEmpty

	^ nonEmpty
	
	! !

!BagTest methodsFor: 'requirements' stamp: 'delaunay 4/14/2009 14:35'!
nonEmpty1Element

	^ Bag new add: self element ;yourself.! !

!BagTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 16:15'!
otherCollection
	^ otherCollection! !

!BagTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/6/2008 17:22'!
result

	^ result.
	! !

!BagTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:40'!
selectedNumber
	^ 4! !

!BagTest methodsFor: 'setup' stamp: 'delaunay 4/15/2009 14:38'!
setUp

	empty := Bag new.
	nonEmpty := self speciesClass new
		add: 13;
		add: -2;
		add: self elementTwiceIn;
		add: 10;
		add: self elementTwiceIn;
		add: self element;
		yourself.
	elementNotIn := 0.
	collectionIn:= Bag new add: -2; add: self elementTwiceIn ; add:10;yourself.
	collectionNotIn:= Bag new add: self elementNotIn ; add: 5;yourself.
	collectionOfString := Bag new add: 1.5; add: 5.5; add: 7.5;yourself.
	otherCollection := Bag new add: 1; add: 20 ; add: 30 ; add: 40 ; yourself.
	result := Bag new add: SmallInteger ;  add: SmallInteger ; add: SmallInteger ; add: SmallInteger ; add: SmallInteger; add: SmallInteger;  yourself.
	emptyButAllocatedWith20 := Bag new: 20.
	collectionWithElement := Bag new add: self element; yourself.
	collectionWithCharacters:= Bag new add:$p ; add: $v ; add: $i ; add:$y ; yourself.! !

!BagTest methodsFor: 'setup' stamp: 'luc.fabresse 11/29/2008 22:58'!
sizeCollection

	^ otherCollection.
	! !

!BagTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 17:39'!
speciesClass
	
	^ Bag! !

!BagTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: self collectionWithElementsToRemove
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: self elementToAdd
		raise: Error! !

!BagTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:39'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each class=Float ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:28'!
testAdd
	"self run: #testAdd"
	"self debug: #testAdd"

	| aBag |
	aBag := Bag new.
	aBag add: 'a'.
	aBag add: 'b'.
	
	self assert: aBag size = 2.
	aBag add: 'a'.
	self assert: aBag size = 3.
	self assert: (aBag occurrencesOf: 'a') = 2
	
! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:28'!
testAddWithOccurrences
	"self debug:#testAddWithOccurrences"
	
	| aBag |
 	aBag := Bag new.	
	aBag add: 'a' withOccurrences: 3.
	self assert: (aBag size = 3).
	
	
	
	
		
! !

!BagTest methodsFor: 'as yet unclassified' stamp: 'damienpollet 1/13/2009 15:57'!
testAnySastify

	self assert: ( self collection anySatisfy: [:each | each = self element]).
	self deny: (self collection anySatisfy: [:each | each isString]).! !

!BagTest methodsFor: 'basic tests' stamp: 'TJ 3/8/2006 08:42'!
testAsBag

	| aBag |
 
	aBag := Bag new.	
	
	self assert: aBag asBag = aBag.! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:29'!
testAsSet

	| aBag aSet |
	aBag := Bag new.	
	aBag add:'a' withOccurrences: 4.
	aBag add:'b' withOccurrences: 2.
	aSet := aBag asSet.
	self assert: aSet size = 2.
	self assert: (aSet occurrencesOf: 'a') = 1 
	! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:30'!
testCopy
	"self run: #testCopy"
	
	| aBag newBag |
	aBag := Bag new.
	aBag add:'a' withOccurrences: 4.
	aBag add:'b' withOccurrences: 2.
	newBag := aBag copy.
	self assert: newBag = newBag.
	self assert: newBag asSet size = 2.! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!BagTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:56'!
testCreation
	"self run: #testCreation"
	"self debug: #testCreation"
		
	| bag |
	bag := Bag new.
	self assert: (bag size) = 0.
	self assert: (bag isEmpty).
	
! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 10:05'!
testCumulativeCounts
	"self run: #testCumulativeCounts"
	"self debug: #testCumulativeCounts"
		
	| bag cumulativeCounts |
	bag := Bag new.
	bag add: '1' withOccurrences: 50.
	bag add: '2' withOccurrences: 40.
	bag add: '3' withOccurrences: 10.
	
	cumulativeCounts := bag cumulativeCounts.
	
	self assert: cumulativeCounts size = 3.
	self assert: cumulativeCounts first = (50 -> '1').
	self assert: cumulativeCounts second = (90 -> '2').
	self assert: cumulativeCounts third = (100 -> '3').
! !

!BagTest methodsFor: 'tests' stamp: 'EP 3/8/2006 08:39'!
testEqual
	"(self run: #testEqual)"
	"(self debug: #testEqual)"
	| bag1 bag2 |
	bag1 := Bag new.
	bag2 := Bag new.
	self assert: bag1 = bag2.
	bag1 add: #a;
		 add: #b.
	bag2 add: #a;
		 add: #a.
	self deny: bag1 = bag2.
	self assert: bag1 = bag1.
	bag1 add: #a.
	bag2 add: #b.
	self assert: bag1 = bag2.
	bag1 add: #c.
	self deny: bag1 = bag2.
	bag2 add: #c.
	self assert: bag1 = bag2! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!BagTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!BagTest methodsFor: 'basic tests' stamp: 'sd 3/21/2006 22:32'!
testOccurrencesOf
	"self debug: #testOccurrencesOf"

	| aBag |
 	aBag := Bag new.	
	aBag add: 'a' withOccurrences: 3.
	aBag add: 'b'.
	aBag add: 'b'.
	aBag add: 'b'.
	aBag add: 'b'.	
	self assert: (aBag occurrencesOf:'a') = 3.
	self assert: (aBag occurrencesOf:'b') = 4.
	self assert: (aBag occurrencesOf:'c') = 0.
	self assert: (aBag occurrencesOf: nil) =0.
	aBag add: nil.
	self assert: (aBag occurrencesOf: nil) =1.
	! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:57'!
testRemove
	"self run: #testRemove"
	"self debug: #testRemove"
	
	| bag item |
	item := 'test item'.
	bag := Bag new.
	
	bag add: item.
	self assert: (bag size) = 1.
	bag remove: item.
	self assert: bag isEmpty.
	
	bag add: item withOccurrences: 2.
	bag remove: item.
	bag remove: item.
	self assert: (bag size) = 0.
	
	self should: [bag remove: item.] raise: Error.! !

!BagTest methodsFor: 'test - remove' stamp: 'damienpollet 1/30/2009 17:07'!
testRemoveElementThatExistsTwice
	"self debug: #testRemoveElementThatDoesExistsTwice"
	| size |
	size := self nonEmpty size.
	self assert: (self nonEmpty includes: self elementTwiceIn).
	self nonEmpty remove: self elementTwiceIn.
	self assert: size - 1 = self nonEmpty size! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:48'!
testSortedCounts
	"self run: #testSortedCounts"
	"self debug: #testSortedCounts"
	
	| bag sortedCounts|
	bag := Bag new.
	bag add: '1' withOccurrences: 10.
	bag add: '2' withOccurrences: 1.
	bag add: '3' withOccurrences: 5.
	
	sortedCounts := bag sortedCounts.
	self assert: sortedCounts size = 3.
	
	self assert: sortedCounts first = (10->'1').
	self assert: sortedCounts second =  (5->'3').
	self assert: sortedCounts third =  (1->'2').	! !

!BagTest methodsFor: 'tests' stamp: 'EP 2/28/2006 09:48'!
testSortedElements
	"self run: #testSortedElements"
	"self debug: #testSortedElements"
	
	| bag sortedElements|
	bag := Bag new.
	bag add: '2' withOccurrences: 1.
	bag add: '1' withOccurrences: 10.
	bag add: '3' withOccurrences: 5.
	
	sortedElements := bag sortedElements.
	
	self assert: sortedElements size = 3.
	
	self assert: sortedElements first = ('1'->10).
	self assert: sortedElements second =  ('2'->1).
	self assert: sortedElements third =  ('3'->5).
	! !

BagTest subclass: #IdentityBagTest
	instanceVariableNames: 'equalNotIdenticalElement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!IdentityBagTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 15:49'!
collection
	^ super collection! !

!IdentityBagTest methodsFor: 'requirements' stamp: 'damienpollet 1/9/2009 18:37'!
element
	^ element ifNil: [ element := 'a copiable value' ]! !

!IdentityBagTest methodsFor: 'requirements' stamp: 'damienpollet 1/9/2009 18:37'!
equalNotIdenticalElement
	^ equalNotIdenticalElement ifNil: [ equalNotIdenticalElement := element copy ]! !

!IdentityBagTest methodsFor: 'requirements' stamp: 'damienpollet 1/9/2009 18:28'!
speciesClass
	^ IdentityBag! !

!IdentityBagTest methodsFor: 'tests' stamp: 'EP 3/8/2006 09:50'!
testIdentity
	"self run:#testIdentity"
	"self debug:#testIdentity"
	| bag identityBag aString anOtherString |
	
	aString := 'hello'.
	anOtherString := aString copy.
	
	self assert: (aString = anOtherString).
	self assert: (aString == anOtherString) not.

	bag := Bag new.
	bag add: aString.
	bag add: aString.
	bag add: anOtherString.
	self assert: (bag occurrencesOf: aString) = 3.
	self assert: (bag occurrencesOf: anOtherString) = 3.
	
	identityBag := IdentityBag new.
	identityBag add: aString.
	identityBag add: aString.
	identityBag add: anOtherString.
	
	self assert: (identityBag occurrencesOf: aString) = 2.
	self assert: (identityBag occurrencesOf: anOtherString) = 1.



! !

!CollectionRootTest class methodsFor: 'as yet unclassified' stamp: 'damienpollet 1/13/2009 15:28'!
isAbstract

	^ self name = #CollectionRootTest! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:20'!
doWithoutNumber

	^ 2! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:32'!
element
	^ 3! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ self explicitRequirement! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'damienpollet 1/30/2009 17:14'!
elementTwiceIn
	^ 1 "12332312322"! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'damienpollet 1/21/2009 18:25'!
expectedElementByDetect

	^ -2! !

!CollectionRootTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 11:32'!
nonEmpty

	^ self explicitRequirement! !

!CollectionRootTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 13:48'!
testDoSeparatedBy
	| string expectedString beforeFirst |
	string := ''.
	self collection 
		do: [ :each | string := string , each asString ]
		separatedBy: [ string := string , '|' ].
	expectedString := ''.
	beforeFirst := true.
	self collection do: 
		[ :each | 
		beforeFirst = true 
			ifTrue: [ beforeFirst := false ]
			ifFalse: [ expectedString := expectedString , '|' ].
		expectedString := expectedString , each asString ].
	self assert: expectedString = string! !

!CollectionRootTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 11:53'!
testRejectNoReject
	| res |
	res := self collection reject: [ :each | each = self elementNotInForOccurrences ].
	self assert: res size = self collection size! !

CollectionRootTest subclass: #DictionaryTest
	uses: TIncludesTest + TDictionaryAddingTest + TDictionaryAccessingTest + TDictionaryComparingTest + TDictionaryCopyingTest + TDictionaryEnumeratingTest + TDictionaryImplementationTest + TDictionaryPrintingTest + TDictionaryRemovingTest + TPutTest - {#testAtPutOutOfBounds}
	instanceVariableNames: 'emptyDict nonEmptyDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

DictionaryTest class
	uses: TIncludesTest classTrait + TDictionaryAddingTest classTrait + TDictionaryAccessingTest classTrait + TDictionaryComparingTest classTrait + TDictionaryCopyingTest classTrait + TDictionaryEnumeratingTest classTrait + TDictionaryImplementationTest classTrait + TDictionaryPrintingTest classTrait + TDictionaryRemovingTest classTrait + TPutTest classTrait
	instanceVariableNames: 'testToto pt1'!

DictionaryTest class
	uses: TIncludesTest classTrait + TDictionaryAddingTest classTrait + TDictionaryAccessingTest classTrait + TDictionaryComparingTest classTrait + TDictionaryCopyingTest classTrait + TDictionaryEnumeratingTest classTrait + TDictionaryImplementationTest classTrait + TDictionaryPrintingTest classTrait + TDictionaryRemovingTest classTrait + TPutTest classTrait
	instanceVariableNames: 'testToto pt1'!

!DictionaryTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:24'!
aValue

	^ 33! !

!DictionaryTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:25'!
anIndex

	^ #GG! !

!DictionaryTest methodsFor: 'requirement' stamp: 'stephane.ducasse 11/21/2008 15:05'!
anotherElementNotIn
	^ 42! !

!DictionaryTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:25'!
anotherValue

	^ 66! !

!DictionaryTest methodsFor: 'setup' stamp: 'AlexandreBergel 1/14/2009 15:14'!
classToBeTested 

	^ Dictionary! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/13/2009 16:15'!
collection
	^ self nonEmptyDict! !

!DictionaryTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 10:50'!
collectionInForIncluding
	self explicitRequirement! !

!DictionaryTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	^ self explicitRequirement! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/30/2009 17:30'!
element
	^ 30! !

!DictionaryTest methodsFor: 'requirement' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 666! !

!DictionaryTest methodsFor: 'requirement' stamp: 'stephane.ducasse 11/21/2008 15:04'!
empty
	^ emptyDict! !

!DictionaryTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 15:06'!
emptyDict
	^ emptyDict! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/21/2009 18:22'!
expectedElementByDetect
	^ 30! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/30/2009 17:44'!
expectedSizeAfterReject
	self flag: 'what should this return?'! !

!DictionaryTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 15:09'!
newEmptyDict
	^ self emptyDict copy! !

!DictionaryTest methodsFor: 'requirement' stamp: 'stephane.ducasse 11/21/2008 15:04'!
nonEmpty
	^ nonEmptyDict! !

!DictionaryTest methodsFor: 'requirement' stamp: 'AlexandreBergel 1/6/2009 15:06'!
nonEmptyDict
	^ nonEmptyDict ! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/13/2009 16:35'!
result
	^ Dictionary newFromPairs: {
		#a . SmallInteger .
		#b . SmallInteger .
		#c . SmallInteger .
		#d . SmallInteger }! !

!DictionaryTest methodsFor: 'setup' stamp: 'damienpollet 1/30/2009 17:45'!
setUp

	emptyDict := self classToBeTested new.
	nonEmptyDict := self classToBeTested new.
	nonEmptyDict
		at: #a put: self elementTwiceIn;
		at: #b put: 30; 
		at: #c put: self elementTwiceIn;
		at: #d put: -2.! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/13/2009 16:55'!
sizeCollection
	^ nonEmptyDict! !

!DictionaryTest methodsFor: 'requirement' stamp: 'damienpollet 1/21/2009 18:04'!
speciesClass
	^ Dictionary! !

!DictionaryTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:07'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each isString ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'AlexandreBergel 1/6/2009 11:56'!
testHasBindingThatBeginsWith
	| newDict |
	newDict := Dictionary new at: #abc put: 10; at: #abcd put: 100; at: #def put: 20; yourself.
	self assert: (newDict hasBindingThatBeginsWith: 'ab').
	self assert: (newDict hasBindingThatBeginsWith: 'def').
	self deny: (newDict hasBindingThatBeginsWith: 'defg').! !

!DictionaryTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'damienpollet 1/30/2009 17:55'!
testIncludeAssociation
	self assert: (nonEmptyDict includesAssociation: #a -> self elementTwiceIn).
	self assert: (nonEmptyDict includesAssociation: (nonEmptyDict associations first)).
! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'damienpollet 1/30/2009 17:57'!
testIncludes
	| o1 o2 newDict |
	self assert: (nonEmptyDict includes: self element).
	
	o1 := 2 @ 3.
	o2 := 2 @ 3.
	self deny: (o1 == o2).
	self assert: (o1 = o2).
	newDict := Dictionary new.
	newDict at: #a put: o1.
	
	self assert: (newDict includes: o2).
! !

!DictionaryTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!DictionaryTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'stephane.ducasse 11/21/2008 15:00'!
testIncludesAssociation
	"self run:#testIncludesAssociation"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	self assert: (dict includesAssociation: (#a -> 1)).
	self assert: (dict includesAssociation: (#b -> 2)).
	
	! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'ndCollectionsTests-Unordered 3/16/2006 10:25'!
testIncludesAssociationNoValue
	"self run:#testIncludesAssociationNoValue"
	"self debug:#testIncludesAssociationNoValue"
	
	| dict a1 a3 |
	a1 := Association key: #Italie.
	a3 := Association key: #France value: 'Paris'.
	
	self assert: (a1 key = #Italie).
	self assert: (a1 value isNil).
	
	dict := Dictionary new.
	dict add: a1.
	dict add: a3.
	self assert: (dict includesKey: #France).
	self assert: (dict includesKey: #Italie).
	self assert: (dict at: #Italie) isNil.
	self assert: (dict at: #France) = 'Paris'

	
	
	! !

!DictionaryTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'damienpollet 1/30/2009 17:55'!
testIncludesIdentity
	| o1 o2 newDict |
	self assert: (nonEmptyDict includesIdentity: self element).
	
	o1 := 2 @ 3.
	o2 := 2 @ 3.
	self deny: (o1 == o2).
	self assert: (o1 = o2).
	newDict := Dictionary new.
	newDict at: #a put: o1.
	
	self deny: (newDict includesIdentity: o2).
! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'AlexandreBergel 1/6/2009 13:44'!
testIncludesKey
	self assert: (nonEmptyDict includesKey: #a).
	self assert: (nonEmptyDict includesKey: #b).
	self deny: (nonEmptyDict includesKey: #zork).
	self deny: (nonEmptyDict includesKey: nil).
! !

!DictionaryTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'AlexandreBergel 1/6/2009 13:48'!
testIsDictionary
	self deny: Object new isDictionary.
	self assert: nonEmptyDict isDictionary.
	self assert: emptyDict isDictionary.! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'damienpollet 1/30/2009 17:57'!
testKeyForIdentity
	self assert: (nonEmptyDict keyForIdentity: 30) = #b.

	"The value 20 is associated to two different associations"
	self assert: (#(a c) includes: (nonEmptyDict keyForIdentity: self elementTwiceIn))! !

!DictionaryTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/14/2009 15:15'!
testNew
	| d |
	d := self classToBeTested new: 10.
	self assert: d size = 0.
	
	"Why 14? Mysterious"
	self assert: d capacity = 14! !

!DictionaryTest methodsFor: 'test - testing' stamp: 'NDCC 3/8/2006 09:41'!
testOccurrencesOf
	"self run:#testOccurrencesOf"
	
	| dict |
	dict := Dictionary new.
	dict at: #a put: 1.
	dict at: #b put: 2.
	dict at: #c put: 1.
	dict at: #d put: 3.
	dict at: nil put: nil.
	dict at: #z put: nil.
	
	
	self assert: (dict occurrencesOf: 1 ) = 2.
	self assert: (dict occurrencesOf: nil ) = 2.
	
	
	
	! !

DictionaryTest subclass: #SmallDictionaryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!SmallDictionaryTest methodsFor: 'as yet unclassified' stamp: 'stephane.ducasse 1/12/2009 17:22'!
classUnderTest

	^ SmallDictionary! !

CollectionRootTest subclass: #IntervalTest
	uses: TCloneTest + TIncludesTest + TIterateSequenceableTest + TSequencedElementAccessTest + TIterateSequencedReadableTest + TSequencedConcatanationTest + TSubCollectionAccess + TAsStringCommaAndDelimiterSequenceableTest + TIndexAccess + TPrintOnSequencedTest + TConvertTest + TCopySequenceableWithReplacement - {#testCopyReplaceAllWithManyOccurence. #collectionWith2TimeSubcollection} + TCopySequenceableWithOrWithoutSpecificElements + TCopySequenceableSameContents - {#testShuffled} + TCopyPartOfSequenceable - {#testCopyEmptyMethod} + TCopyTest + TBeginsEndsWith
	instanceVariableNames: 'empty nonEmpty one elementIn elementNotIn subCollectionNotIn collectionOfFloat anotherCollection nonEmpty1Element subCollection collectionWithSubCollection subCollectionInNonEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

IntervalTest class
	uses: TCloneTest classTrait + TIncludesTest classTrait + TIterateSequenceableTest classTrait + TSequencedElementAccessTest classTrait + TIterateSequencedReadableTest classTrait + TSequencedConcatanationTest classTrait + TSubCollectionAccess classTrait + TAsStringCommaAndDelimiterSequenceableTest classTrait + TIndexAccess classTrait + TPrintOnSequencedTest classTrait + TConvertTest classTrait + TCopySequenceableWithReplacement classTrait + TCopySequenceableWithOrWithoutSpecificElements classTrait + TCopySequenceableSameContents classTrait + TCopyPartOfSequenceable classTrait + TCopyTest classTrait + TBeginsEndsWith classTrait
	instanceVariableNames: ''!

!IntervalTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 17:21'!
accessCollection
	^ -2 to: 14 by: 4! !

!IntervalTest methodsFor: 'requirements' stamp: 'stephane.ducasse 11/21/2008 15:39'!
anotherElementNotIn
	^ 42! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 18:55'!
collection
	^ nonEmpty! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 16:36'!
collectionClass

	^ Interval! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:51'!
collectionInForIncluding
	^ nonEmpty copyWithout: (self nonEmpty last).! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:35'!
collectionNotIncluded
	^ (nonEmpty last + 1) to: (nonEmpty last +5)! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:10'!
collectionOfFloat
	^collectionOfFloat ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 14:44'!
collectionWith1TimeSubcollection
	^ collectionWithSubCollection ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:32'!
collectionWithElementsToRemove
	
	^ subCollectionInNonEmpty .! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 15:39'!
collectionWithoutEqualElements
	^ nonEmpty ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:13'!
collectionWithoutEqualsElements
	^ nonEmpty ! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 18:59'!
doWithoutNumber

	^ 6! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:22'!
elementInCollectionOfFloat
	^ collectionOfFloat anyOne! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:40'!
elementInForElementAccessing
" return an element inculded in 'accessCollection '"
	^ self accessCollection anyOne! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:47'!
elementInForIncludesTest

	^ elementIn ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 15:03'!
elementInForIndexAccess
	^ self accessCollection  anyOne! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:48'!
elementInForIndexAccessing

	^ elementIn ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:48'!
elementNotIn
	^elementNotIn! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:43'!
elementNotInForElementAccessing
" return an element not included in 'accessCollection' "
	^ elementNotIn ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:48'!
elementNotInForIndexAccessing

	^elementNotIn ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 9! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:27'!
elementToAdd
	^ elementNotIn ! !

!IntervalTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/5/2008 13:08'!
empty

	^ empty
! !

!IntervalTest methodsFor: 'requirements' stamp: 'marcus.denker 2/20/2009 16:30'!
expectedElementByDetect
	"Returns the first even element of #collection"
	^ -2
	
	! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 19:01'!
expectedSizeAfterReject
	"Number of even elements in #collection"
	^ 3! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:55'!
firstCollection
	^ nonEmpty.! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 18:57'!
firstEven
	"Returns the first even number of #collection"
	^ -2! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 18:58'!
firstOdd
	"Returns the first odd number of #collection"
	^ -5! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:14'!
indexInForCollectionWithoutDuplicates
	^ 2.! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:02'!
indexInNonEmpty
	^2.! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:04'!
integerCollection
	^ nonEmpty ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:31'!
integerCollectionWithoutEqualElements
	^ 1 to: 23.! !

!IntervalTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/5/2008 13:08'!
nonEmpty

	^ nonEmpty! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 15:00'!
nonEmpty1Element

	^ nonEmpty1Element  ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:46'!
nonEmptyMoreThan1Element
	^nonEmpty .! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 14:44'!
oldSubCollection
	^ subCollection ! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 14:45'!
replacementCollection
	^ 5 to: 7.! !

!IntervalTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 18:57'!
result
	^ {SmallInteger . SmallInteger . SmallInteger . SmallInteger . SmallInteger . SmallInteger}! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:56'!
secondCollection
	^anotherCollection ! !

!IntervalTest methodsFor: 'setup' stamp: 'delaunay 4/17/2009 15:35'!
setUp

	empty := (1 to: 0).
	one := (1 to:1).
	nonEmpty := -5 to: 10 by: 3.
	subCollectionInNonEmpty := -2 to: 4 by: 3.
	nonEmpty1Element:= 1to:1.
	anotherCollection:= 2 to: 15.
	elementIn:=-2.
	elementNotIn:= 12.
	subCollectionNotIn:= -2 to: 1.
	collectionOfFloat := 1.5 to: 7.5 by: 1.
	subCollection := 2 to: 8.
	collectionWithSubCollection := 1 to: 10.! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:07'!
sizeCollection
	"Answers a collection whose #size is 4"
	^ 1 to: 4.! !

!IntervalTest methodsFor: 'requirements' stamp: 'damiencassou 1/27/2009 17:25'!
speciesClass
	
	^ Array! !

!IntervalTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 09:52'!
subCollectionNotIn

	^subCollectionNotIn ! !

!IntervalTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: [ self collectionWithElementsToRemove ]
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: [ self elementToAdd ]
		raise: Error.
	self 
		shouldnt: [ self collectionNotIncluded ]
		raise: Error.
	self collectionNotIncluded do: [ :each | self deny: (self nonEmpty includes: each) ]! !

!IntervalTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:14'!
test0FixtureIncludeTest
	| elementInB |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementInB := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementInB := false ].
	self assert: elementInB = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementInB := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementInB := false ].
	self assert: elementInB = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: (each class=Float) ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementInB := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementInB := false ].
	self assert: elementInB = true! !

!IntervalTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:07'!
test0IndexAccessingTest
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self assert: self accessCollection size = 5.
	self 
		shouldnt: [ self subCollectionNotIn ]
		raise: Error.
	self subCollectionNotIn 
		detect: [ :each | (self accessCollection includes: each) not ]
		ifNone: [ self assert: false ].
	self 
		shouldnt: [ self elementNotInForIndexAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForIndexAccessing).
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self deny: each class = SmallInteger ]! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:23'!
testAdd
	self assert: (1 to: 10)
			+ 5
			= (6 to: 15)! !

!IntervalTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/6/2009 15:24'!
testAllButFirstElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButFirst: 2.
	1 
		to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i + 2) ].
	self assert: abf size + 2 = col size! !

!IntervalTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/6/2009 15:25'!
testAllButLastElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButLast: 2.
	1 
		to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i) ].
	self assert: abf size + 2 = col size! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 3/27/2008 00:24'!
testAsInterval
	"This is the same as newFrom:"

	self shouldnt: [
		self assert: (#(1 2 3) as: Interval) = (1 to: 3).
		self assert: (#(33 5 -23) as: Interval) = (33 to: -23 by: -28).
		self assert: (#(2 4 6) asByteArray as: Interval) = (2 to: 6 by: 2).
	] raise: Error.

	self should: [#(33 5 -22) as: Interval]
		raise: Error
		description: 'This is not an arithmetic progression'
! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:15'!
testAt
	self assert: ((1 to: 10)
			at: 1)
			= 1.
	self assert: ((1 to: 10)
			at: 3)
			= 3.
	self assert: ((1 to: 10 by: 2)
			at: 1)
			= 1.
	self assert: ((1 to: 10 by: 2)
			at: 3)
			= 5! !

!IntervalTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/10/2009 16:20'!
testAtWrap
	"self debug: #testAt"
	"
	self assert: (self accessCollection at: 1) = 1.
	self assert: (self accessCollection at: 2) = 2.
	"
	| index |
	index := self accessCollection indexOf: self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index + self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index - self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: 1 + self accessCollection size) = (self accessCollection at: 1)! !

!IntervalTest methodsFor: 'tests' stamp: 'stephane.ducasse 1/16/2009 21:06'!
testCollectThenSelectLocal
	| letters vowels |
	letters := 'abcdefghijklmnopqrstuvwxyz'.
	vowels := (1 to: 26) collect: [:index | letters at: index] thenSelect: [:char | char isVowel].
	self assert: (vowels hasEqualElements: 'aeiou').! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!IntervalTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:34'!
testDo
	| s i |
	s := OrderedCollection new.
	i := (10 to: 20).
	i do: [ :each | s addLast: each].
	self assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals

	self shouldnt: [
		self assert: (3 to: 5) = #(3 4 5).
		self deny: (3 to: 5) = #(3 5).
		self deny: (3 to: 5) = #().

		self assert: #(3 4 5) = (3 to: 5).
		self deny: #(3 5) = (3 to: 5).
		self deny: #() = (3 to: 5).
	] raise: MessageNotUnderstood.! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals2

	self assert: (3 to: 5) = #(3 4 5).
	self deny: (3 to: 5) = #(3 5).
	self deny: (3 to: 5) = #().

	self assert: #(3 4 5) = (3 to: 5).
	self deny: #(3 5) = (3 to: 5).
	self deny: #() = (3 to: 5).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals3

	self assert: (3 to: 5 by: 2) first = (3 to: 6 by: 2) first.
	self assert: (3 to: 5 by: 2) last = (3 to: 6 by: 2) last.
	self assert: (3 to: 5 by: 2) = (3 to: 6 by: 2).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:13'!
testEquals4

	self assert: (3 to: 5 by: 2) = #(3 5).
	self deny: (3 to: 5 by: 2) = #(3 4 5).
	self deny: (3 to: 5 by: 2) = #().

	self assert: #(3 5) = (3 to: 5 by: 2).
	self deny: #(3 4 5) = (3 to: 5 by: 2).
	self deny: #() = (3 to: 5 by: 2).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:14'!
testEquals5

	self assert: (3 to: 5 by: 2) = (Heap withAll: #(3 5)).
	self deny: (3 to: 5 by: 2) = (Heap withAll: #(3 4 5)).
	self deny: (3 to: 5 by: 2) = Heap new.

	self assert: (Heap withAll: #(3 5)) = (3 to: 5 by: 2).
	self deny: (Heap withAll: #(3 4 5)) = (3 to: 5 by: 2).
	self deny: Heap new = (3 to: 5 by: 2).! !

!IntervalTest methodsFor: 'tests' stamp: 'md 10/12/2003 20:14'!
testEquals6

	self assert: #() = Heap new.
	self assert: #(3 5) = (Heap withAll: #(3 5)).
	self deny: (3 to: 5 by: 2) = (Heap withAll: #(3 4 5)).
	self deny: (3 to: 5 by: 2) = Heap new.

	self assert: Heap new = #().
	self assert: (Heap withAll: #(3 5)) = #(3 5).
	self deny: (Heap withAll: #(3 4 5)) = #(3 5).
	self deny: Heap new = #(3 5).! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:08'!
testExtent
	self assert: (1 to: 10) extent = 9.
	self assert: (1 to: 10 by: 2) extent = 9.
	self assert: (1 to:-1) extent = -2! !

!IntervalTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!IntervalTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!IntervalTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!IntervalTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!IntervalTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 4/29/2007 21:34'!
testInclusionBug1603
	"This test is by german morales.
	It is about mantis bug 1603"
	
	self shouldnt: ((1 to: 5 by: 1) includes: 2.5). "obvious"
	self shouldnt: ((100000000000000 to: 500000000000000 by: 100000000000000)
 		  includes: 250000000000000). "same as above with 14 zeros appended"! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 2/3/2008 21:43'!
testInclusionBug6455
	"This test is about mantis bug http://bugs.squeak.org/view.php?id=6455
	It should work as long as Fuzzy inclusion test feature for Interval of Float is maintained.
	This is a case when tested element is near ones of actual value, but by default.
	Code used to work only in the case of close numbers by excess..."
	
	self assert: ((0 to: Float pi by: Float pi / 100) includes: Float pi * (3/100))! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 4/29/2007 21:35'!
testIndexOfBug1602
	"This test is by german morales.
	It is about mantis bug 1602"
	
	self should: ((1 to: 5 by: 1) indexOf: 2.5) = 0. "obvious"
	self should: ((100000000000000 to: 500000000000000 by: 100000000000000)
 		  indexOf: 250000000000000) = 0. "same as above with 14 zeros appended"! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 2/3/2008 21:35'!
testIndexOfBug6455
	"This test is about mantis bug http://bugs.squeak.org/view.php?id=6455
	It should work as long as Fuzzy inclusion test feature for Interval of Float is maintained.
	This is a case when tested element is near ones of actual value, but by default.
	Code used to work only in the case of close numbers by excess..."
	
	self assert: ((0 to: Float pi by: Float pi / 100) indexOf: Float pi * (3/100)) = 4! !

!IntervalTest methodsFor: 'tests' stamp: 'nice 4/30/2007 18:40'!
testInfiniteLoopBug6456
	"This is a non regression test against mantis bug #6456.
	Some Float interval size was not consistent with do: loop.
	Some Float Interval used to do: infinite loops"
	
	| x interval counter size |
	x := (1.0 timesTwoPower: 53). "Note: x+1 = x due to inexact arithmetic"
	interval := x to: x+4.
	size := interval size.
	counter := 0.
	interval do: [:each | self should: (counter := counter + 1) <= size].! !

!IntervalTest methodsFor: 'tests' stamp: ''!
testInvalid
	"empty, impossible ranges"
	self assert: (1 to: 0) = #().
	self assert: (1 to: -1) = #().
	self assert: (-1 to: -2) = #().
	self assert: (1 to: 5 by: -1) = #().
	
	"always contains only start value."
	self assert: (1 to: 1) = #(1).
	self assert: (1 to: 5 by: 10) = #(1).
	self assert: (1 to: 0 by: -2) = #(1).
! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:11'!
testIsEvaluating
	self assert: (1 to: 10) isSelfEvaluating.
	self assert: (1 to: 10 by: 2) isSelfEvaluating! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/7/2005 13:29'!
testIsInterval
	self assert: (1 to: 10) isInterval.
	self assert: (1 to: 10 by: 2) isInterval! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/7/2005 13:29'!
testLast

self assert: (1 to:10) last = 10.
self assert: (1 to:10 by:2) last = 9 ! !

!IntervalTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/9/2009 16:25'!
testLastIndexOfStartingAt
	"self debug: #testLastIndexOf"
	| collection |
	collection := self accessCollection copyWith: (self accessCollection at: 1).
	self assert: (collection lastIndexOf: (self accessCollection at: 1)) = collection size.
	self assert: (collection lastIndexOf: self elementNotInForIndexAccessing) = 0! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:28'!
testMinus
	self assert: (1 to: 10)
			- 5
			= (-4 to: 5)! !

!IntervalTest methodsFor: 'tests' stamp: 'md 1/14/2004 11:43'!
testNewFrom

	self shouldnt: [
		 self assert: ( (Interval newFrom: (1 to: 1)) = (1 to: 1)).
		 self assert: ( (Interval newFrom: #(1)) = (1 to: 1)).
		 self assert: ( (Interval newFrom: #()) =  ( 1 to: 0)) .
	] raise: Error.! !

!IntervalTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:17'!
testNumericTypes

	(3 asNumber) to: 5 = #(3 4 5).
	
	3.0 to: 5.0 = #(3.0 4.0 5.0).
	3.0 to: 5.0 by: 0.5 = #(3.0 3.5 4.0 4.5 5.0).
	
	3/1 to: 5/1 = #(3 4 5).
	1/2 to: 5/2 by: 1/2 = #(1/2 1 3/2 2 5/2).! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:03'!
testOtherNewFrom

	self assert: (Interval newFrom: #(1 2 3 )) = (1 to: 3).
	self assert: (Interval newFrom: #(33  5 -23 )) = (33 to: -23 by: -28).
	self should: [(Interval newFrom: #(33  5 -22 ))] raise: Error.
	self assert: (#(33  5 -23) as: Interval) = (33 to: -23 by: -28).
	self should: [( #(33  5 -22 ) as: Interval)] raise: Error.
	
	self assert: ( (-4 to: -12 by: -1) as: Interval) = (-4 to: -12 by: -1).
	self assert: ( Interval newFrom: (1 to: 1)) = (1 to: 1).
	self assert: ( Interval newFrom: (1 to: 0)) = (1 to: 0).
	self assert: (#(1) as: Interval) = (1 to: 1).
	self assert: (#() as: Interval) = (1 to: 0).! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:03'!
testPermutationsDo

	| i oc |
	i := (1.234 to: 4.234).
	oc := OrderedCollection new.
	i permutationsDo: [:e | oc add: e].
	self assert: (oc size == i size factorial).
	^ oc! !

!IntervalTest methodsFor: 'tests' stamp: 'zz 12/5/2005 18:20'!
testRangeIncludes
	self
		assert: ((1 to: 10)
				rangeIncludes: 3).
	self
		assert: ((1 to: 10 by: 2)
				rangeIncludes: 3).
	self
		deny: ((1 to: 10)
				rangeIncludes: 0).
	self
		deny: ((1 to: 10)
				rangeIncludes: 11).
	self
		deny: ((1 to: 10 by: 2)
				rangeIncludes: 0).
	self
		deny: ((1 to: 10 by: 2)
				rangeIncludes: 11)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:35'!
testReverseDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20.
	i
		reverseDo: [:each | s addFirst: each].
	self
		assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:36'!
testReverseUnevenDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20 by: 3.
	i
		reverseDo: [:each | s addFirst: each].
	self
		assert: (s hasEqualElements: i)! !

!IntervalTest methodsFor: 'tests' stamp: 'apb 4/22/2007 12:39'!
testUnevenDo
	| s i |
	s := OrderedCollection new.
	i := 10 to: 20 by: 3.
	i
		do: [:each | s addLast: each].
	self
		assert: (s hasEqualElements: i)! !

CollectionRootTest subclass: #OrderedCollectionTest
	uses: TEmptySequenceableTest + TAddTest + TSequencedElementAccessTest + TIncludesTest + TCloneTest + TSetAritmetic + TRemoveForMultiplenessTest + TCreationWithTest + TCopyTest + TPutTest + TIterateSequencedReadableTest + TSubCollectionAccess + TIndexAccess + TCopySequenceableWithReplacement + TCopyPartOfSequenceable + TCopySequenceableSameContents + TCopySequenceableWithOrWithoutSpecificElements - {#testForceToPaddingWith. #testForceToPaddingStartWith} + TPrintOnSequencedTest + TAsStringCommaAndDelimiterSequenceableTest + TConvertTest + TConvertAsSetForMultiplinessTest + TSequencedConcatanationTest + TBeginsEndsWith + TReplacementSequencedOldTest + TIndexAccessForMultipliness + TCopyPartOfSequenceableForMultipliness
	instanceVariableNames: 'empty nonEmpty collection result emptyButAllocatedWith20 otherCollection indexCollection elementExistsTwice collectionWithElement collectionOfFloat elementNotIn indexArray withoutEqualElements floatCollectionWithSameBeginingEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!OrderedCollectionTest commentStamp: 'BG 1/10/2004 22:07' prior: 0!
These test cases demonstrate addition of items into an OrderedCollection as well as item removal.

Some of the assertions are quite complicated and use a lot of collection protocol. Such methods do not test one single method, but protocol in general.!

OrderedCollectionTest class
	uses: TEmptySequenceableTest classTrait + TAddTest classTrait + TSequencedElementAccessTest classTrait + TIncludesTest classTrait + TCloneTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait + TCreationWithTest classTrait + TRemoveForMultiplenessTest classTrait + TPutTest classTrait + TIterateSequencedReadableTest classTrait + TSubCollectionAccess classTrait + TIndexAccess classTrait + TCopySequenceableWithReplacement classTrait + TCopyPartOfSequenceable classTrait + TCopySequenceableSameContents classTrait + TCopySequenceableWithOrWithoutSpecificElements classTrait + TPrintOnSequencedTest classTrait + TAsStringCommaAndDelimiterSequenceableTest classTrait + TConvertTest classTrait + TConvertAsSetForMultiplinessTest classTrait + TSequencedConcatanationTest classTrait + TBeginsEndsWith classTrait + TReplacementSequencedOldTest classTrait + TIndexAccessForMultipliness classTrait + TCopyPartOfSequenceableForMultipliness classTrait
	instanceVariableNames: ''!

!OrderedCollectionTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:13'!
aValue

	^ 33! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/30/2008 19:03'!
accessCollection

	^ indexCollection! !

!OrderedCollectionTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:13'!
anIndex

	^ 2! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:57'!
anotherElementNotIn
	^ 42! !

!OrderedCollectionTest methodsFor: 'helpers' stamp: 'stephane.ducasse 1/12/2009 17:19'!
anotherValue

	^ 66! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 15:53'!
collection

	^ collection! !

!OrderedCollectionTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 18:31'!
collectionClass 
	
	^ OrderedCollection! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:00'!
collectionInForIncluding
	^ self nonEmpty copyWithoutFirst.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:26'!
collectionNotIncluded
	^ OrderedCollection new add: elementNotIn ; add: elementNotIn ; yourself.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 10:56'!
collectionOfFloat
	^ collectionOfFloat ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:40'!
collectionWith1TimeSubcollection
" return a collection including 'oldSubCollection'  only one time "
	^ ((OrderedCollection new add: elementNotIn; yourself),self oldSubCollection) add: elementNotIn;yourself   ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:27'!
collectionWith2TimeSubcollection
" return a collection including 'oldSubCollection'  two or many time "
	^ (((OrderedCollection new add: elementNotIn; yourself),self oldSubCollection ) add: elementNotIn;yourself),self  oldSubCollection! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:21'!
collectionWith5Elements
" return a collection of size 5 including 5 elements"
^ indexCollection ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'sd 1/28/2009 16:29'!
collectionWithElement
	"Returns a collection that already includes what is returned by #element."
	^ collectionWithElement! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:22'!
collectionWithElementsToRemove
	^ nonEmpty copyWithoutFirst.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 11:14'!
collectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals .
(others elements of the collection are not equal to those elements)"
	^ floatCollectionWithSameBeginingEnd ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:00'!
collectionWithoutEqualElements
" return a collection without equal elements"
	^ withoutEqualElements ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:35'!
collectionWithoutEqualsElements

" return a collection not including equal elements "
	^ withoutEqualElements ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 16:30'!
element
	^ 3! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:03'!
elementInCollectionOfFloat
	^ collectionOfFloat anyOne.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:42'!
elementInForElementAccessing
" return an element inculded in 'accessCollection '"
	^ self accessCollection anyOne! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:03'!
elementInForIncludesTest
" return an element included in nonEmpty "
	^ self nonEmpty anyOne! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:27'!
elementInForIndexAccessing

^ self accessCollection  anyOne.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:02'!
elementNotIn
	^ elementNotIn ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:42'!
elementNotInForElementAccessing
" return an element not included in 'accessCollection' "
	^ elementNotIn ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:32'!
elementNotInForIndexAccessing

	^ elementNotIn ! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 666! !

!OrderedCollectionTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ 55! !

!OrderedCollectionTest methodsFor: 'test - remove' stamp: 'damienpollet 1/30/2009 17:16'!
elementTwiceIn
	^ super elementTwiceIn! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 12:10'!
empty
	
	^ empty! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/6/2008 16:29'!
emptyButAllocatedWith20 

	^ emptyButAllocatedWith20! !

!OrderedCollectionTest methodsFor: 'test - iterate' stamp: 'luc.fabresse 11/29/2008 23:09'!
expectedSizeAfterReject
	^1! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:12'!
firstCollection
" return a collection that will be the first part of the concatenation"
	^ nonEmpty 	! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:41'!
firstIndex
" return an index between 'nonEmpty' bounds that is < to 'second index' "
	^1! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:10'!
floatCollectionWithEqualElements
" return a collection of float including equal elements (classic equality)"
	^ collectionOfFloat,collectionOfFloat  ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 13:48'!
floatCollectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals only with classic equality (they are not the same object).
(others elements of the collection are not equal to those elements)"
	^ floatCollectionWithSameBeginingEnd ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:09'!
floatCollectionWithoutEqualElements
" return a collection that does niot incllude equal elements ( classic equality )"
	^ collectionOfFloat ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:49'!
indexArray
	^ indexArray .! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:35'!
indexInForCollectionWithoutDuplicates
" return an index between 'collectionWithoutEqualsElements'  bounds"
	^ 2! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:44'!
indexInNonEmpty
" return an index between bounds of 'nonEmpty' "

	^ 2! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:37'!
integerCollection
" return a collection only including SmallInteger elements"
	^ indexCollection ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:01'!
integerCollectionWithoutEqualElements
" return a collection of integer without equal elements"
	^ withoutEqualElements ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:42'!
newElement
"return an element that will be put in the collection in place of another"
	^999! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 12:10'!
nonEmpty
	
	^ nonEmpty! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 14:57'!
nonEmpty1Element
" return a collection of size 1 including one element"
	^ OrderedCollection new add:( self nonEmpty anyOne); yourself.! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:24'!
nonEmptyMoreThan1Element
" return a collection with more than one element"
	^ collection .! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:38'!
oldSubCollection
" return a subCollection included in collectionWith1TimeSubcollection .
ex :   subCollection := #( 2 3 4) and collectionWith1TimeSubcollection := #(1 2 3 4 5)"
	^ nonEmpty ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 16:04'!
otherCollection
	^ otherCollection! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 13:41'!
replacementCollection
" return a collection that will be used to replace 'oldSubcollection' in ' collectionWith1TimeSubcollection'  " 
	^ collection! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:45'!
replacementCollectionSameSize
" return a collection of size (secondIndex - firstIndex + 1)"

| res |
res := OrderedCollection new.
1 to: (self secondIndex - self firstIndex + 1) do:
	[
	:i |
	res add: 99.
	].
	^res.! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/5/2008 15:53'!
result

	^ result! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:13'!
secondCollection
" return a collection that will be the second part of the concatenation"
	^ collection ! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 15:42'!
secondIndex
" return an index between 'nonEmpty' bounds that is > to 'second index' "
	^2! !

!OrderedCollectionTest methodsFor: 'parameters' stamp: 'stephane.ducasse 10/5/2008 12:38'!
selectorToAccessValuePutIn
	"return the selector of the method that should be invoked to access an element"
	
	^ #first! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'delaunay 4/21/2009 13:47'!
setUp

	nonEmpty := OrderedCollection new  add: self valuePutIn; add: self elementTwiceIn; add: self elementTwiceIn; yourself.
	empty := OrderedCollection new. 
	elementNotIn := 99.
	collection := OrderedCollection new add: 1; add: -2; add: 3; add: 1; yourself.
	indexArray := { 3. 1.}.
	indexCollection := OrderedCollection new add: 1; add: 2;add: 3; add: 4; add:5;  yourself.
	otherCollection := OrderedCollection new add: 1;add: 20; add: 30; yourself.
	withoutEqualElements := OrderedCollection new add: 1;add: 20; add: 30; yourself.
	result := OrderedCollection new add: SmallInteger; add: SmallInteger; add: SmallInteger;add: SmallInteger; yourself.
	emptyButAllocatedWith20 := OrderedCollection new: 20. 
	collectionWithElement := OrderedCollection new add: self element; yourself.
	collectionOfFloat := OrderedCollection new add: 4.1; add: 7.2; add: 2.5; yourself.
	floatCollectionWithSameBeginingEnd := OrderedCollection new add: 4.1; add: 7.2; add: 4.1 copy ; yourself.! !

!OrderedCollectionTest methodsFor: 'setup' stamp: 'stephane.ducasse 10/6/2008 19:45'!
sizeCollection

	^ collection! !

!OrderedCollectionTest methodsFor: 'test - iterate' stamp: 'stephane.ducasse 10/6/2008 17:38'!
speciesClass
	
	^ OrderedCollection! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:02'!
subCollectionNotIn

	^ self collectionNotIncluded .! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 19:00'!
testAdd
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88.
	self assert: (l =  #(1 2 3 4 88) asOrderedCollection).
	l add: 99.
	self assert: (l =  #(1 2 3 4 88 99) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 18:52'!
testAddAfter

	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88 after: 1.
	self assert: (l =  #(1 88 2 3 4) asOrderedCollection).
	l add: 99 after: 2.
	self assert: (l =  #(1 88 2 99 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'testing-public methods' stamp: 'nice 10/8/2008 23:16'!
testAddAfterIndex
	"self run: #testAddAfterIndex"
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 77 afterIndex: 0.
	self assert: (l =  #(77 1 2 3 4) asOrderedCollection).
	l add: 88 afterIndex: 2.
	self assert: (l =  #(77 1 88 2 3 4) asOrderedCollection). 
	l add: 99 afterIndex: l size.
	self assert: (l =  #(77 1 88 2 3 4 99) asOrderedCollection). 
	self should:[l add: 666 afterIndex: -1] raise: Error.
	self should:[l add: 666 afterIndex: l size+1] raise: Error.
	
	"Now make room by removing first two and last two elements,
	and see if the illegal bounds test still fails"
	(l first: 2) , (l last: 2) reverse do: [:e | l remove: e].
	self should: [l add: 666 afterIndex: -1] raise: Error.
	self should: [l add: 666 afterIndex: l size+1] raise: Error.! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 3/21/2006 22:36'!
testAddAll
	"Allows one to add each element of an orderedCollection at the end of another
	orderedCollection "
	"self run:#testAddAll" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c1 addAll: c2.
	self assert: c1 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 3/21/2006 22:36'!
testAddAllFirst
	"Allows one to add each element of an orderedCollection at the beginning of another
	orderedCollection "
	"self run:#testAddAllFirst" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c2 addAllFirst: c1.
	self assert: c2 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 3/21/2006 22:36'!
testAddAllFirstUnlessAlreadyPresent
	"Allows one to add each element of an orderedCollection at the beginning of
	another orderedCollection preserving the order but no duplicate element"
	"self run:#testAddAllFirstUnlessAlreadyPresent" 
	
	| c1 c2 c3 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c3 := #(0 1 ) asOrderedCollection.
	c2 addAllFirstUnlessAlreadyPresent: c1.
	self assert: c2 = #(1 2 3 4 5 6 7 8 9 ) asOrderedCollection.
	c1 addAllFirstUnlessAlreadyPresent: c3.
	self deny: c1 = #(0 1 1 2 3 4 ) asOrderedCollection.
	self assert: c1 = #(0 1 2 3 4 ) asOrderedCollection.
	! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 3/21/2006 22:36'!
testAddAllLast
	"Allows one to add each element of an orderedCollection at the beginning of another
	orderedCollection "
	"self run:#testAddAllLast" 
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := #(5 6 7 8 9 ) asOrderedCollection.
	c1 addAllLast: c2.
	self assert: c1 = #(1 2 3 4 5 6 7 8 9) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 18:57'!
testAddBefore

	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 88 before: 1.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l add: 99 before: 2.
	self assert: (l =  #(88 1 99 2 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 13:42'!
testAddBeforeAndRemove

	| l initialCollection |
	l := #(1 2 3 4) asOrderedCollection.
	initialCollection := l shallowCopy.
	l add: 88 before: 1.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l add: 99 before: 2.
	self assert: (l =  #(88 1 99 2 3 4) asOrderedCollection). 
	l remove: 99.
	l remove: 88.
	self assert: l = initialCollection.

! !

!OrderedCollectionTest methodsFor: 'testing-public methods' stamp: 'nice 10/8/2008 23:17'!
testAddBeforeIndex
	"self run: #testAddBeforeIndex"
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l add: 77 beforeIndex: 1.
	self assert: (l =  #(77 1 2 3 4) asOrderedCollection).
	l add: 88 beforeIndex: 3.
	self assert: (l =  #(77 1 88 2 3 4) asOrderedCollection). 
	l add: 99 beforeIndex: l size+1.
	self assert: (l =  #(77 1 88 2 3 4 99) asOrderedCollection). 
	self should:[l add: 666 beforeIndex: 0] raise: Error.
	self should:[l add: 666 beforeIndex: l size+2] raise: Error.
	
	"Now make room by removing first two and last two elements,
	and see if the illegal bounds test still fails"
	(l first: 2) , (l last: 2) reverse do: [:e | l remove: e].
	self should:[l add: 666 beforeIndex: 0] raise: Error.
	self should:[l add: 666 beforeIndex: l size+2] raise: Error.

! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 13:43'!
testAddDuplicateItem1
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection add: 'John' before: 'John'.
	self
		assert: ((collection asBag occurrencesOf: 'John')
					= 2
				and: [(collection at: (collection indexOf: 'John')
							+ 1)
						= (collection
								at: (collection indexOf: 'John'))])! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 18:58'!
testAddFirst
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l addFirst: 88.
	self assert: (l =  #(88 1 2 3 4) asOrderedCollection).
	l addFirst: 99.
	self assert: (l =  #(99 88 1 2 3 4) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 6/5/2005 09:21'!
testAddItem1

   | collection size |
   collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
   size := collection size.
   collection add: 'James' before: 'Jim'.
   collection add: 'Margaret' before: 'Andrew'.
   self assert: size + 2 = collection size.
! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 19:07'!
testAddItem2
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection add: 'James' before: 'Jim'.
	collection add: 'Margaret' before: 'Andrew'.
	self assert: (collection indexOf: 'James')
			+ 1
			= (collection indexOf: 'Jim').
	self assert: (collection indexOf: 'Margaret')
			+ 1
			= (collection indexOf: 'Andrew')! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'zz 12/7/2005 18:59'!
testAddLast
	| l |
	l := #(1 2 3 4) asOrderedCollection.
	l addLast: 88.
	self assert: (l =  #(1 2 3 4 88) asOrderedCollection).
	l addLast: 99.
	self assert: (l =  #(1 2 3 4 88 99) asOrderedCollection). 

! !

!OrderedCollectionTest methodsFor: 'tests - accessing' stamp: 'zz 12/7/2005 18:50'!
testAt
	| collection |
	collection := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	self assert: (collection at:1) = 'Jim'.
	self assert: (collection at:2) = 'Mary'! !

!OrderedCollectionTest methodsFor: 'tests - adding' stamp: 'sd 3/21/2006 22:37'!
testAtIfAbsentPut
	"Allows one to add an element at an index if no element exist at this index"
	"self run:#testAtIfAbsentPut" 
	
	| c |
	c := #(1 2 3 4 ) asOrderedCollection.
	self
		shouldnt: [c at: 2 ifAbsentPut: [5]]
		raise: Error.
	self assert: c = #(1 2 3 4 ) asOrderedCollection.
	c at: 5 ifAbsentPut: [5].
	self assert: c = #(1 2 3 4 5 ) asOrderedCollection.
	c at: 7 ifAbsentPut: [7].
	self assert: c = #(1 2 3 4 5 nil 7 ) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - accessing' stamp: 'sd 3/21/2006 22:38'!
testAtPut
	"Allows one to replace an element but not at an off range index"
	"self run:#testAtPut"
	| c |
	c := #(1 2 3 4 ) asOrderedCollection.
	c at: 2 put: 5.
	self assert: c = #(1 5 3 4 ) asOrderedCollection.
	self
		should: [c at: 5 put: 8]
		raise: Error.
	self deny: c = #(1 5 3 4 8 ) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - accessing' stamp: 'stephane.ducasse 10/6/2008 16:32'!
testCapacityFromAsOrderedCollection
	"Allows one to check the current capacity of an Ordered collection"
	"self run:#testCapacityFromAsOrderedCollection"
	
	| c1 c2 c3 |
	c1 := #(1 2 ) asOrderedCollection.
	self assert: (c1 capacity =  2).
	c2 := OrderedCollection new: 10.
	c2 add: 3.
	self assert: (c2 capacity = 10).	
	c3 := OrderedCollection new.
	self deny: (c3 capacity =  0).
	! !

!OrderedCollectionTest methodsFor: 'tests - enumerating' stamp: 'sd 3/21/2006 22:41'!
testCollect
	"Allows one to collect some element of a collection into another collection"
	"self run: #testCollect"
	 
	| c1 c2 res |
	c1 := #(-1 2 -3 4) asOrderedCollection.
	c2 := #(1 2 3 4) asOrderedCollection.
	res := c1 collect: [:each | each abs].
	self assert: (c2 = res).! !

!OrderedCollectionTest methodsFor: 'tests - enumerating' stamp: 'cm 3/8/2006 09:09'!
testCollectFromTo
	"Allows one to collect some element of a collection into another collection between a first index and an end index for the collect"
	"self run: #testCollectFromTo"
	
	| c1 res |
	c1 := #(-1 2 -3 4 -5 6 -7 8) asOrderedCollection.
	res := c1 collect: [:each | each abs] from: 1 to: 3.
	self assert: (res = #(1 2 3) asOrderedCollection).
	self should: [c1 collect: [:each | each abs] from: 10 to: 13] raise: Error.
	self should: [c1 collect: [:each | each abs] from: 5 to: 2] raise: Error.! !

!OrderedCollectionTest methodsFor: 'tests - copying' stamp: 'stephane.ducasse 1/12/2009 15:02'!
testCopyEmptyOld
	"Allows one to create a copy of the receiver that contains no elements"
	"self run:#testCopyEmpty"
	
	| c1 c2 |
	c1 := #(1 2 3 4 ) asOrderedCollection.
	c2 := c1 copyEmpty.
	self assert: (c2 size = 0).! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!OrderedCollectionTest methodsFor: 'tests - copying' stamp: 'sd 3/21/2006 22:41'!
testCopyFromTo
	"Allows one to create a copy of the receiver that contains elements from position start to end"
	"self run: #testCopyFromTo"
	
	| c1 c2 c3 | 
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := (c1 copyFrom: 1 to: 2).
	self assert: c2 = #(1 2) asOrderedCollection.
	self should: [c1 copyFrom: 10 to: 20] raise: Error.
	
	c3 := c1 copyFrom: 4 to: 2.
	self assert: c3 isEmpty.
	
	self should: [c1 copyFrom: 4 to: 5 ] raise: Error.
	
	
	
! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!OrderedCollectionTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!OrderedCollectionTest methodsFor: 'tests - copying' stamp: 'sd 3/21/2006 22:41'!
testCopyReplaceFromToWith
	"Allows one to create a copy from the receiver which elements between start and end of the 	receiver being replace by 	element of the collection after with:"
	"self run:#testCopyReplaceFromToWith"

	| c1 c2 c3 c4 |
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := #(5 6 7 8 9) asOrderedCollection.
	c3 := (c2 copyReplaceFrom: 1 to: 2 with: c1).
	self assert: c3 = #(1 2 3 4 7 8 9) asOrderedCollection.
	self should: [c2 copyReplaceFrom: 3 to: 1 with: c1] raise: Error.
	
	c4 := (c2 copyReplaceFrom: 10 to: 25 with: c1).
	self assert: c4 = #(5 6 7 8 9 1 2 3 4) asOrderedCollection.
	
	! !

!OrderedCollectionTest methodsFor: 'tests - copying' stamp: 'sd 3/21/2006 22:41'!
testCopyWith
	"Allows one to create a copy of the receiver that contains the new element at the end"
	"self run: #testCopyWith"
	
	| c1 | 
	c1 := #(1 2 3 4) asOrderedCollection.
	c1 := c1 copyWith: 6.
	self assert: c1 = #(1 2 3 4 6) asOrderedCollection.
	
	

	
	
	
! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!OrderedCollectionTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!OrderedCollectionTest methodsFor: 'tests - enumerating' stamp: 'sd 6/5/2005 09:21'!
testIndexOfWithDuplicates

   | collection indices bagOfIndices |
   collection := #('Jim' 'Mary' 'John' 'Andrew' 'Mary' 'John' 'Jim' 'Micheal') asOrderedCollection.
   indices := collection collect: [:item | collection indexOf: item].
   self assert: indices asSet size = collection asSet size.
   bagOfIndices := indices asBag.
   self assert: (indices asSet 
                    allSatisfy: [:index | (bagOfIndices occurrencesOf: index)
	                                       = (collection occurrencesOf: (collection at: index))]).

  "  indexOf:  returns the index of the first occurrence of an item.
     For an item with n occurrences, the index of its first occurrence
     is found  n  times. "! !

!OrderedCollectionTest methodsFor: 'tests - removing' stamp: 'zz 12/7/2005 19:05'!
testRemoveAllSuchThat
	| collection |
	collection := (1 to: 10) asOrderedCollection.
	collection
		removeAllSuchThat: [:e | e even].
	self assert: collection = (1 to: 10 by: 2) asOrderedCollection! !

!OrderedCollectionTest methodsFor: 'tests - removing' stamp: 'sd 3/21/2006 22:39'!
testRemoveAt
	"Allows one to remove an element from a collection at an index"
	"self run:#testRemoveAt" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeAt: 2.
	self assert: (c1 = #(2 4 6) asOrderedCollection).
	self should: [c1 removeAt: 10] raise: Error.
	self should: [c1 removeAt: -1] raise: Error.
	! !

!OrderedCollectionTest methodsFor: 'tests - removing' stamp: 'sd 3/21/2006 22:39'!
testRemoveFirst
	"Allows one to remove n element of a collection at the first"
	"self run:#testRemoveFirst" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeFirst: 1.
	self assert: (c1 = #(3 4 6) asOrderedCollection).
	c1 removeFirst: 2.
	self assert: (c1 = #(6) asOrderedCollection).
	self should: [c1 removeFirst: 10] raise: Error.
	
	! !

!OrderedCollectionTest methodsFor: 'tests - removing' stamp: 'sd 3/21/2006 22:39'!
testRemoveIfAbsent
	"Allows one to remove an element from a collection and to copy it in another collection."
	"If the element isn't in the first collection, the second collection copy the element after ifAbsent"
	"self run:#testRemoveIfAbsent"
	
	| c1 c2 |
	c1 := #(1 2 3 4) asOrderedCollection.
	c2 := OrderedCollection new.
	
	c2 add: (c1 remove: 2 ifAbsent: [6]).
	self assert: (c1 = #(1 3 4) asOrderedCollection).
	self assert: (c2 = #(2) asOrderedCollection).
	
	c2 add: (c1 remove: 18 ifAbsent: [6]).
	self assert: (c1 = #(1 3 4) asOrderedCollection).
	self assert: (c2 = #(2 6) asOrderedCollection).! !

!OrderedCollectionTest methodsFor: 'tests - removing' stamp: 'sd 3/21/2006 22:39'!
testRemoveLast
	"Allows one to remove n element of a collection at the end"
	"self run:#testRemoveLast" 
	
	| c1 |
	c1 := #(2 3 4 6) asOrderedCollection.
	c1 removeLast: 1.
	self assert: (c1 = #(2 3 4) asOrderedCollection).
	c1 removeLast: 2.
	self assert: (c1 = #(2) asOrderedCollection).
	self should: [c1 removeLast: 10] raise: Error.! !

!OrderedCollectionTest methodsFor: 'tests - copying' stamp: 'zz 12/7/2005 13:47'!
testReversed
	| collection1 collection2 |
	collection1 := #('Jim' 'Mary' 'John' 'Andrew' ) asOrderedCollection.
	collection2 := collection1 reversed.
	self assert: collection2 first = 'Andrew'.
	self assert: collection2 last = 'Jim'! !

!OrderedCollectionTest methodsFor: 'tests - enumerating' stamp: 'cm 3/8/2006 10:02'!
testWithCollect
	"Allows one to collect some element of two collections into another collection with element corresponding to the condition in the blocks"
	"self run: #testWithCollect"
	
	| c1 c2 res |
	c1 := #(-1 2 -3 4 -5 6 -7 8) asOrderedCollection.
	c2 := #(-9 10 -11 12 -13 14 -15 16) asOrderedCollection.
	res := c1 with: c2 collect: [:each1 :each2 | each1 < each2
		ifTrue: [each1]
		ifFalse: [each2]].
	self assert: (res = #(-9 2 -11 4 -13 6 -15 8) asOrderedCollection).
	! !

!OrderedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 11:52'!
valueArray
	
	| |
	^ { elementNotIn . elementNotIn }.! !

CollectionRootTest subclass: #SetTest
	uses: TAddForUniquenessTest + TIncludesTest + TCloneTest + TCopyTest + TSetAritmetic + TRemoveTest + TCreationWithTest - {#testOfSize} + TGrowableTest + TStructuralEqualityTest + TSizeTest + TPrintTest + TAsStringCommaAndDelimiterTest + TConvertTest
	instanceVariableNames: 'full empty collection result emptyButAllocatedWith20 elementNotIn collectionOfFloat collectionIncluded nonEmpty1element withoutEqualElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!SetTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Set. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

SetTest class
	uses: TAddForUniquenessTest classTrait + TIncludesTest classTrait + TCloneTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait + TRemoveTest classTrait + TCreationWithTest classTrait + TGrowableTest classTrait + TStructuralEqualityTest classTrait + TSizeTest classTrait + TPrintTest classTrait + TAsStringCommaAndDelimiterTest classTrait + TConvertTest classTrait
	instanceVariableNames: ''!

!SetTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 14:58'!
anotherElementNotIn
	^ 42! !

!SetTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 17:08'!
collection

	^ collection! !

!SetTest methodsFor: 'test - creation' stamp: 'stephane.ducasse 12/9/2008 21:14'!
collectionClass 
	
	^ Set! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:54'!
collectionInForIncluding
	^ collectionIncluded ! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:55'!
collectionNotIncluded
	^ Set with: elementNotIn. ! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:42'!
collectionOfFloat
	^ collectionOfFloat ! !

!SetTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 15:42'!
collectionWithElement
	^ full! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:54'!
collectionWithElementsToRemove
	^ collectionIncluded ! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:04'!
collectionWithoutEqualElements
" return a collection without equal elements"
	^ withoutEqualElements ! !

!SetTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/8/2008 15:42'!
element
	^ 4! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/20/2009 16:40'!
elementInForIncludesTest
" return an element included in nonEmpty "
	^ self nonEmpty anyOne! !

!SetTest methodsFor: 'ambiguous' stamp: 'delaunay 4/20/2009 16:42'!
elementNotIn
	^elementNotIn ! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 666! !

!SetTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ 42! !

!SetTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:11'!
empty
	
	^ empty! !

!SetTest methodsFor: 'tests - size capacity' stamp: 'stephane.ducasse 10/6/2008 17:09'!
emptyButAllocatedWith20

	^ emptyButAllocatedWith20! !

!SetTest methodsFor: 'test - iterate' stamp: 'luc.fabresse 11/29/2008 23:09'!
expectedSizeAfterReject
	^1! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:05'!
integerCollectionWithoutEqualElements
" return a collection of integer without equal elements"
	^ withoutEqualElements ! !

!SetTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 10/5/2008 13:11'!
nonEmpty
	
	^ full! !

!SetTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:01'!
nonEmpty1Element
" return a collection of size 1 including one element"
	^ nonEmpty1element ! !

!SetTest methodsFor: 'requirements' stamp: 'damiencassou 1/20/2009 13:25'!
otherCollection
	"Returns a collection that does not include what is returned by #element."
	^ result! !

!SetTest methodsFor: 'requirements' stamp: 'stephane.ducasse 10/6/2008 17:08'!
result

	 ^ result! !

!SetTest methodsFor: 'test - iterate' stamp: 'stephane.ducasse 10/6/2008 19:40'!
selectedNumber

	^ 2
	! !

!SetTest methodsFor: 'initialize' stamp: 'delaunay 4/21/2009 10:04'!
setUp
	empty := Set new.
	full := Set with: 1 with: 2 with: 3 with: 4.
	collectionIncluded := Set with: 2 with: 3 .
	collection := Set new.
	collection add: 1; add: -2; add: 3. 
	result := Set new.
	result add: SmallInteger.
	emptyButAllocatedWith20 := Set new: 20.
	elementNotIn := 99.
	collectionOfFloat := Set with: 2.5 with: 4.6 with: 4.2.
	nonEmpty1element := Set with: 32.
	withoutEqualElements := Set with: 4 with: 5 with: 2.! !

!SetTest methodsFor: 'tests - size capacity' stamp: 'stephane.ducasse 10/6/2008 19:33'!
sizeCollection

	^ full! !

!SetTest methodsFor: 'test - iterate' stamp: 'stephane.ducasse 10/6/2008 17:39'!
speciesClass
	
	^ Set! !

!SetTest methodsFor: 'sunit original tests' stamp: 'GL 2/28/2006 08:26'!
testAdd
	empty add: 5.
	self assert: (empty includes: 5).! !

!SetTest methodsFor: 'tests - growable' stamp: 'damiencassou 1/20/2009 10:44'!
testAddEmptyGrows
	"self debug: #testAddEmptyGrows"
	
	| oldSize |
	oldSize := self empty size.
	self empty add: self element.
	self assert: self empty size = (oldSize + 1).! !

!SetTest methodsFor: 'tests - growable' stamp: 'delaunay 4/2/2009 11:53'!
testAddNonEmptyGrowsWhenNewElement
	"self debug: #testAddNonEmptyGrowsWhenNewElement"
	| oldSize |
	oldSize := self nonEmpty size.
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self nonEmpty add: self elementNotInForOccurrences.
	self assert: self nonEmpty size > oldSize! !

!SetTest methodsFor: 'some tests' stamp: 'GL 3/16/2006 10:36'!
testAddWithOccurences

	empty add: 2 withOccurrences: 3.
	self assert: (empty includes: 2).
	self assert: ((empty occurrencesOf: 2) = 1).! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 19:34'!
testAsSet
	"could be moved in Array or Collection"

	| newFull |
	newFull := #(1 2 3 ) asSet.
	newFull add: 4.
	self assert: (newFull = full).! !

!SetTest methodsFor: 'some tests' stamp: 'GL 2/28/2006 09:25'!
testAtRandom
	| rand |
	rand := Random new.
	full add: 3.
	full add: 2.
	full add: 4.
	full add: 1.
	self assert: (full includes: (full atRandom: rand)).
	! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 16:56'!
testCollect2
	| newFull result |
	newFull := Set withAll: (1 to: 10).
	result := newFull collect: [:each | each >= 1 ifTrue: [each] ifFalse: ['no']].
	self assert: (result = newFull).
	result := newFull collect: [:each | each >= 5 ifTrue: [each] ifFalse: ['no']].
	self assert: (result = ((Set withAll: (5 to: 10)) add: 'no'; yourself)).! !

!SetTest methodsFor: 'some tests' stamp: 'GL 2/28/2006 09:46'!
testCopy
	| newFull |
	full add: 3.
	full add: 2.
	newFull := full copy.
	self assert: (full size = newFull size).
	self assert: ((full select: [:each | (newFull includes: each) not]) isEmpty).
	self assert: ((newFull select: [:each | (full includes: each) not]) isEmpty).! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!SetTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!SetTest methodsFor: 'some tests' stamp: 'GL 3/8/2006 00:06'!
testCopyWithout
	| newFull |
	full add: 3.
	full add: 2.
	newFull := full copyWithout: 3.
	self assert: (newFull size = (full size - 1)).
	self deny: (newFull includes: 3).
	self assert: ((newFull select: [:each | (full includes: each) not]) isEmpty).
	self assert: ((full select: [:each | (newFull includes: each) not]) = (Set with: 3)).
	! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 16:56'!
testDo2
	| newFull result |
	newFull := Set withAll: (1 to: 5).
	result := 0.
	newFull do: [:each | result := (result + each)].
	self assert: (result = 15).! !

!SetTest methodsFor: 'test - iterate' stamp: 'damienpollet 1/30/2009 18:04'!
testDoWithoutNoDuplicates
	"self debug: #testDoWithoutNoDuplicates"
	| res |
	res := self speciesClass new.  
	self collection do: [:each | res add: each] without: -2.
	self assert: res size = self doWithoutNumber.! !

!SetTest methodsFor: 'sunit original tests' stamp: 'GL 2/28/2006 08:27'!
testGrow
	empty addAll: (1 to: 100).
	self assert: (empty size = 100).
			! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!SetTest methodsFor: 'sunit original tests' stamp: 'GL 2/28/2006 08:23'!
testIllegal
	self 
		should: [empty at: 5] raise: TestResult error.
	self 
		should: [empty at: 5 put: #abc] raise: TestResult error.
			! !

!SetTest methodsFor: 'sunit original tests' stamp: 'stephane.ducasse 10/6/2008 19:35'!
testIncludes
	self assert: (full includes: 4).
	self assert: (full includes: 3).
	self deny: (full includes: 6).
			! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!SetTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!SetTest methodsFor: 'some tests' stamp: 'GL 3/8/2006 09:55'!
testIntersection
	| newFull col |
	full add: 3; add: 2.
	col := full intersection: full.
	self assert: (full = col).

	newFull := Set with: 8 with: 9 with: #z.
	col := newFull intersection: full.
	self assert: (col isEmpty).
	
	newFull add: 5; add: #abc; add: 7.
	col := newFull intersection: full.
	self assert: ((full select: [:each | (newFull includes: each)]) = col).
	
	
	! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 19:35'!
testLike
	self assert: ((full like: 3) = 3).
	self assert: ((full like: 8) isNil).! !

!SetTest methodsFor: 'sunit original tests' stamp: 'stephane.ducasse 10/6/2008 19:39'!
testOccurrences
	self assert: ((empty occurrencesOf: 0) = 0).
	self assert: ((full occurrencesOf: 4) = 1).
	full add: 4.
	self assert: ((full occurrencesOf: 4) = 1).! !

!SetTest methodsFor: 'sunit original tests' stamp: 'stephane.ducasse 10/6/2008 19:34'!
testRemove
	full remove: 4.
	self assert: (full includes: 3).
	self deny: (full includes: 4).! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 19:39'!
testRemoveIfAbsent
	| result1 result2  |
	result1 := true.
	result2 := true.
	full remove: 8 ifAbsent: [ result1 := false ].
	self assert: (result1 = false).
	full remove: 4 ifAbsent: [ result2 := false ].
	self assert: (result2 = true).
	
	
	! !

!SetTest methodsFor: 'some tests' stamp: 'stephane.ducasse 10/6/2008 19:34'!
testSize2
	self assert: (empty size = 0).
	self assert: (full size = 4).
	empty add: 2.
	empty add: 1.
	full add: 2.
	self assert: (empty size = 2).
	self assert: (full size = 4).
	empty remove: 2.
	self assert: (empty size = 1).! !

!SetTest methodsFor: 'some tests' stamp: 'GL 3/8/2006 00:50'!
testUnion
	| newFull col newCol |
	full add: 3.
	full add: 2.
	col := full union: full.
	self assert: (full = col).

	newFull := Set with: 8 with: 9 with: #z.
	col := newFull union: full.
	self assert: (col size = (full size + newFull size)).
	self assert: ((col select: [:each | (full includes: each) not]) = newFull).
	self assert: ((col select: [:each | (newFull includes: each) not]) = full).

	full add: 9.
	col := newFull union: full.
	newCol := newFull copy.
	newCol remove: 9.
	self assert: (col size = (full size + newFull size - 1)).
	self assert: ((col select: [:each | (full includes: each) not]) = newCol).
	newCol := full copy.
	newCol remove: 9.
	self assert: ((col select: [:each | (newFull includes: each) not]) = newCol).
	
	
	! !

CollectionRootTest subclass: #SortedCollectionTest
	uses: TEmptyTest + TIterateSequencedReadableTest + TPrintOnSequencedTest + TAsStringCommaAndDelimiterSequenceableTest + TIndexAccess + TSequencedElementAccessTest - {#testAtAll} + TSubCollectionAccess + TIndexAccessForMultipliness + TIncludesTest + TRemoveTest + TConvertTest + TConvertAsSetForMultiplinessTest + TAddTest + TBeginsEndsWith + TCopySequenceableSameContents - {#testShuffled} + TSetAritmetic + TRemoveByIndexTest + TCopyPartOfSequenceable + TCopyPartOfSequenceableForMultipliness
	instanceVariableNames: 'nonEmpty elementExistsTwice empty collection result nonEmpty1Element collectionOfFloat accessCollection elementNoteIn oldSubcollection floatCollectionSameEndAndBegining withoutEqualElements collectionOfFloatWithDuplicate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

SortedCollectionTest class
	uses: TEmptyTest classTrait + TIterateSequencedReadableTest classTrait + TPrintOnSequencedTest classTrait + TAsStringCommaAndDelimiterSequenceableTest classTrait + TIndexAccess classTrait + TSequencedElementAccessTest classTrait + TSubCollectionAccess classTrait + TIndexAccessForMultipliness classTrait + TIncludesTest classTrait + TRemoveTest classTrait + TConvertTest classTrait + TConvertAsSetForMultiplinessTest classTrait + TAddTest classTrait + TBeginsEndsWith classTrait + TCopySequenceableSameContents classTrait + TSetAritmetic classTrait + TRemoveByIndexTest classTrait + TCopyPartOfSequenceable classTrait + TCopyPartOfSequenceableForMultipliness classTrait
	instanceVariableNames: ''!

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:32'!
accessCollection
" return a collection of size 5 "
	^accessCollection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:33'!
anotherElementNotIn
" return an element different of 'elementNotIn'  not included in 'nonEmpty' "
	^666! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:24'!
collection

	^ collection! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 16:30'!
collectionClass
" return the class to be used to create instances of the class tested"
	^ SortedCollection! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:31'!
collectionOfFloat
" return a collection only includiing Float elements "
	^ collectionOfFloat ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:48'!
collectionWith1TimeSubcollection
" return a collection including 'oldSubCollection'  only one time "
	^ (SortedCollection new add: elementNoteIn ; yourself) , self oldSubCollection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:48'!
collectionWith2TimeSubcollection
" return a collection including 'oldSubCollection'  two or many time "
	^ (SortedCollection  new add: elementNoteIn ; yourself) , self oldSubCollection , self oldSubCollection  ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:23'!
collectionWith5Elements
" return a collection of size 5 including 5 elements"
^ accessCollection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 15:10'!
collectionWithElement
	"Returns a collection that already includes what is returned by #element."
	^ SortedCollection new add: self element ; add: 5 ; add: 2; yourself.! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 11:12'!
collectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals .
(others elements of the collection are not equal to those elements)"
	^ floatCollectionSameEndAndBegining ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:47'!
collectionWithoutEqualElements
" return a collection without equal elements"
	^ withoutEqualElements ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:47'!
collectionWithoutEqualsElements

" return a collection not including equal elements "
	^withoutEqualElements ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 15:09'!
element
	"Returns an object that can be added to the collection returned by #collection."
	^ 88! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:33'!
elementInCollectionOfFloat
" return an element included in 'collectionOfFloat' "
	^ collectionOfFloat anyOne! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:38'!
elementInForElementAccessing
" return an element inculded in 'accessCollection '"
	^ self accessCollection anyOne! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:34'!
elementInForIncludesTest
" return an element included in nonEmpty "
	^ nonEmpty anyOne ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:33'!
elementInForIndexAccessing
" return an element included in 'accessCollection' "
	^ accessCollection anyOne! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:34'!
elementNotIn
"return an element not included in 'nonEmpty' "

	^ elementNoteIn ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:39'!
elementNotInForElementAccessing
" return an element not included in 'accessCollection' "
	^ elementNoteIn ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:34'!
elementNotInForIndexAccessing
" return an element not included in 'accessCollection' "
	^ elementNoteIn  	! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ 666! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:17'!
empty
	
	^ empty! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:24'!
expectedSizeAfterReject
	^1! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 15:02'!
floatCollectionWithEqualElements
" return a collection of float including equal elements (classic equality)"
	^ collectionOfFloatWithDuplicate ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 13:41'!
floatCollectionWithSameAtEndAndBegining
" return a collection with elements at end and begining equals only with classic equality (they are not the same object).
(others elements of the collection are not equal to those elements)"
	^ floatCollectionSameEndAndBegining ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 15:03'!
floatCollectionWithoutEqualElements
" return a collection that does niot incllude equal elements ( classic equality )"
	^ collectionOfFloat ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/22/2009 10:48'!
indexInForCollectionWithoutDuplicates
" return an index between 'collectionWithoutEqualsElements'  bounds"
	^ 2! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 16:01'!
integerCollection
" return a collection only including SmallInteger elements"
	^ accessCollection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 14:47'!
integerCollectionWithoutEqualElements
" return a collection of integer without equal elements"
	^ withoutEqualElements ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:17'!
nonEmpty
	
	^ nonEmpty! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:27'!
nonEmpty1Element
" return a collection of size 1 including one element"
	^ nonEmpty1Element ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:23'!
nonEmptyMoreThan1Element
" return a collection of integer with more than one element" 
	^collection
	.! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:51'!
oldSubCollection
" return a subCollection included in collectionWith1TimeSubcollection .
ex :   subCollection := #( 2 3 4) and collectionWith1TimeSubcollection := #(1 2 3 4 5)"
	^ oldSubcollection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 15:11'!
otherCollection
	"Returns a collection that does not include what is returned by #element."
	 ^ SortedCollection new add:  7 ; add: 5 ; add: 2; yourself.! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:43'!
replacementCollection
" return a collection that will be used to replace 'oldSubcollection' in ' collectionWith1TimeSubcollection'  " 
	^ collection ! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:29'!
result

	 ^ result! !

!SortedCollectionTest methodsFor: 'setup' stamp: 'delaunay 4/21/2009 15:02'!
setUp

	nonEmpty := SortedCollection new.
	elementExistsTwice := 12332312321.
	nonEmpty add: 2.
	nonEmpty add: elementExistsTwice.
	nonEmpty add: elementExistsTwice.
	empty := SortedCollection  new. 
	collection := SortedCollection new.
	collection add: 1.
	collection add: -2.
	collection add: 3.
	collection add: 1.
	result := OrderedCollection new. "SortedCollection sortBlock: [:a :b | a name < b name]."
	result add: SmallInteger.
	result add: SmallInteger.
	result add: SmallInteger.
	result add: SmallInteger.
	nonEmpty1Element := SortedCollection new add:5; yourself.
	collectionOfFloat := SortedCollection new add:1.2 ; add: 5.6 ; add:4.4 ; add: 1.9 ; yourself.
	collectionOfFloatWithDuplicate := SortedCollection new add:1.2 ; add: 5.6 ; add:4.4 ; add: 1.2 ; yourself.
	accessCollection := SortedCollection new add:1 ; add: 5 ; add:4 ; add: 2 ; add:7 ; yourself.
	elementNoteIn := 999.
	oldSubcollection := SortedCollection new add: 2 ; add: 2 ; add: 2 ; yourself.
	floatCollectionSameEndAndBegining := SortedCollection new add: 1.5 ; add: 1.5 copy ; yourself.
	withoutEqualElements := SortedCollection new add: 1 ; add: 8 copy ; add: 4;yourself.
	! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'damienpollet 1/14/2009 16:34'!
sizeCollection
	^ collection! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'stephane.ducasse 12/20/2008 22:23'!
speciesClass
	
	^ SortedCollection! !

!SortedCollectionTest methodsFor: 'requirements' stamp: 'delaunay 4/21/2009 10:49'!
subCollectionNotIn
" return a collection for which at least one element is not included in 'accessCollection' "
	^ SortedCollection new add: elementNoteIn ; add: elementNoteIn ; yourself.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:27'!
testAdd
	"self run: #testAdd"
	"self debug: #testAdd"
	
	| collection |
	collection := #(10 9 8 7 5 4 4 2) asSortedCollection.
	self assert: collection first = 2.
	self assert: collection last = 10.
	self assert: (collection size = 8).
	collection add:1.
	self assert: (collection size = 9).
	collection add: 6.
	self assert: ((collection at: 5) = 5).
	self assert: (collection size = 10).
	collection add: 3.
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:30'!
testAddAll
	"self run: #testAddAll"
	"self debug: #testAddAll"
	
	| sorted2 sorted|
	sorted2 := SortedCollection new.
	sorted2 add: 'brochet'; add:'truitelle'.
	sorted := SortedCollection new.
	sorted addAll: sorted2.
	self assert: (sorted hasEqualElements: sorted2).
	 ! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:32'!
testAddAll2
	"self run: #testAddAll2"
	"self debug: #testAddAll2"
	
	| sorted2 sorted|
	sorted2 := SortedCollection new.
	sorted2 add: 'brochet'; add:'truitelle'.
	sorted := SortedCollection new.
	sorted add: 'perche'.
	sorted addAll: sorted2.
	self assert: (sorted size = (sorted2 size + 1)).
	sorted2 do: 
			[ :each | self assert: (sorted includes: each)]
	 ! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:33'!
testCollect
	"self run: #testCollect"
	
	|result aSortedCollection|
	aSortedCollection := SortedCollection new.
	result := OrderedCollection new.
	result add:true ; add: true ; add: true ;add: false ; add: false.
	aSortedCollection := (1 to: 5) asSortedCollection.
	self assert: (result = (aSortedCollection collect: [:each | each < 4])).
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:39'!
testCopy
	"self run: #testCopy"
	"self debug: #testCopy"
	
	|aSortedCollection copySorted|
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a < b].
	aSortedCollection add: 'truite' ; add: 'brochet'.
	self assert: aSortedCollection first = 'brochet'.
	
	copySorted := aSortedCollection copy.
	
	self assert: (copySorted  hasEqualElements: aSortedCollection).
	self assert: (copySorted  species = aSortedCollection species).
	self assert: (copySorted  sortBlock = aSortedCollection sortBlock).
	self assert: copySorted first = 'brochet'.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:30'!
testCreation
	"self run: #testCreation"
	"self debug: #testCreation"
	
	| collection |
	collection := #(10 9 3 6 1 8 7 5 4 2) asSortedCollection.
	self assert: collection = (1 to: 10) asSortedCollection.
	! !

!SortedCollectionTest methodsFor: 'test - iterate' stamp: 'ab 12/26/2008 18:03'!
testDo2

	| res |
	res := self speciesClass sortBlock: [:a :b | a name < b name]..  
	self collection do: [:each | res add: each class].
	self assert: res asArray = self result asArray. 
! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:42'!
testEquals
	"self run: #testEquals"
	"self debug: #testEquals"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	aSortedCollection add:'truite' ; add: 'brochet'.
	self assert: aSortedCollection copy = aSortedCollection.! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:43'!
testMedian
	"self run: #testMedian"
	"self debug: #testMedian"
	
	|aSortedCollection|
	aSortedCollection := (1 to: 10) asSortedCollection.
	self assert: aSortedCollection median=5.
	
	aSortedCollection := SortedCollection new.
	aSortedCollection add:'truite' ; add:'porcinet' ; add:'carpe'.
	self assert: (aSortedCollection median = 'porcinet').
	! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:46'!
testShouldPrecede
	"self run: #testShouldPrecede"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	self assert: (aSortedCollection should: 'za' precede: 'zb').
	self assert: (aSortedCollection isEmpty).
	self assert: (aSortedCollection should: 1 precede: 2).
	
	aSortedCollection sortBlock: [:a :b | a > b].
	aSortedCollection reSort.
	self assert: (aSortedCollection should: 'zb' precede: 'za').
	self assert: (aSortedCollection isEmpty).
	self assert: (aSortedCollection should: 2 precede: 1).
		! !

!SortedCollectionTest methodsFor: 'basic' stamp: 'DM 3/16/2006 11:38'!
testSortBlock
	"self run: #testSortBlock"
	"self debug: #testSortBlock"
	
	|aSortedCollection|
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a < b].
	aSortedCollection add: 'truite' ; add: 'brochet' ; add: 'tortue'.
	self assert: aSortedCollection first = 'brochet'.
	
	aSortedCollection := SortedCollection new.
	aSortedCollection sortBlock: [:a :b | a >b].
	aSortedCollection add: 'truite' ; add: 'brochet' ; add: 'tortue'.
	self assert: aSortedCollection first = 'truite'.
	
	
	! !

ClassTestCase subclass: #HeapTest
	uses: TAddTest + TRemoveTest + TIterateTest + TGrowableTest + TSequencedElementAccessTest
	instanceVariableNames: 'collectionWithElement otherCollection nonEmpty empty collection elementNotIn result expectedElementByDetect speciesClass elementTwiceIn doWithoutNumber element expectedSizeAfterReject accessCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

HeapTest class
	uses: TAddTest classTrait + TRemoveTest classTrait + TIterateTest classTrait + TGrowableTest classTrait + TSequencedElementAccessTest classTrait
	instanceVariableNames: ''!

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 15:55'!
accessCollection

	^accessCollection.! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 10:17'!
collection
	^ collection.! !

!HeapTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	self explicitRequirement! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/18/2009 15:07'!
collectionWithElement
	^ collectionWithElement! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 13:37'!
element
	^ element! !

!HeapTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:22'!
elementInCollectionOfFloat
	self explicitRequirement! !

!HeapTest methodsFor: 'requirements' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ elementNotIn! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 11:33'!
elementTwiceIn
	^elementTwiceIn! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/18/2009 15:24'!
empty
	^empty.! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 11:19'!
expectedElementByDetect
	"Returns the first even element of #collection"
	^ expectedElementByDetect.
	
	! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 13:46'!
expectedSizeAfterReject
	"Number of even elements in #collection"
	^ expectedSizeAfterReject.! !

!HeapTest methodsFor: 'examples' stamp: 'md 2/12/2006 15:33'!
heapExample	"HeapTest new heapExample"
	"Create a sorted collection of numbers, remove the elements
	sequentially and add new objects randomly.
	Note: This is the kind of benchmark a heap is designed for."
	| n rnd array time sorted |
	n := 5000. "# of elements to sort"
	rnd := Random new.
	array := (1 to: n) collect:[:i| rnd next].
	"First, the heap version"
	time := Time millisecondsToRun:[
		sorted := Heap withAll: array.
		1 to: n do:[:i| 
			sorted removeFirst.
			sorted add: rnd next].
	].
	Transcript cr; show:'Time for Heap: ', time printString,' msecs'.
	"The quicksort version"
	time := Time millisecondsToRun:[
		sorted := SortedCollection withAll: array.
		1 to: n do:[:i| 
			sorted removeFirst.
			sorted add: rnd next].
	].
	Transcript cr; show:'Time for SortedCollection: ', time printString,' msecs'.
! !

!HeapTest methodsFor: 'examples' stamp: 'md 2/12/2006 15:34'!
heapSortExample	"HeapTest new heapSortExample"
	"Sort a random collection of Floats and compare the results with
	SortedCollection (using the quick-sort algorithm) and 
	ArrayedCollection>>mergeSortFrom:to:by: (using the merge-sort algorithm)."
	| n rnd array  time sorted |
	n := 10000. "# of elements to sort"
	rnd := Random new.
	array := (1 to: n) collect:[:i| rnd next].
	"First, the heap version"
	time := Time millisecondsToRun:[
		sorted := Heap withAll: array.
		1 to: n do:[:i| sorted removeFirst].
	].
	Transcript cr; show:'Time for heap-sort: ', time printString,' msecs'.
	"The quicksort version"
	time := Time millisecondsToRun:[
		sorted := SortedCollection withAll: array.
	].
	Transcript cr; show:'Time for quick-sort: ', time printString,' msecs'.
	"The merge-sort version"
	time := Time millisecondsToRun:[
		array mergeSortFrom: 1 to: array size by: [:v1 :v2| v1 <= v2].
	].
	Transcript cr; show:'Time for merge-sort: ', time printString,' msecs'.
! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/18/2009 15:23'!
nonEmpty
	^nonEmpty.! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/18/2009 15:08'!
otherCollection
	^ otherCollection! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 10:34'!
result
	^result.! !

!HeapTest methodsFor: 'running' stamp: 'cyrille.delaunay 3/20/2009 15:56'!
setUp
	element := 33.
	elementNotIn := 66.
	elementTwiceIn := 3.
	expectedSizeAfterReject := 1.
	expectedElementByDetect:= -2.
	
	accessCollection := Heap new.
	accessCollection add: 2.
	accessCollection add: 98.
	accessCollection add: 4.
	accessCollection add: 25.
	accessCollection add: 1.	
	
	collectionWithElement := Heap new.
	{4 . 5 . 6 .2 . 1 . 1 . self element} do: [:nb | collectionWithElement add: nb].
	
	collection := Heap new.
	collection add: 1.
	collection add: -2.
	collection add: 3.
	collection add: 1.
	
	otherCollection := Heap new
		add: 1;
		add: 20;
		add: 30; yourself.
		
	empty := Heap new. 

	nonEmpty := Heap new.
	nonEmpty add: self valuePutIn.
	nonEmpty add: self element.
	nonEmpty add: self elementTwiceIn.
	nonEmpty add: self elementTwiceIn.
	
	
	doWithoutNumber:= 3.
	
	
	
	
	result := collection collect: [: each | each+1].
	
	speciesClass := Heap.! !

!HeapTest methodsFor: 'requirements' stamp: 'cyrille.delaunay 3/20/2009 11:26'!
speciesClass
	
	^ speciesClass! !

!HeapTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/2/2009 11:53'!
test0FixtureRequirementsOfTGrowableTest
	self 
		shouldnt: [ self empty ]
		raise: Exception.
	self 
		shouldnt: [ self nonEmpty ]
		raise: Exception.
	self 
		shouldnt: [ self element ]
		raise: Exception.
	self 
		shouldnt: [ self elementNotInForOccurrences ]
		raise: Exception.
	self assert: self empty isEmpty.
	self deny: self nonEmpty isEmpty.
	self assert: (self nonEmpty includes: self element).
	self deny: (self nonEmpty includes: self elementNotInForOccurrences)! !

!HeapTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:07'!
test0IndexAccessingTest
	self 
		shouldnt: [ self accessCollection ]
		raise: Error.
	self assert: self accessCollection size = 5.
	self 
		shouldnt: [ self subCollectionNotIn ]
		raise: Error.
	self subCollectionNotIn 
		detect: [ :each | (self accessCollection includes: each) not ]
		ifNone: [ self assert: false ].
	self 
		shouldnt: [ self elementNotInForIndexAccessing ]
		raise: Error.
	self deny: (self accessCollection includes: self elementNotInForIndexAccessing).
	self 
		shouldnt: [ self elementInForIndexAccessing ]
		raise: Error.
	self assert: (self accessCollection includes: self elementInForIndexAccessing).
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self deny: each class = SmallInteger ]! !

!HeapTest methodsFor: 'testing' stamp: 'zz 12/7/2005 19:25'!
test1
	| data h |

	"The first element of each array is the sort value, and the second will be updated by the heap with the index of the element within the heap."
	data :=  (1 to: 8) collect: [:i | {i*2. 0}].

	"Repeat with different data ordering."
	5 timesRepeat: [
		h := Heap new sortBlock: [:e1 :e2 | e1 first < e2 first].
		h indexUpdateBlock: [:array :index | array at: 2 put: index].

		data shuffled do: [:d | h add: d].
		data do: [:d | self should: (h at: d second) == d].
	]! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/28/2006 09:37'!
testAdd
	"self run: #testAdd"

	| heap |
	heap := Heap new.
	self assert: heap size = 0.
	heap add: 3.
	self assert: heap size = 1.
	self assert: heap isEmpty not.
	self assert: heap first = 3.
	self assert: (heap at: 1) = 3.
	heap add: 2.
	self assert: heap size = 2.
	self assert: heap first = 2.
	self assert: (heap at: 2) = 3.
	! !

!HeapTest methodsFor: 'tests - growable' stamp: 'delaunay 4/2/2009 11:53'!
testAddNonEmptyGrowsWhenNewElement
	"self debug: #testAddNonEmptyGrowsWhenNewElement"
	| oldSize |
	oldSize := self nonEmpty size.
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self nonEmpty add: self elementNotInForOccurrences.
	self assert: self nonEmpty size > oldSize! !

!HeapTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/6/2009 15:24'!
testAllButFirstElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButFirst: 2.
	1 
		to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i + 2) ].
	self assert: abf size + 2 = col size! !

!HeapTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/6/2009 15:25'!
testAllButLastElements
	"self debug: #testAllButFirst"
	| abf col |
	col := self accessCollection.
	abf := col allButLast: 2.
	1 
		to: abf size
		do: [ :i | self assert: (abf at: i) = (col at: i) ].
	self assert: abf size + 2 = col size! !

!HeapTest methodsFor: 'basic tests' stamp: 'sd 4/8/2006 21:37'!
testAt
	"self run: #testAt"

	| heap |
	heap := Heap new.
	heap add: 3.
	self assert: (heap at: 1) = 3.
	self should: [heap at: 2] raise: Error.
	heap add: 4.
	self assert: (heap at: 1) = 3.
	self assert: (heap at: 2) = 4.

	! !

!HeapTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/10/2009 16:20'!
testAtWrap
	"self debug: #testAt"
	"
	self assert: (self accessCollection at: 1) = 1.
	self assert: (self accessCollection at: 2) = 2.
	"
	| index |
	index := self accessCollection indexOf: self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index + self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: index - self accessCollection size) = self elementInForIndexAccessing.
	self assert: (self accessCollection atWrap: 1 + self accessCollection size) = (self accessCollection at: 1)! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 4/3/2006 11:22'!
testDo
	"self run: #testDo"

	| heap coll |
	heap := Heap withAll: #(1 3 5).
	coll := OrderedCollection new.
	
	heap do: [:each | coll add: each].
	
	self assert: coll = #(1 3 5) asOrderedCollection.
! !

!HeapTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 13:48'!
testDoSeparatedBy
	| string expectedString beforeFirst |
	string := ''.
	self collection 
		do: [ :each | string := string , each asString ]
		separatedBy: [ string := string , '|' ].
	expectedString := ''.
	beforeFirst := true.
	self collection do: 
		[ :each | 
		beforeFirst = true 
			ifTrue: [ beforeFirst := false ]
			ifFalse: [ expectedString := expectedString , '|' ].
		expectedString := expectedString , each asString ].
	self assert: expectedString = string! !

!HeapTest methodsFor: 'testing' stamp: 'md 2/12/2006 15:35'!
testExamples
	self shouldnt: [self heapExample] raise: Error.
	self shouldnt: [self heapSortExample] raise: Error.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/28/2006 09:36'!
testFirst
	"self run: #testFirst"
	| heap |
	heap := Heap new.
	heap add: 5.
	heap add: 12.
	heap add: 1.
	self assert: heap first = 1.
	heap removeFirst.
	self assert: heap first = 5.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 4/3/2006 11:27'!
testHeap
	"self run: #testHeap"

	| heap |
	heap := Heap new.
	self assert: heap isHeap.
	
	self assert: heap isEmpty.
	heap add: 1.
	self assert: heap isEmpty not
	
! !

!HeapTest methodsFor: 'tests - indexable access' stamp: 'delaunay 4/9/2009 16:25'!
testLastIndexOfStartingAt
	"self debug: #testLastIndexOf"
	| collection |
	collection := self accessCollection copyWith: (self accessCollection at: 1).
	self assert: (collection lastIndexOf: (self accessCollection at: 1)) = collection size.
	self assert: (collection lastIndexOf: self elementNotInForIndexAccessing) = 0! !

!HeapTest methodsFor: 'tests - iterate' stamp: 'delaunay 4/2/2009 11:53'!
testRejectNoReject
	| res |
	res := self collection reject: [ :each | each = self elementNotInForOccurrences ].
	self assert: res size = self collection size! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 2/15/2006 09:55'!
testRemove
	"self run: #testRemove"
	
	| heap |
	heap := Heap new.
	self should: [heap removeFirst] raise: Error.
	heap add: 5.
	self shouldnt: [heap removeFirst] raise: Error.
	self assert: heap size = 0.
	heap add: 5.
	self should: [heap removeAt: 2] raise: Error.! !

!HeapTest methodsFor: 'basic tests' stamp: 'nm 4/3/2006 11:17'!
testSortBlock
	"self run: #testSortBlock"

	| heap |
	heap := Heap withAll: #(1 3 5).
	self assert: heap = #(1 3 5).
	
	heap sortBlock: [ :e1 :e2 | e1 >= e2 ].
	self assert: heap = #(5 3 1)
! !

!HeapTest methodsFor: 'parameters' stamp: 'cyrille.delaunay 3/20/2009 13:22'!
valuePutIn
	"the value that we will put in the non empty collection"
	
	^ 7! !

ClassTestCase subclass: #ReadWriteStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!ReadWriteStreamTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class ReadWriteStream.
Unit tests are a good way to exercise the
functionality of your system in a repeatable and
automatic manner. They are therefore recommended if
you plan to release anything. For more information,
see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ReadWriteStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:24'!
testConstructionUsingWith
	"Use the with: constructor."

	| aStream |
	aStream := ReadWriteStream with: #(1 2).
	self assert: (aStream contents = #(1 2)) description: 'Ensure correct initialization.'! !

!ReadWriteStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:24'!
testNew

	self should: [ReadWriteStream new] raise: Error.! !

ClassTestCase subclass: #StringTest
	uses: TIncludesTest + TCloneTest + TCopyTest + TSetAritmetic
	instanceVariableNames: 'string emptyString nonEmptyString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!StringTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class String. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

StringTest class
	uses: TIncludesTest classTrait + TCloneTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait
	instanceVariableNames: ''!

!StringTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:09'!
anotherElementNotIn
	^ $k! !

!StringTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:09'!
collection
	^ 'ghj'! !

!StringTest methodsFor: 'test - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:57'!
collectionClass

	^ String! !

!StringTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 10:50'!
collectionInForIncluding
	self explicitRequirement! !

!StringTest methodsFor: 'requirements' stamp: 'delaunay 3/30/2009 10:37'!
collectionNotIncluded
	^ self explicitRequirement! !

!StringTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	^ self explicitRequirement! !

!StringTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:24'!
collectionWithElementsToRemove
	^ self explicitRequirement! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ $z! !

!StringTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ $u! !

!StringTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:08'!
empty
	^ emptyString! !

!StringTest methodsFor: 'tests - indexof' stamp: 'on 5/13/2008 16:34'!
findFirstInString: aString fromString: searchString

	^ String findFirstInString: aString inSet: (CharacterSet newFrom: searchString) byteArrayMap startingAt: 1! !

!StringTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:08'!
nonEmpty
	^ nonEmptyString! !

!StringTest methodsFor: 'initialization' stamp: 'stephane.ducasse 11/21/2008 15:08'!
setUp

	string := 'Hi, I am a String'.
	emptyString := ''.
	nonEmptyString := 'abcdae'! !

!StringTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: self collectionWithElementsToRemove
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: self elementToAdd
		raise: Error! !

!StringTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:07'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each isString ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:26'!
testAsInteger

	self assert: '1796exportFixes-tkMX' asInteger = 1796.
	self assert: 'donald' asInteger isNil.
	self assert: 'abc234def567' asInteger = 234.
	self assert: '-94' asInteger = -94.
	self assert: 'foo-bar-92' asInteger = -92! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testAsSmalltalkComment

	| exampleStrings  |
	exampleStrings := #(
		''
		' '
		'"'
		'""'
		'"""'
		'abc"abc'
		'abc""abc'
		'abc"hello"abc'
		'abc"'
		'"abc' ).

	"check that the result of scanning the comment is empty"
	exampleStrings do: [ :s |
		| tokens  |
		tokens :=  Scanner new scanTokens: s asSmalltalkComment.
		self assert: (tokens isEmpty) ].

	"check that the result has the same non-quote characters as the original"
	exampleStrings do: [ :s |
		self assert: ( (s copyWithout: $") = (s asSmalltalkComment copyWithout: $"))].

	"finnaly, test for some common kinds of inputs"
	self assert: ( 'abc' asSmalltalkComment = '"abc"').
	self assert: ( 'abc"abc' asSmalltalkComment = '"abc""abc"').
	self assert: ('abc""abc' asSmalltalkComment = '"abc""abc"' ).
		! !

!StringTest methodsFor: 'tests - accessing' stamp: 'sd 6/5/2005 09:27'!
testAt

	self assert: (string at: 1) = $H.! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testCapitalized

	| uc lc empty |		
	uc := 'MElViN'.
	lc := 'mElViN'.
	empty := ' '.
	self assert:  lc capitalized = uc.
	self assert: uc capitalized = uc.
	"the string gets copied"
	self deny: uc capitalized == uc.
	self deny: empty capitalized == empty.! !

!StringTest methodsFor: 'test-comparing' stamp: 'lr 7/7/2006 11:32'!
testComparing
	self assert: 'foo' < 'foo:'.
	self assert: 'foo' < 'fooBar'.
	self assert: 'foo' <= 'foo:'.
	self assert: 'foo' <= 'fooBar'.
	self assert: 'foo:' > 'foo'.
	self assert: 'fooBar' > 'foo'.
	self assert: 'foo:' >= 'foo'.
	self assert: 'fooBar' >= 'foo'! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!StringTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!StringTest methodsFor: 'testing' stamp: 'md 3/16/2006 22:12'!
testEquality

	self assert: 'abc' = 'abc' asWideString.
	self assert: 'abc' asWideString = 'abc'.
	self assert: ((ByteArray with: 97 with: 0 with: 0 with: 0) asString ~= 'a000' asWideString).
	self assert: ('a000' asWideString ~= (ByteArray with: 97 with: 0 with: 0 with: 0) asString).

	self assert: ('abc' sameAs: 'aBc' asWideString).
	self assert: ('aBc' asWideString sameAs: 'abc').
	self assert: ((ByteArray with: 97 with: 0 with: 0 with: 0) asString 
						sameAs: 'Abcd' asWideString) not.
	self assert: ('a000' asWideString sameAs: 
					(ByteArray with: 97 with: 0 with: 0 with: 0) asString) not.! !

!StringTest methodsFor: 'tests - indexof' stamp: 'on 5/14/2008 13:27'!
testFindFirstInString
	
	"These tests are more specific than thsoe in testIndexOf."
	
	"more boundary tests"
	self assert: (self findFirstInString: '' fromString: '') = 0.
	self assert: (self findFirstInString: 'x' fromString: '') = 0.
	self assert: (self findFirstInString: '' fromString: 'x') = 0.
	
	self assert: (self findFirstInString: 'x' fromString: 'x') = 1. "<- FIXED"
	
	self assert: (self findFirstInString: 'hello' fromString: 'hello') = 1.
	self assert: (self findFirstInString: 'hello' fromString: 'ello') = 2.
	self assert: (self findFirstInString: 'hello' fromString: 'llo') = 3.
	self assert: (self findFirstInString: 'hello' fromString: 'o') = 5. "<- FIXED"
	self assert: (self findFirstInString: 'hello' fromString: 'x') = 0.
! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy01

	| tokens |
	string := 'this, is, "a, test"'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size == 3! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:20'!
testFindTokensEscapedBy02

	| tokens |
	string := ''.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens isEmpty! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy03

	| tokens |
	string := 'this, is, a, test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size == 4! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy04

	| tokens |
	string := 'this, is, a"," test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size == 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy05

	| tokens |
	string := 'this, /is, a"," test/'.
	tokens := string findTokens: ',#' escapedBy: '"/'.
	self assert: tokens size = 2.
	self assert: tokens first = 'this'.
	self assert: tokens second = ' is, a"," test'.! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy06

	| tokens |
	string := 'this, is, "a, test'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size == 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy07

	| tokens |
	string := 'a:b::c'.
	tokens := string findTokens: ':' escapedBy: '"'.
	self assert: tokens size == 4.
	self assert: tokens first = 'a'.
	self assert: tokens second = 'b'.
	self assert: tokens third = ''.
	self assert: tokens fourth = 'c'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy08

	| tokens |
	string := 'this, is, ##a, test'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size == 4.
	self assert: tokens third = ' a'.
	self assert: tokens fourth = ' test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy09

	| tokens |
	string := 'this, is, ###a, test#'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size == 3.
	self assert: tokens third = ' #a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy10

	| tokens |
	string := 'this, is, ###a, test'.
	tokens := string findTokens: ',' escapedBy: '#'.
	self assert: tokens size == 3.
	self assert: tokens third = ' #a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy11

	| tokens |
	string := 'this, is, """a, test"'.
	tokens := string findTokens: ',' escapedBy: '"'.
	self assert: tokens size == 3.
	self assert: tokens third = ' "a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy12

	| tokens |
	string := 'one, two# three; four. five'.
	tokens := string findTokens: ',#;.' escapedBy: '"'.
	self assert: tokens size == 5.
	self assert: tokens third = ' three'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy13

	| tokens |
	string := 'one, two# three; four. five'.
	tokens := string findTokens: ',#;.' escapedBy: nil.
	self assert: tokens size == 5.
	self assert: tokens third = ' three'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy14

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: ',#;.' escapedBy: '"&'.
	self assert: tokens size == 3.
	self assert: tokens second = ' two# three'.
	self assert: tokens third = ' four. five'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy15

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: nil escapedBy: '"&'.
	self assert: tokens size = 1.
	self assert: tokens first = 'one, two# three; four. five'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy16

	| tokens |
	string := 'one, "two# three"; &four. five&'.
	tokens := string findTokens: nil escapedBy: nil.
	self assert: tokens size = 1.
	self assert: tokens first = string! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy21

	| tokens |
	string := 'this, is, "a, test"'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size == 3! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy22

	| tokens |
	string := ''.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size = 0! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy23

	| tokens |
	string := 'this, is, a, test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size == 4! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy24

	| tokens |
	string := 'this, is, a"," test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size == 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'fbs 2/13/2006 22:19'!
testFindTokensEscapedBy25

	| tokens |
	string := 'this, /is, a"," test/'.
	tokens := string findTokens: $, escapedBy: $/.
	self assert: tokens size = 2.
	self assert: tokens first = 'this'.
	self assert: tokens second = ' is, a"," test'.! !

!StringTest methodsFor: 'testing - tokenizing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testFindTokensEscapedBy26

	| tokens |
	string := 'this, is, "a, test'.
	tokens := string findTokens: $, escapedBy: $".
	self assert: tokens size == 3.
	self assert: tokens third = ' a, test'! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!StringTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!StringTest methodsFor: 'tests - indexof' stamp: 'nice 3/15/2007 21:11'!
testIndexOf
	
	"test for http://bugs.impara.de/view.php?id=3574"
	self assert: ('abc-' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 4.
	self assert: ('ab7' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 3.
	self assert: ('a2c' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 2.
	self assert: ('3bc' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 1.
	self assert: ('abc' asWideString indexOfAnyOf: (CharacterSet newFrom: ' -0123456789')) = 0.
	
	"extension to wide characters"
	self assert: ((String with: 803 asCharacter with: 811 asCharacter) indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: 812 asCharacter))) = 2.
	
	self assert: ('abc' indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: 812 asCharacter))) = 0.
	
	self assert: ('abc' indexOfAnyOf: (CharacterSet newFrom: (String with: 811 asCharacter with: $c))) = 3.
	
	"make sure start index is used in wide string algorithm"
	self assert: ('ab bcd abc' copyWith: 811 asCharacter) substrings = {'ab'. 'bcd'. 'abc' copyWith: 811 asCharacter}.! !

!StringTest methodsFor: 'testing' stamp: 'nice 10/17/2008 23:53'!
testNumArgs
	"This is about http://code.google.com/p/pharo/issues/detail?id=237"
	
	self assert: ('*+-/\~=<>&@%,|' allSatisfy: [:char | (String with: char) numArgs = 1])
		description: 'binary selectors have 1 argument'.
		
	self assert: 'x' numArgs = 0
		description: 'unary selectors have 0 arguments'.
	self assert: 'x0' numArgs = 0
		description: 'unary selectors have 0 arguments'.
	self assert: 'yourself' numArgs = 0
		description: 'unary selectors have 0 arguments'.
		
	self assert: 'x:' numArgs = 1
		description: 'keyword selectors have as many elements as colons characters'.
	self assert: 'x:y:' numArgs = 2
		description: 'keyword selectors have as many elements as colons characters'.
	self assert: 'at:put:' numArgs = 2
		description: 'keyword selectors have as many elements as colons characters'.
		
	self assert: 'at:withoutTrailingColon' numArgs = -1
		description: 'keyword selectors should have a trailing colon character'.
		
	self assert: ':x' numArgs = -1
		description: 'keyword selectors cannot begin with a colon character'.
		
	self assert: 'x::y:' numArgs = -1
		description: 'keyword selectors cannot have two consecutive colon characters'.
	
	self assert: '0x' numArgs = -1
		description: 'selectors cannot begin with a digit'.
	
	"This one is known to fail...
	self assert: 'x:0:' numArgs = -1
		description: 'no keyword selector part can begin with a digit'.
	"
	
	! !

!StringTest methodsFor: 'testing - converting' stamp: 'KR 06/24/2005 11:21'!
testPercentEncodingJa
	| leading hiraA hiraO hiraAO encodedHiraA encodedHiraO encodedHiraAO |

    "Make Japanese String from unicode. see http://www.unicode.org/charts/PDF/U3040.pdf"
     leading := JapaneseEnvironment leadingChar.
	hiraA := (Character leadingChar: leading code: 16r3042) asString.  "HIRAGANA LETTER A"
	hiraO := (Character leadingChar: leading code: 16r304A) asString.  "HIRAGANA LETTER O"
	hiraAO := hiraA, hiraO.

	"Percent Encoded Japanese String"
	encodedHiraA := hiraA encodeForHTTP.
	self assert: encodedHiraA = '%E3%81%82'.
	encodedHiraO := hiraO encodeForHTTP.
	self assert: encodedHiraO = '%E3%81%8A'.
	encodedHiraAO := hiraAO encodeForHTTP.
	self assert: encodedHiraAO =  '%E3%81%82%E3%81%8A'.

     "without percent encoded string"
	self assert: '' unescapePercents = ''.
	self assert: 'abc' unescapePercents = 'abc'.	"latin1 character"
	self assert: hiraAO unescapePercents = hiraAO.  "multibyte character"

	"encoded latin1 string"
	self assert: '%61' unescapePercents = 'a'.
	self assert: '%61%62%63' unescapePercents = 'abc'.

	"encoded multibyte string"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: encodedHiraA unescapePercents = hiraA.
		self assert: encodedHiraAO unescapePercents = hiraAO].

	"mixed string"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: (encodedHiraAO,'a') unescapePercents = (hiraAO, 'a').
		self assert: ('a', encodedHiraA) unescapePercents = ('a', hiraA).
		self assert: ('a', encodedHiraA, 'b')  unescapePercents = ('a', hiraA, 'b').
		self assert: ('a', encodedHiraA, 'b', encodedHiraO) unescapePercents = ('a', hiraA, 'b', hiraO).
		self assert: (encodedHiraA, encodedHiraO, 'b', encodedHiraA) unescapePercents = (hiraA, hiraO, 'b', hiraA)].


	"for Seaside"
	Locale currentPlatform: (Locale isoLanguage: 'ja') during: [ 
		self assert: (encodedHiraA, '+', encodedHiraO) unescapePercents = (hiraA, ' ', hiraO)].

! !

!StringTest methodsFor: 'testing - instance creation' stamp: 'dc 3/21/2007 11:22'!
testSpace
	"self debug: #testSpace"
	
	string := String new.
	self assert: string size = 0. "instead of #isEmpty to be consistent with the following test"
	
	string := String space.
	self assert: string size = 1.
	self assert: string = ' '! !

!StringTest methodsFor: 'tests - converting' stamp: 'ky 7/8/2006 15:28'!
testUnescapePercents
	self assert: '' unescapePercents = ''.
	self assert: 'x' unescapePercents = 'x'.

	self assert: '+' unescapePercents = ' '.
	self assert: 'x+' unescapePercents = 'x '.
	self assert: '+x' unescapePercents = ' x'.
	self assert: 'x+x' unescapePercents = 'x x'.

	self assert: '%' unescapePercents = '%'.
	self assert: '%3' unescapePercents = '%3'.
	self assert: '%3C' unescapePercents = '<'.
	
	self assert: '%3Cx%3E4%3C%2Fx%3E' unescapePercents = '<x>4</x>'.
	
	self assert: '!!@#$%25%5E&*()%7B%7D%5B%5D=:/;?+''%22' unescapePercents  = '!!@#$%^&*(){}[]=:/;? ''"'.
	self assert: '!!%40%23%24%25%5E%26*()%7B%7D%5B%5D%3D%3A%2F%3B%3F%2B''%22' unescapePercents  = '!!@#$%^&*(){}[]=:/;?+''"'.
	self assert: '%21@%23%24%25%5E%26*%28%29%7B%7D%5B%5D%3D%3A/%3B%3F+%27%22' unescapePercents = '!!@#$%^&*(){}[]=:/;? ''"'! !

!StringTest methodsFor: 'tests - converting' stamp: 'ky 7/8/2006 18:01'!
testUnescapePercentsWithTextEncoding
	| leading kataTe kataSu kataTo |
	leading := JapaneseEnvironment leadingChar.
	"Katakana letter Te"
	kataTe := (Character leadingChar: leading code: 12486) asString.
	"Katakana letter Su"
	kataSu := (Character leadingChar: leading code: 12473) asString.
	"Katakana letter To"
	kataTo := (Character leadingChar: leading code: 12488) asString.
	self assert: ('%83e%83X%83g' unescapePercentsWithTextEncoding: 'shift_jis')
			= (kataTe , kataSu , kataTo).
	self assert: ('%83e%83X%83g%20and%20%83e%83X%83g' unescapePercentsWithTextEncoding: 'shift_jis')
			= (kataTe , kataSu , kataTo , ' and ' , kataTe , kataSu , kataTo)! !

!StringTest methodsFor: 'tests - converting' stamp: 'dc 4/10/2007 09:35'!
testUpTo
	"self debug: #testUpTo"
	self assert: #up:to: keywords = #(up: to:).
	self assert: #copy:from:to: keywords = #(copy: from: to:).
	self assert: #up keywords = #(up).
	self assert: #at: keywords = #(at:).
	! !

!StringTest methodsFor: 'tests - converting' stamp: 'sd 6/5/2005 09:27'!
testWithFirstCharacterDownshifted

	| uc lc empty |		
	uc := 'MElViN'.
	lc := 'mElViN'.
	empty := ' '.
	self assert:  uc withFirstCharacterDownshifted = lc.
	self assert: lc withFirstCharacterDownshifted = lc.
	"the string gets copied"
	self deny: lc withFirstCharacterDownshifted == lc.
	self deny: empty withFirstCharacterDownshifted == empty.! !

!StringTest methodsFor: 'tests - converting' stamp: 'md 2/5/2007 15:21'!
testZipped
	| compressed |
	
	compressed := 'hello' zipped.
	self assert: (compressed unzipped = 'hello').! !

ClassTestCase subclass: #SymbolTest
	uses: TIncludesTest + TCloneTest - {#testCopyCreatesNewObject} + TCopyPreservingIdentityTest + TCopyTest + TSetAritmetic - {#testDifferenceWithNonNullIntersection}
	instanceVariableNames: 'nonEmptySymbol emptySymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!SymbolTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Symbol. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

SymbolTest class
	uses: TIncludesTest classTrait + TCloneTest classTrait + TCopyPreservingIdentityTest classTrait + TCopyTest classTrait + TSetAritmetic classTrait
	instanceVariableNames: ''!

!SymbolTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:24'!
anotherElementNotIn
	^ $k! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:24'!
collection
	^ #'ghj'! !

!SymbolTest methodsFor: 'test - set arithmetic' stamp: 'stephane.ducasse 12/20/2008 21:57'!
collectionClass

	^ Symbol! !

!SymbolTest methodsFor: 'requirements' stamp: 'delaunay 4/9/2009 10:50'!
collectionInForIncluding
	self explicitRequirement! !

!SymbolTest methodsFor: 'requirements' stamp: 'delaunay 3/30/2009 10:37'!
collectionNotIncluded
	^ self explicitRequirement! !

!SymbolTest methodsFor: 'requirements' stamp: 'delaunay 4/15/2009 14:07'!
collectionOfFloat
	^ self explicitRequirement! !

!SymbolTest methodsFor: 'requirements' stamp: 'delaunay 4/17/2009 15:24'!
collectionWithElementsToRemove
	^ self explicitRequirement! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
elementNotInForOccurrences
	^ $z! !

!SymbolTest methodsFor: 'test - copy' stamp: 'delaunay 4/17/2009 15:26'!
elementToAdd
	^ $u! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:23'!
empty
	^ emptySymbol ! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'stephane.ducasse 11/21/2008 15:23'!
nonEmpty
	^ nonEmptySymbol 	! !

!SymbolTest methodsFor: 'setup' stamp: 'stephane.ducasse 11/21/2008 15:23'!
setUp

	emptySymbol :=  #''.
	nonEmptySymbol := #'abcdae'
	! !

!SymbolTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/17/2009 15:26'!
test0CopyTest
	self 
		shouldnt: self empty
		raise: Error.
	self assert: self empty size = 0.
	self 
		shouldnt: self nonEmpty
		raise: Error.
	self assert: (self nonEmpty size = 0) not.
	self 
		shouldnt: self collectionWithElementsToRemove
		raise: Error.
	self assert: (self collectionWithElementsToRemove size = 0) not.
	self 
		shouldnt: self elementToAdd
		raise: Error! !

!SymbolTest methodsFor: 'tests - fixture' stamp: 'delaunay 4/15/2009 14:07'!
test0FixtureIncludeTest
	| elementIn |
	self 
		shouldnt: [ self nonEmpty ]
		raise: Error.
	self deny: self nonEmpty isEmpty.
	self 
		shouldnt: [ self elementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self anotherElementNotIn ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self anotherElementNotIn ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = false.
	self 
		shouldnt: [ self collection ]
		raise: Error.
	"self shouldnt: [self collectionInForIncluding  ] raise: Error."
	"collectionIn:=false.
	index:=1.
	1 to: self nonEmpty size do:
		[:i|
		collectionIn = false
			ifTrue:[(self nonEmpty at:i)=(self collectionInForIncluding at:index)
				ifTrue:[
					index=self collectionInForIncluding 
						ifTrue:[collectionIn := true].					
					index:=index+1.]
				ifFalse:[index:=1].			
				]
		].
	self assert: collectionIn=true."
	self 
		shouldnt: [ self empty ]
		raise: Error.
	self assert: self empty isEmpty.
	self 
		shouldnt: [ self collectionOfFloat ]
		raise: Error.
	self collectionOfFloat do: [ :each | self assert: each isString ].
	self 
		shouldnt: [ self elementInForIncludesTest ]
		raise: Error.
	elementIn := true.
	self nonEmpty 
		detect: [ :each | each = self elementInForIncludesTest ]
		ifNone: [ elementIn := false ].
	self assert: elementIn = true! !

!SymbolTest methodsFor: 'tests' stamp: 'md 9/6/2005 20:02'!
testAsMutator

	self assert: #x asMutator = #x:.
	self assert: #x asMutator isSymbol! !

!SymbolTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:29'!
testCapitalized

	| uc lc |		
	uc := #MElViN.
	lc := #mElViN.
	self assert:  lc capitalized = uc.
	self assert: uc capitalized = uc.
! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWith
	"self debug: #testCopyWith"
	| res |
	res := self empty copyWith: self elementToAdd.
	self assert: res size = (self empty size + 1).
	self assert: (res includes: self elementToAdd)! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyEmptyWithout
	"self debug: #testCopyEmptyWithout"
	| res |
	res := self empty copyWithout: self elementToAdd.
	self assert: res size = self empty size.
	self deny: (res includes: self elementToAdd)! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyEmptyWithoutAll
	"self debug: #testCopyEmptyWithoutAll"
	| res |
	res := self empty copyWithoutAll: self collectionWithElementsToRemove.
	self assert: res size = self empty size.
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ]! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWith
	"self debug: #testCopyNonEmptyWith"
	| res |
	res := self nonEmpty copyWith: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self assert: (res includes: self elementToAdd).
	self nonEmpty do: [ :each | res includes: each ]! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:24'!
testCopyNonEmptyWithoutAll
	"self debug: #testCopyNonEmptyWithoutAll"
	| res |
	res := self nonEmpty copyWithoutAll: self collectionWithElementsToRemove.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self collectionWithElementsToRemove do: [ :each | self deny: (res includes: each) ].
	self nonEmpty do: 
		[ :each | 
		(self collectionWithElementsToRemove includes: each) ifFalse: [ self assert: (res includes: each) ] ]! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 3/30/2009 10:33'!
testCopyNonEmptyWithoutAllNotIncluded
	! !

!SymbolTest methodsFor: 'tests - copy' stamp: 'delaunay 4/17/2009 15:26'!
testCopyNonEmptyWithoutNotIncluded
	"self debug: #testCopyNonEmptyWithoutNotIncluded"
	| res |
	res := self nonEmpty copyWithout: self elementToAdd.
	"here we do not test the size since for a non empty set we would get a problem.
	Then in addition copy is not about duplicate management. The element should 
	be in at the end."
	self nonEmpty do: [ :each | self assert: (res includes: each) ]! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/15/2009 14:07'!
testIdentityIncludes
	"self debug: #testIndentityIncludes"
	| collection element |
	element := self elementInForIncludesTest copy.
	collection := self collectionOfFloat copyWith: element.
	self deny: (self collectionOfFloat identityIncludes: element).
	self assert: (collection identityIncludes: element)! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAllOfNoneThere
	"self debug: #testIncludesAllOfNoneThere'"
	self deny: (self empty includesAllOf: self collection).
	self deny: (self nonEmpty includesAllOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesAnyOfNoneThere
	"self debug: #testIncludesAnyOfNoneThere'"
	self deny: (self nonEmpty includesAnyOf: self empty).
	self deny: (self nonEmpty includesAnyOf: { 
				(self elementNotIn).
				(self anotherElementNotIn)
			 })! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/2/2009 11:53'!
testIncludesElementIsNotThere
	"self debug: #testIncludesElementIsNotThere"
	self deny: (self nonEmpty includes: self elementNotInForOccurrences).
	self assert: (self nonEmpty includes: self nonEmpty anyOne).
	self deny: (self empty includes: self elementNotInForOccurrences)! !

!SymbolTest methodsFor: 'tests - includes' stamp: 'delaunay 4/9/2009 10:44'!
testIncludesSubstringAnywhere
	"self debug: #testIncludesSubstringAnywher'"
	self assert: (self empty includesAllOf: self empty).
	self assert: (self nonEmpty includesAllOf: {  (self nonEmpty anyOne)  }).
	self assert: (self nonEmpty includesAllOf: self nonEmpty)! !

!SymbolTest methodsFor: 'tests' stamp: 'md 2/16/2006 17:17'!
testNumArgs2
    "TODO: need to be extended to support shrinking and for selectors like #+ " 
	
	self assert: (#test numArgs: 0) = #test.
	self assert: (#test numArgs: 1) = #test:.
	self assert: (#test numArgs: 2) = #test:with:.
	self assert: (#test numArgs: 3) = #test:with:with:.
	

	self assert: (#test: numArgs: 0) = #test:.
	self assert: (#test: numArgs: 1) = #test:.
	self assert: (#test: numArgs: 2) = #test:with:.
	self assert: (#test: numArgs: 3) = #test:with:with:.
	
	self assert: (#test:with: numArgs: 0) = #test:with:.
	self assert: (#test:with: numArgs: 1) = #test:with:.
	self assert: (#test:with: numArgs: 2) = #test:with:.
	self assert: (#test:with: numArgs: 3) = #test:with:with:.
	self assert: (#test:with: numArgs: 4) = #test:with:with:with:.
	
	self assert: (#test:with:with: numArgs: 0) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 1) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 2) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 3) = #test:with:with:.
	self assert: (#test:with:with: numArgs: 4) = #test:with:with:with:.! !

!SymbolTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:29'!
testWithFirstCharacterDownshifted

	| uc lc empty |		
	uc := #MElViN.
	lc := #mElViN.
	empty := #' '.
	self assert:  uc withFirstCharacterDownshifted = lc.
	self assert: lc withFirstCharacterDownshifted = lc.
	
! !

ClassTestCase subclass: #WeakIdentityKeyDictionaryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Weak'!

!WeakIdentityKeyDictionaryTest methodsFor: 'tests' stamp: 'md 7/15/2006 18:02'!
testNoNils
	| d |
	d := WeakIdentityKeyDictionary new
	at: 'hello' copy put: 'world';
		yourself.
	Smalltalk garbageCollect.
	self deny: (d keys includes: nil)! !

ClassTestCase subclass: #WideStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!WideStringTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class String. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!WideStringTest methodsFor: 'tests - converting' stamp: 'ar 4/12/2005 17:36'!
testAsInteger
	self assert: '1796exportFixes-tkMX' asWideString asInteger = 1796.
	self assert: 'donald' asWideString asInteger isNil.
	self assert: 'abc234def567' asWideString asInteger = 234.
	self assert: '-94' asWideString asInteger = -94.
	self assert: 'foo-bar-92' asWideString asInteger = -92.

	self assert: '1796exportFixes-tkMX' asWideString asSignedInteger = 1796.
	self assert: 'donald' asWideString asSignedInteger isNil.
	self assert: 'abc234def567' asWideString asSignedInteger = 234.
	self assert: '-94' asWideString asSignedInteger = -94.
	self assert: 'foo-bar-92' asWideString asSignedInteger = -92.

	self assert: '1796exportFixes-tkMX' asWideString asUnsignedInteger = 1796.
	self assert: 'donald' asWideString asUnsignedInteger isNil.
	self assert: 'abc234def567' asWideString asUnsignedInteger = 234.
	self assert: '-94' asWideString asUnsignedInteger = 94.
	self assert: 'foo-bar-92' asWideString asUnsignedInteger = 92! !

!WideStringTest methodsFor: 'testing' stamp: 'nice 4/2/2008 23:49'!
testAtPut
	"Non regression test for http://bugs.squeak.org/view.php?id=6998"
	
	| w1 |
	w1 := WideString with: (Unicode value: 402) with: $a with: (Unicode value: 400) with: $b.
	self assert: (w1 at: 2 put: $b) = $b description: 'at:put: should return the put-object'
! !

!WideStringTest methodsFor: 'tests - beginswith' stamp: 'nice 7/28/2007 23:29'!
testBeginsWith
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	| w1 w2 |
	self assert: ('abc' beginsWith: 'ab').
	self assert: ('abc' beginsWith: 'ab' asWideString).
	self assert: ('abc' asWideString beginsWith: 'ab').
	self assert: ('abc' beginsWith: 'aX') not .
	self assert: ('abc' beginsWith: 'AB') not.
	self assert: ('abc' beginsWith: 'AB' asWideString) not .
	self assert: ('ABC' asWideString beginsWith: 'ab') not.

	w1 := WideString with: (Unicode value: 402) with: $a with: (Unicode value: 400) with: $b.
	w2 := WideString with: (Unicode value: 402).
	w1 beginsWith: w2.
! !

!WideStringTest methodsFor: 'tests - match' stamp: 'nice 7/28/2007 23:22'!
testCharactersExactlyMatching
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('abc' charactersExactlyMatching: 'abc') = 3.
	self assert: ('abd' charactersExactlyMatching: 'abc') = 2.
	self assert: ('abc' charactersExactlyMatching: 'abc' asWideString) = 3.
	self assert: ('abd' charactersExactlyMatching: 'abc' asWideString) = 2.
	self assert: ('abc' asWideString charactersExactlyMatching: 'abc') = 3.
	self assert: ('abd' asWideString charactersExactlyMatching: 'abc') = 2.
	self assert: ('abc' asWideString charactersExactlyMatching: 'abc' asWideString) = 3.
	self assert: ('abd' asWideString charactersExactlyMatching: 'abc' asWideString)= 2.
	self assert: ('abc' charactersExactlyMatching: 'ABC') = 0.

! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 7/28/2007 23:19'!
testCompare
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('abc' compare: 'abc') = 2.
	self assert: ('abc' compare: 'abd') = 1.
	self assert: ('abd' compare: 'abc') = 3.
	self assert: ('abc' compare: 'abC') = 2.
	self assert: ('abc' compare: 'abD') = 1.
	self assert: ('abd' compare: 'abC') = 3.
	self assert: ('aBc' compare: 'abC') = 2.
	self assert: ('aBc' compare: 'abD') = 1.
	self assert: ('aDd' compare: 'abC') = 3.
	
	
	self assert: ('abc' compare: 'abc' asWideString) = 2.
	self assert: ('abc' compare: 'abd' asWideString) = 1.
	self assert: ('abd' compare: 'abc' asWideString) = 3.
	self assert: ('abc' compare: 'abC' asWideString) = 2.
	self assert: ('abc' compare: 'abD' asWideString) = 1.
	self assert: ('abd' compare: 'abC' asWideString) = 3.
	self assert: ('aBc' compare: 'abC' asWideString) = 2.
	self assert: ('aBc' compare: 'abD' asWideString) = 1.
	self assert: ('aDd' compare: 'abC' asWideString) = 3.
	
	self assert: ('abc' asWideString compare: 'abc') = 2.
	self assert: ('abc' asWideString compare: 'abd') = 1.
	self assert: ('abd' asWideString compare: 'abc') = 3.
	self assert: ('abc' asWideString compare: 'abC') = 2.
	self assert: ('abc' asWideString compare: 'abD') = 1.
	self assert: ('abd' asWideString compare: 'abC') = 3.
	self assert: ('aBc' asWideString compare: 'abC') = 2.
	self assert: ('aBc' asWideString compare: 'abD') = 1.
	self assert: ('aDd' asWideString compare: 'abC') = 3.
	
	self assert: ('abc' asWideString compare: 'abc' asWideString) = 2.
	self assert: ('abc' asWideString compare: 'abd' asWideString) = 1.
	self assert: ('abd' asWideString compare: 'abc' asWideString) = 3.
	self assert: ('abc' asWideString compare: 'abC' asWideString) = 2.
	self assert: ('abc' asWideString compare: 'abD' asWideString) = 1.
	self assert: ('abd' asWideString compare: 'abC' asWideString) = 3.
	self assert: ('aBc' asWideString compare: 'abC' asWideString) = 2.
	self assert: ('aBc' asWideString compare: 'abD' asWideString) = 1.
	self assert: ('aDd' asWideString compare: 'abC' asWideString) = 3.
	
	self assert: ('abc' compare: 'abc' caseSensitive: true) = 2.
	self assert: ('abc' compare: 'abC' caseSensitive: false) = 2.
	self assert: ('abc' compare: 'abc' asWideString caseSensitive: true) = 2.
	self assert: ('abc' compare: 'abC' asWideString caseSensitive: false) = 2.
	self assert: ('abc' asWideString compare: 'abc' caseSensitive: true) = 2.
	self assert: ('abc' asWideString compare: 'abC' caseSensitive: false) = 2.
	self assert: ('abc' asWideString compare: 'abc' asWideString caseSensitive: true) = 2.
	self assert: ('abc' asWideString compare: 'abC' asWideString caseSensitive: false) = 2.! !

!WideStringTest methodsFor: 'tests - endswith' stamp: 'nice 7/28/2007 23:30'!
testEndsWith
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('abc' endsWith: 'bc').
	self assert: ('abc' endsWith: 'bc' asWideString).
	self assert: ('abc' asWideString endsWith: 'bc').
	self assert: ('abc' endsWith: 'bX') not .
	self assert: ('abc' endsWith: 'BC') not.
	self assert: ('abc' endsWith: 'BC' asWideString) not .
	self assert: ('ABC' asWideString endsWith: 'bc') not.

! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 7/28/2007 23:24'!
testEqual
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: 'abc' = 'abc'.
	self assert: 'abc' = 'abc' asWideString.
	self assert: 'abc' asWideString = 'abc'.
	self assert: 'abc' asWideString = 'abc' asWideString.
	self assert: ('abc' = 'ABC') not.
	self assert: ('abc' = 'ABC' asWideString) not.
	self assert: ('abc' asWideString = 'ABC') not.
	self assert: ('abc' asWideString = 'abc' asWideString).
	self assert: ((ByteArray with: 97 with: 0 with: 0 with: 0) asString ~= 'a000' asWideString).
	self assert: ('a000' asWideString ~= (ByteArray with: 97 with: 0 with: 0 with: 0) asString).! !

!WideStringTest methodsFor: 'tests - substrings' stamp: 'nice 7/28/2007 00:46'!
testFindSubstring
	"This is related to http://bugs.squeak.org/view.php?id=6366
	finding substring in a WideString was broken because matchTable are byte-wise"
	
	| ws1 ws2 |
	
	self assert: ('abcd' findString: 'bc' startingAt: 1) = 2.
	self assert: ('abcd' asWideString findString: 'bc' startingAt: 1) = 2.
	self assert: ('abcd' findString: 'bc' asWideString startingAt: 1) = 2.
	self assert: ('abcd' asWideString findString: 'bc' asWideString startingAt: 1) = 2.
	
	ws1 := 'A' , (WideString with: (Unicode value: 530)) , 'BCD'.
	self assert: (ws1 findString: 'bc' startingAt: 1 caseSensitive: true) = 0.
	self assert: (ws1 findString: 'bc' startingAt: 1 caseSensitive: false) = 3.
	
	ws2 := (WideString with: (Unicode value: 530)) , 'b'.
	self assert: (ws1 findString: ws2 startingAt: 1 caseSensitive: true) = 0.
	self assert: (ws1 findString: ws2 startingAt: 1 caseSensitive: false) = 2.
	
	self assert: ('abc' findString: ws2 startingAt: 1 caseSensitive: true) = 0.
	self assert: ('abc' findString: ws2 startingAt: 1 caseSensitive: false) = 0.! !

!WideStringTest methodsFor: 'tests - match' stamp: 'nice 7/28/2007 23:16'!
testMatch
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
	
	self assert: ('*baz' match: 'mobaz' ).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' ).
	self assert: ('*baz' match: 'mobaz' ).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' ).
	
	self assert: ('*baz' match: 'mobaz' asWideString).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' asWideString).
	self assert: ('*baz' match: 'mobaz' asWideString).
	self assert: ('*foo#zort' match: 'afoo3zortthenfoo3zort' asWideString).
	
	self assert: ('*baz' asWideString match: 'mobaz' ).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' ).
	self assert: ('*baz' asWideString match: 'mobaz' ).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' ).
	
	self assert: ('*baz' asWideString match: 'mobaz' asWideString).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' asWideString).
	self assert: ('*baz' asWideString match: 'mobaz' asWideString).
	self assert: ('*foo#zort' asWideString match: 'afoo3zortthenfoo3zort' asWideString).! !

!WideStringTest methodsFor: 'tests - relation order' stamp: 'nice 7/28/2007 23:14'!
testRelationOrder
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"

	self assert: ('aa' < 'ab').
	self assert: ('aa' <= 'ab').
	self assert: ('aa' <= 'aa').
	self assert: ('ab' > 'aa').
	self assert: ('ab' >= 'aa').
	self assert: ('aa' >= 'aa').
	
	self assert: ('aa' < 'ab' asWideString).
	self assert: ('aa' <= 'ab' asWideString).
	self assert: ('aa' <= 'aa' asWideString).
	self assert: ('ab' > 'aa' asWideString).
	self assert: ('ab' >= 'aa' asWideString).
	self assert: ('aa' >= 'aa' asWideString).
	
	self assert: ('aa' asWideString < 'ab').
	self assert: ('aa' asWideString <= 'ab').
	self assert: ('aa' asWideString <= 'aa').
	self assert: ('ab' asWideString > 'aa').
	self assert: ('ab' asWideString >= 'aa').
	self assert: ('aa' asWideString >= 'aa').
	
	self assert: ('aa' asWideString< 'ab' asWideString).
	self assert: ('aa' asWideString<= 'ab' asWideString).
	self assert: ('aa' asWideString<= 'aa' asWideString).
	self assert: ('ab' asWideString> 'aa' asWideString).
	self assert: ('ab' asWideString >= 'aa' asWideString).
	self assert: ('aa' asWideString>= 'aa' asWideString).! !

!WideStringTest methodsFor: 'tests - relation order' stamp: 'nice 7/28/2007 23:21'!
testRelationOrderWithCase
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"
		
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abc').
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abd').
	self assert: ('ABD' caseInsensitiveLessOrEqual: 'abc') not.
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('ABC' caseInsensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('ABD' caseInsensitiveLessOrEqual: 'abc' asWideString) not.
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abc').
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abd').
	self assert: ('ABD' asWideString caseInsensitiveLessOrEqual: 'abc') not.
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('ABC' asWideString caseInsensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('ABD' asWideString caseInsensitiveLessOrEqual: 'abc' asWideString) not.
	
	
	self assert: ('abc' caseSensitiveLessOrEqual: 'abc').
	self assert: ('abc' caseSensitiveLessOrEqual: 'abd').
	self assert: ('abd' caseSensitiveLessOrEqual: 'abc') not.
	self assert: ('abc' caseSensitiveLessOrEqual: 'abc' asWideString).
	self assert: ('abc' caseSensitiveLessOrEqual: 'abd' asWideString).
	self assert: ('abd' caseSensitiveLessOrEqual: 'abc' asWideString) not.
	self assert: ('abc' asWideString caseSensitiveLessOrEqual: 'abc').
	self assert: ('abc' asWideString caseSensitiveLessOrEqual: 'abd').
	self assert: ('abd' asWideString caseSensitiveLessOrEqual: 'abc') not.
	self assert: ('abc' caseSensitiveLessOrEqual: 'ABC') not.
	! !

!WideStringTest methodsFor: 'tests - compare' stamp: 'nice 7/28/2007 23:25'!
testSameAs
	"from johnmci at http://bugs.squeak.org/view.php?id=5331"

	self assert: ('abc' sameAs: 'aBc' asWideString).
	self assert: ('aBc' asWideString sameAs: 'abc').
	self assert: ((ByteArray with: 97 with: 0 with: 0 with: 0) asString sameAs: 'Abcd' asWideString) not.
	self assert: ('a000' asWideString sameAs: (ByteArray with: 97 with: 0 with: 0 with: 0) asString) not.
	! !

!WideStringTest methodsFor: 'tests - substrings' stamp: 'nice 3/23/2007 01:52'!
testSubstrings
	"this is related to http://bugs.squeak.org/view.php?id=6367"
	
	| w1 w2 |
	w1 := WideString with: 401 asCharacter with: $a with: 402 asCharacter with: $b.
	w2 := WideString with: 403 asCharacter with: 404 asCharacter.

	self assert: w1 substrings first = w1.
	self assert: (w1 , ' ' , w2) substrings size = 2.
	self assert: (w1 , ' ' , w2) substrings last = w2.! !

ClassTestCase subclass: #WriteStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!WriteStreamTest methodsFor: 'testing' stamp: 'dc 2/11/2007 14:14'!
testEnsureASpace
	"self debug: #testEnsureASpace"
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: 'this is a test'.
	stream ensureASpace.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = 'this is a test for WriteStreamTest'.
	
	"Manually put a space and verify there are no 2 consecutive spaces"
	stream := WriteStream on: String new.
	stream nextPutAll: 'this is a test '.
	stream ensureASpace.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = 'this is a test for WriteStreamTest'.! !

!WriteStreamTest methodsFor: 'testing' stamp: 'dc 2/11/2007 14:17'!
testEnsureASpace2
	"self debug: #testEnsureASpace2"
	| stream |
	stream := WriteStream on: String new.
	stream ensureASpace.
	self assert: stream contents = ' '.
	! !

!WriteStreamTest methodsFor: 'testing' stamp: 'dc 2/11/2007 14:26'!
testEnsureEndsWith
	"self debug: #testEnsureEndsWith"
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: 'this is a test'.
	stream ensureEndsWith: Character cr.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = (('this is a test' copyWith: Character cr), 'for WriteStreamTest').
	
	"Manually put a new line and verify there are no 2 new lines"
	stream := WriteStream on: String new.
	stream nextPutAll: ('this is a test' copyWith: Character cr).
	stream ensureEndsWith: Character cr.
	stream nextPutAll: 'for WriteStreamTest'.
	self assert: stream contents = (('this is a test' copyWith: Character cr), 'for WriteStreamTest').! !

!WriteStreamTest methodsFor: 'testing' stamp: 'sd 6/5/2005 09:24'!
testNew

	self should: [WriteStream new] raise: Error. ! !

!WriteStreamTest methodsFor: 'testing' stamp: 'sd 6/5/2005 09:24'!
testSetToEnd

	| string stream |
	string := 'hello'.
	stream := WriteStream with: ''.
	stream nextPutAll: string.
	self assert: stream position = string size.
	stream setToEnd.
	self assert: stream position = string size.
	self assert: stream contents = string! !

TestCase subclass: #Base64MimeConverterTest
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!Base64MimeConverterTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Base64MimeConverter. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!Base64MimeConverterTest methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:10'!
setUp
	message := ReadWriteStream on: (String new: 10).
	message nextPutAll: 'Hi There!!'.! !

!Base64MimeConverterTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:23'!
testMimeEncodeDecode

	| encoded |
	encoded := Base64MimeConverter mimeEncode: message.
	self assert: (encoded contents = 'SGkgVGhlcmUh').
     self assert: ((Base64MimeConverter mimeDecodeToChars: encoded) contents = message contents).! !

TestCase subclass: #ByteArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!ByteArrayTest methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/19/2009 13:31'!
testFourthByteArraysReturnTheCorrectValues

self assert: [(#(16r3F 16r80 0 0) asByteArray floatAt:1 bigEndian: true) = 1.0].
self assert: [(#(16rC0 0 0 0) asByteArray floatAt:1 bigEndian: true) = -2.0].

! !

TestCase subclass: #CharacterSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!CharacterSetTest commentStamp: 'nice 11/20/2007 00:35' prior: 0!
CharacterSetTest holds tests for CharacterSet!

!CharacterSetTest methodsFor: 'testing' stamp: 'nice 11/20/2007 00:38'!
testCopy
    | theOriginal theCopy |
    theOriginal := CharacterSet newFrom: 'abc'.
    theCopy := theOriginal copy.
    theCopy remove: $a.
    ^self should: [theOriginal includes: $a] description: 'Changing the copy should not change the original'.! !

TestCase subclass: #FloatArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!FloatArrayTest commentStamp: 'nice 5/30/2006 01:24' prior: 0!
These tests are used to assert that FloatArrayPlugin has same results as Float asIEEE32BitWord!

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 11/23/2007 23:53'!
testArithmeticCoercion
	"This test is related to http://bugs.squeak.org/view.php?id=6782"
	
	self should: [3.0 / (FloatArray with: 2.0) = (FloatArray with: 1.5)].
	self should: [3.0 * (FloatArray with: 2.0) = (FloatArray with: 6.0)].
	self should: [3.0 + (FloatArray with: 2.0) = (FloatArray with: 5.0)].
	self should: [3.0 - (FloatArray with: 2.0) = (FloatArray with: 1.0)].! !

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 5/30/2006 03:17'!
testFloatArrayPluginPrimitiveAt
	"if FloatArrayPlugin primitive are not here, this test is dumb.
	Otherwise, it will compare primitive and #fromIEEE32Bit:"
	
	#(
		"regular numbers no truncation or rounding"
		2r0.0 2r1.0 2r1.1 2r1.00000000000000000000001
		2r1.0e-10 2r1.1e-10 2r1.00000000000000000000001e-10
		2r1.0e10 2r1.1e10 2r1.00000000000000000000001e10
		
		"smallest float32 before gradual underflow"
		2r1.0e-126
		
		"biggest float32"
		2r1.11111111111111111111111e127
		
		"overflow"
		2r1.11111111111111111111111e128
		
		"gradual underflow"
		2r0.11111111111111111111111e-126
		2r0.00000000000000000000001e-126
		
		"with rounding mode : tests on 25 bits"
		
		2r1.0000000000000000000000001
		2r1.0000000000000000000000010
		2r1.0000000000000000000000011
		2r1.0000000000000000000000100
		2r1.0000000000000000000000101
		2r1.0000000000000000000000110
		2r1.0000000000000000000000111
		2r1.1111111111111111111111001
		2r1.1111111111111111111111010
		2r1.1111111111111111111111011
		2r1.1111111111111111111111101
		2r1.1111111111111111111111110
		2r1.1111111111111111111111111
		
		"overflow"
		2r1.1111111111111111111111110e127
		
		"gradual underflow"
		2r0.1111111111111111111111111e-126
		2r0.1111111111111111111111110e-126
		2r0.1111111111111111111111101e-126
		2r0.1111111111111111111111011e-126
		2r0.1111111111111111111111010e-126
		2r0.1111111111111111111111001e-126
		2r0.0000000000000000000000111e-126
		2r0.0000000000000000000000110e-126
		2r0.0000000000000000000000101e-126
		2r0.0000000000000000000000011e-126
		2r0.0000000000000000000000010e-126
		2r0.0000000000000000000000001e-126
		2r0.0000000000000000000000010000000000000000000000000001e-126
		) do: [:e |
			self assert: ((FloatArray with: e) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1)).
			self assert: ((FloatArray with: e negated) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e negated) basicAt: 1))].
		
	"special cases"
	(Array with: Float infinity with: Float infinity negated with: Float negativeZero)
		do: [:e | self assert: ((FloatArray with: e) at: 1) = (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1))].
		
	"Cannot compare NaN"
	(Array with: Float nan)
		do: [:e | self assert: (Float fromIEEE32Bit: ((FloatArray with: e) basicAt: 1)) isNaN].! !

!FloatArrayTest methodsFor: 'testing' stamp: 'nice 5/30/2006 03:17'!
testFloatArrayPluginPrimitiveAtPut
	"if FloatArrayPlugin primitive are not here, this test is dumb.
	Otherwise, it will compare primitive and #asIEEE32BitWord"
	
	#(
		"regular numbers no truncation or rounding"
		2r0.0 2r1.0 2r1.1 2r1.00000000000000000000001
		2r1.0e-10 2r1.1e-10 2r1.00000000000000000000001e-10
		2r1.0e10 2r1.1e10 2r1.00000000000000000000001e10
		
		"smallest float32 before gradual underflow"
		2r1.0e-126
		
		"biggest float32"
		2r1.11111111111111111111111e127
		
		"overflow"
		2r1.11111111111111111111111e128
		
		"gradual underflow"
		2r0.11111111111111111111111e-126
		2r0.00000000000000000000001e-126
		
		"with rounding mode : tests on 25 bits"
		
		2r1.0000000000000000000000001
		2r1.0000000000000000000000010
		2r1.0000000000000000000000011
		2r1.0000000000000000000000100
		2r1.0000000000000000000000101
		2r1.0000000000000000000000110
		2r1.0000000000000000000000111
		2r1.1111111111111111111111001
		2r1.1111111111111111111111010
		2r1.1111111111111111111111011
		2r1.1111111111111111111111101
		2r1.1111111111111111111111110
		2r1.1111111111111111111111111
		
		"overflow"
		2r1.1111111111111111111111110e127
		
		"gradual underflow"
		2r0.1111111111111111111111111e-126
		2r0.1111111111111111111111110e-126
		2r0.1111111111111111111111101e-126
		2r0.1111111111111111111111011e-126
		2r0.1111111111111111111111010e-126
		2r0.1111111111111111111111001e-126
		2r0.0000000000000000000000111e-126
		2r0.0000000000000000000000110e-126
		2r0.0000000000000000000000101e-126
		2r0.0000000000000000000000011e-126
		2r0.0000000000000000000000010e-126
		2r0.0000000000000000000000001e-126
		2r0.0000000000000000000000010000000000000000000000000001e-126
		) do: [:e |
			self assert: ((FloatArray with: e) basicAt: 1) = e asIEEE32BitWord.
			self assert: ((FloatArray with: e negated) basicAt: 1) = e negated asIEEE32BitWord].
		
	"special cases"
	(Array with: Float infinity with: Float infinity negated with: Float negativeZero with: Float nan)
		do: [:e | self assert: ((FloatArray with: e) basicAt: 1) = e asIEEE32BitWord].
		! !

TestCase subclass: #LinkedListTest
	uses: TAddTest - {#testTAddWithOccurences. #testTAddTwice} + TEmptyTest
	instanceVariableNames: 'nextLink n list link1 link2 link3 link4 nonEmpty otherList link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!LinkedListTest commentStamp: 'mk 8/3/2005 11:55' prior: 0!
A set of test cases which thoroughly test functionality of the LinkedList class.!

LinkedListTest class
	uses: TAddTest classTrait + TEmptyTest classTrait
	instanceVariableNames: ''!

!LinkedListTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 19:10'!
collection
	^ self nonEmpty! !

!LinkedListTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 16:14'!
collectionWithElement
	"Returns a collection that already includes what is returned by #element."
	^ self collection! !

!LinkedListTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 16:21'!
element
	^ link ifNil: [link := StackLink with: 42. "so that we can recognize this link"]! !

!LinkedListTest methodsFor: 'tests - empty' stamp: 'damien.pollet 10/31/2008 14:36'!
empty
	^ list! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:47'!
n
	^n! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:47'!
n: number
	n := number.
	! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:46'!
nextLink
	^nextLink! !

!LinkedListTest methodsFor: 'accessing' stamp: 'md 10/14/2004 10:46'!
nextLink: aLink
	nextLink := aLink! !

!LinkedListTest methodsFor: 'tests - empty' stamp: 'damienpollet 1/29/2009 19:11'!
nonEmpty
	^ nonEmpty ifNil: [nonEmpty := LinkedList with: self element with: Link new]! !

!LinkedListTest methodsFor: 'requirements' stamp: 'damienpollet 1/29/2009 16:22'!
otherCollection
	^ otherList ifNil: [otherList := LinkedList with: Link new with: Link new]! !

!LinkedListTest methodsFor: 'running' stamp: 'damienpollet 1/29/2009 16:22'!
setUp
	super setUp.
	list := LinkedList new.
	link1 := Link new.
	link2 := Link new.
	link3 := Link new.
	link4 := Link new.
	
	"link := StackLink with: 42." "so that we can recognize this link"
	"nonEmpty := LinkedList with: link with: Link new."
	"otherList := LinkedList with: Link new with: Link new."
! !

!LinkedListTest methodsFor: 'running' stamp: 'damien.pollet 10/31/2008 14:48'!
tearDown
	list := nil.
	link1 := nil.
	link2 := nil.
	link3 := nil.
	link4 := nil.
	
	link := nil.
	nonEmpty := nil.
	otherList := nil.
	
	^ super tearDown! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test01add
	self assert: list isEmpty.
	list add: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list add: link2.
	self assert: list size = 2.
	self assert: list first = link1.
	self assert: list second = link2.
	
	list add: link3.
	self assert: list size = 3.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	
	list add: link4.
	self assert: list size = 4.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	self assert: list fourth = link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test02addLast
	self assert: list isEmpty.
	
	list addLast: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list addLast: link2.
	self assert: list size = 2.
	self assert: list first = link1.
	self assert: list second = link2.
	
	list addLast: link3.
	self assert: list size = 3.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	
	list addLast: link4.
	self assert: list size = 4.
	self assert: list first = link1.
	self assert: list second = link2.
	self assert: list third = link3.
	self assert: list fourth = link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:04'!
test03addFirst
	self assert: list isEmpty.
	
	list addFirst: link1.
	self assert: list size = 1.
	self assert: list first = link1.
	
	list addFirst: link2.
	self assert: list size = 2.
	self assert: list first = link2.
	self assert: list second = link1.
	
	list addFirst: link3.
	self assert: list size = 3.
	self assert: list first = link3.
	self assert: list second = link2.
	self assert: list third = link1.
	
	list addFirst: link4.
	self assert: list size = 4.
	self assert: list first = link4.
	self assert: list second = link3.
	self assert: list third = link2.
	self assert: list fourth = link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test04addBefore
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 before: link1.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link1.
	
	list add: link3 before: link1.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link1.
	
	list add: link4 before: link1.
	self assert: list size = 4.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	self assert: list fourth == link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test05addBefore
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 before: link1.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link1.
	
	list add: link3 before: link2.
	self assert: list size = 3.
	self assert: list first == link3.
	self assert: list second == link2.
	self assert: list third == link1.
	
	list add: link4 before: link3.
	self assert: list size = 4.
	self assert: list first == link4.
	self assert: list second == link3.
	self assert: list third == link2.
	self assert: list fourth == link1! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:05'!
test06addAfter
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 after: link1.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list add: link3 after: link2.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	
	list add: link4 after: link3.
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test07addAfter
	self assert: list isEmpty.
	
	list add: link1.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list add: link2 after: link1.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list add: link3 after: link1.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link3.
	self assert: list third == link2.
	
	list add: link4 after: link1.
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link4.
	self assert: list third == link3.
	self assert: list fourth == link2! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:09'!
test08addAfter
	| l first |
	l := LinkedList new.
	first := self class new n: 1.
	
	l add: first.
	l add: (self class new n: 3).

	self assert: (l collect:[:e | e n]) asArray  = #(1 3).
	
	l add: (self class new n: 2) after: first.

	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:09'!
test09addAfter
	| l last |
	l := LinkedList new.
	last := self class new n: 2.
	l add: (self class new n: 1).
	l add: last.
	
	self assert: (l collect:[:e | e n]) asArray  = #(1 2).
	 
	l add: (self class new n: 3) after: last.

	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test10removeFirst
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list removeFirst.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	
	list removeFirst.
	self assert: list size = 2.
	self assert: list first == link3.
	self assert: list second == link4.
	
	list removeFirst.
	self assert: list size = 1.
	self assert: list first == link4.
	
	list removeFirst.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:10'!
test11removeLast
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list removeLast.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	
	list removeLast.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link2.
	
	list removeLast.
	self assert: list size = 1.
	self assert: list first == link1.
	
	list removeFirst.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:11'!
test12remove
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list remove: link3.
	self assert: list size = 3.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link4.
	
	list remove: link2.
	self assert: list size = 2.
	self assert: list first == link1.
	self assert: list second == link4.
	
	list remove: link1.
	self assert: list size = 1.
	self assert: list first == link4.
	
	list remove: link4.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:12'!
test13remove
	list add: link1.
	list add: link2.
	list add: link3.
	list add: link4.
	
	self assert: list size = 4.
	self assert: list first == link1.
	self assert: list second == link2.
	self assert: list third == link3.
	self assert: list fourth == link4.
	
	list remove: link1.
	self assert: list size = 3.
	self assert: list first == link2.
	self assert: list second == link3.
	self assert: list third == link4.
	
	list remove: link4.
	self assert: list size = 2.
	self assert: list first == link2.
	self assert: list second == link3.
	
	list remove: link2.
	self assert: list size = 1.
	self assert: list first == link3.
	
	list remove: link3.
	self assert: list isEmpty! !

!LinkedListTest methodsFor: 'testing' stamp: 'mk 8/3/2005 12:13'!
test14removeIfAbsent
	list add: link1.
	
	self assert: list size = 1.
	self assert: list first == link1.
	
	list remove: link1.
	self assert: list isEmpty.
	
	[list remove: link1]
		on: Error
		do: [^ self].
		
	"The execution should not get here. If yes, something went wrong."
	self assert: false! !

!LinkedListTest methodsFor: 'testing' stamp: 'zz 12/7/2005 19:08'!
test22addAll
	| link5 link6 link7 link8 listToBeAdded |
	link5 := Link new.
	link6 := Link new.
	link7 := Link new.
	link8 := Link new.
	
	list
		add: link1;
		add: link2;
		add: link3;
		add: link4.
		
	listToBeAdded := LinkedList new.
	listToBeAdded
		add: link5;
		add: link6;
		add: link7;
		add: link8.
		
	list addAll: listToBeAdded.
	
	self should: [(list at: 1) == link1].
	self should: [(list at: 2) == link2].
	self should: [(list at: 3) == link3].
	self should: [(list at: 4) == link4].
	self should: [(list at: 5) == link5].
	self should: [(list at: 6) == link6].
	self should: [(list at: 7) == link7].
	self should: [(list at: 8) == link8].! !

!LinkedListTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:20'!
testAddAfter

	| l first |
	l := LinkedList new.
	first := self class new n: 1.
	
	l add: first.
	l add: (self class new n: 3).
	self assert: (l collect:[:e | e n]) asArray  = #(1 3).
	l add: (self class new n: 2) after: first.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:20'!
testAddAfterLast

	| l last |
	l := LinkedList new.
	last := self class new n: 2.
	l add: (self class new n: 1).
	l add: last.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2). 
	l add: (self class new n: 3) after: last.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).! !

!LinkedListTest methodsFor: 'tests' stamp: 'nice 7/28/2008 22:27'!
testAddAfterLast2
	"LinkedListTest new testAddAfterLast2"

	| l first second third fourth |
	l := LinkedList new.
	first := self class new n: 1.
	second := self class new n: 2.
	third := self class new n: 3.
	fourth :=self class new n: 4.
	l addLast: first.
	l addLast: second.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2). 
	l add: third after: second.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3).
	l addLast: fourth.
	self assert: (l collect:[:e | e n]) asArray  = #(1 2 3 4).! !

!LinkedListTest methodsFor: 'tests - adding' stamp: 'damien.pollet 10/31/2008 23:16'!
testTAdd

	| added |
	added := self otherCollection add: self element.
	self assert: added = self element. "equality or identity ?"
	self assert: (self otherCollection includes: self element).

	! !

TestCase subclass: #MatrixTest
	instanceVariableNames: 'matrix1 matrix2 matrix3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Unordered'!

!MatrixTest methodsFor: 'testing' stamp: 'dc 3/3/2007 17:58'!
setUp
	matrix1 := Matrix new: 2.
	matrix1 at:1 at:1 put: 1.
	matrix1 at:1 at:2 put: 3.
	matrix1 at:2 at:1 put: 2.
	matrix1 at:2 at:2 put: 4.
	
	matrix2 := Matrix new: 2.
	matrix2 at:1 at:1 put: 3.
	matrix2 at:1 at:2 put: 7.
	matrix2 at:2 at:1 put: 4.
	matrix2 at:2 at:2 put: 8.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:47'!
testAtAt
	self should:[matrix1 at: 2 at: 3] raise: Error.
	self should:[matrix1 at: 3 at: 2] raise: Error.
	self should:[matrix1 at: 3 at: 3] raise: Error.
	self should:[matrix1 at: 0 at: 1] raise: Error.
	self should:[matrix1 at: 1 at: 0] raise: Error.
	self should:[matrix1 at: 0 at: 0] raise: Error.
	
	self assert: (matrix1 at: 1 at: 1) = 1! !

!MatrixTest methodsFor: 'tests - copying' stamp: 'dc 3/3/2007 17:48'!
testCopy

	| copyMatrix |
	
	copyMatrix := matrix1 copy.
	self assert: matrix1 = copyMatrix ! !

!MatrixTest methodsFor: 'tests - testing' stamp: 'dc 3/3/2007 17:49'!
testIncludes
	self assert:
			((1 to: 4)
				allSatisfy: [:i | matrix1 includes: i])
! !

!MatrixTest methodsFor: 'tests - arithmetic' stamp: 'dc 3/3/2007 17:50'!
testMultiply
	
	| result |
	self	should: [matrix1	preMultiplyByMatrix: (Matrix new: 3)]raise: Error.

	result := matrix2 preMultiplyByMatrix: matrix1.
	self assert: (result at: 1 at: 1) = 15.
	self assert: (result at: 1 at: 2) = 31.
	self assert: (result at: 2 at: 1) = 22.
	self assert: (result at: 2 at: 2) = 46! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:52'!
testReplaceAll

	matrix1 replaceAll: 1 with: 10.
	self assert: (matrix1 at:1 at:1) = 10.
	self assert: (matrix1 at:2 at:1) = 2.
	self assert: (matrix1 at:1 at:2) = 3.
	self assert: (matrix1 at:2 at:2) = 4.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:53'!
testSwap
	matrix1 swap: 1 at: 2 with: 1 at: 1.
	self assert: (matrix1 at: 1 at: 1) = 3.
	self assert: (matrix1 at: 1 at: 2) = 1.! !

!MatrixTest methodsFor: 'tests - accessing' stamp: 'dc 3/3/2007 17:58'!
testTransposed
	| transposedMatrix |
	
	transposedMatrix := matrix1 transposed.
	self assert: [(transposedMatrix at:1 at:1) = 1].
	self assert: [(transposedMatrix at:1 at:2) = 2].
	self assert: [(transposedMatrix at:2 at:1) = 3].
	self assert: [(transposedMatrix at:2 at:2) = 4].! !

TestCase subclass: #ReadStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!ReadStreamTest commentStamp: 'tlk 12/5/2004 14:36' prior: 0!
I am an SUnit test for ReadStream.
I have no test fixtures.!

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 5/30/2008 14:26'!
streamOn: collection upToAll: subcollection 
	^ collection readStream upToAll: subcollection! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 5/30/2008 14:26'!
streamOn: collection upToAll: subcollection1 upToAll: subcollection2 
	^ collection readStream
		upToAll: subcollection1;
		upToAll: subcollection2! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:53'!
testBack
	|stream|
	stream := 'abc' readStream.
	stream setToEnd.
	self assert: stream back = $c.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:54'!
testBackOnPosition1
	"Test the new implementation of the method back."
	|stream|
	stream := 'abc' readStream.
	stream next.
	self assert: stream back = $a.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:55'!
testOldBack
	"Test the old behavior of the method back. The method #oldBack is a misconception about what a stream is. A stream contains a pointer *between* elements with past and future elements. The method #oldBack considers that the pointer is *on* an element. (Damien Cassou - 1 August 2007)"
	|stream|
	stream := 'abc' readStream.
	stream setToEnd.
	self assert: stream oldBack = $b.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:56'!
testOldBackOnPosition1
	"Test the old behavior of the method back. The method #oldBack is a misconception about what a stream is. A stream contains a pointer *between* elements with past and future elements. The method #oldBack considers that the pointer is *on* an element. (Damien Cassou - 1 August 2007)"
	|stream|
	stream := 'abc' readStream.
	stream next.
	self assert: stream oldBack = nil.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:56'!
testOldPeekBack
	"Test the old behavior of the method peekBack. The method #oldBack is a misconception about what a stream is. A stream contains a pointer *between* elements with past and future elements. The method #oldBack considers that the pointer is *on* an element. (Damien Cassou - 1 August 2007)"
	|stream|
	stream := 'abc' readStream.
	stream setToEnd.
	self assert: stream oldPeekBack = $b.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'damiencassou 11/23/2008 16:57'!
testPeekBack
	"Test the new implementation of the method peekBack due to changing #back."
	|stream|
	stream := 'abc' readStream.
	stream setToEnd.
	self assert: stream peekBack = $c.
! !

!ReadStreamTest methodsFor: 'tests' stamp: 'tlk 12/5/2004 14:34'!
testPositionOfSubCollection
	
	self assert: ('xyz' readStream positionOfSubCollection: 'q' ) = 0.
	self assert: ('xyz' readStream positionOfSubCollection: 'x' ) = 1.

	self assert: ('xyz' readStream positionOfSubCollection: 'y' ) = 2.
	self assert: ('xyz' readStream positionOfSubCollection: 'z' ) = 3.! !

!ReadStreamTest methodsFor: 'tests' stamp: 'bp 10/29/2004 06:16'!
testUpToAll

	self assert: (self streamOn: 'abcdefgh' upToAll: 'cd') = 'ab'.
	self assert: (self streamOn: 'abcdefgh' upToAll: 'cd' upToAll: 'gh') = 'ef'.

	self assert: (self streamOn: '' upToAll: '') = ''.

	self assert: (self streamOn: 'a' upToAll: '') = ''.
	self assert: (self streamOn: 'a' upToAll: 'a') = ''.
	self assert: (self streamOn: 'a' upToAll: 'b') = 'a'.

	self assert: (self streamOn: 'ab' upToAll: '') = ''.
	self assert: (self streamOn: 'ab' upToAll: 'a') = ''.
	self assert: (self streamOn: 'ab' upToAll: 'b') = 'a'.
	self assert: (self streamOn: 'ab' upToAll: 'c') = 'ab'.
	self assert: (self streamOn: 'ab' upToAll: 'ab') = ''.

	self assert: (self streamOn: 'abc' upToAll: '') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'a') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'b') = 'a'.
	self assert: (self streamOn: 'abc' upToAll: 'c') = 'ab'.
	self assert: (self streamOn: 'abc' upToAll: 'd') = 'abc'.
	self assert: (self streamOn: 'abc' upToAll: 'ab') = ''.
	self assert: (self streamOn: 'abc' upToAll: 'bc') = 'a'.
	self assert: (self streamOn: 'abc' upToAll: 'cd') = 'abc'.
! !

TestCase subclass: #RunArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Arrayed'!

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'zz 7/2/2008 16:18'!
testANewRunArrayIsEmpty

	| t |
	t := RunArray new.
	self assert:t isEmpty 
! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:25'!
testAt
	"self debug: #testAt"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: (array at: 3) = 2.
	
	array at: 3 put: 5.
	self assert: (array at: 3) = 5
! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:26'!
testAtPut
	"self debug: #testAtPut"
	| array |
	array := RunArray new: 5 withAll: 2.
	
	array at: 3 put: 5.
	self assert: array = #(2 2 5 2 2).
	
	array at: 1 put: 1.
	self assert: array = #(1 2 5 2 2).! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/15/2007 10:29'!
testAtPut2
	"self debug: #testAtPut2"
	| array |
	array := RunArray new: 5 withAll: 2.
	
	self should: [array at: 0 put: 5] raise: Error.
	self should: [array at: 6 put: 5] raise: Error.
	self should: [array at: $b put: 5] raise: Error.! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:49'!
testFirst
	"self debug: #testFirst"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array first = 2.
	
	array := #($a $b $c $d) as: RunArray.
	self assert: array first = $a.! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:49'!
testLast
	"self debug: #testLast"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array last = 2.
	
	array := #($a $b $c $d) as: RunArray.
	self assert: array last = $d.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:22'!
testNew
	"self debug: #testNew"
	| array |
	array := RunArray new.
	self assert: array size = 0.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:24'!
testNewFrom
	"self debug: #testNewFrom"
	| array |
	array := RunArray newFrom: #($a $b $b $b $b $c $c $a).
	self assert: array size = 8.
	self assert: array = #($a $b $b $b $b $c $c $a).! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:15'!
testNewWithAll
	"self debug: #testNewWithAll"
	| array |
	array := RunArray new: 5 withAll: 2.
	self assert: array size = 5.
	self assert: array = #(2 2 2 2 2)! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 5/24/2007 10:56'!
testNewWithSize
	|array|
	array := RunArray new: 5.
	self assert: array size = 5.
	1 to: 5 do: [:index | self assert: (array at: index) isNil]! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 09:58'!
testRunLengthAt
	"self debug: #testRunLengthAt"
	| array |
	array := #($a $b $b $c $c $c $d $d) as: RunArray.
	self assert: (array runLengthAt: 1) = 1.
	self assert: (array runLengthAt: 2) = 2.
	self assert: (array runLengthAt: 3) = 1.
	self assert: (array runLengthAt: 4) = 3.
	self assert: (array runLengthAt: 5) = 2.
	self assert: (array runLengthAt: 6) = 1.
	self assert: (array runLengthAt: 7) = 2.
	self assert: (array runLengthAt: 8) = 1.! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'dc 2/12/2007 09:30'!
testRunsValues
	"self debug: #testRunsValues"
	| array |
	array := RunArray runs: #(1 4 2 1) values: #($a $b $c $a).
	self assert: array size = 8.
	self assert: array = #($a $b $b $b $b $c $c $a).! !

!RunArrayTest methodsFor: 'tests - instance creation' stamp: 'damiencassou 5/30/2008 14:26'!
testScanFromANSICompatibility
	"self run: #testScanFromANSICompatibility"
	RunArray scanFrom: '()f1dNumber new;;' readStream.
	RunArray scanFrom: '()a1death;;' readStream.
	RunArray scanFrom: '()F1death;;' readStream! !

!RunArrayTest methodsFor: 'tests - accessing' stamp: 'dc 2/12/2007 10:12'!
testWithStartStopAndValueDo
	"self debug: #testWithStartStopAndValueDo"
	| array elements startStops |
	array := #($a $b $b $c $c $c $d $d) as: RunArray.
	elements := OrderedCollection new.
	startStops := OrderedCollection new.
	array withStartStopAndValueDo: [:start :stop :value | elements add: value. startStops add: start->stop].
	
	self assert: elements asArray = #($a $b  $c  $d).
	self assert: startStops asArray = {1->1 . 2->3 . 4->6 . 7->8}! !

TestCase subclass: #SharedQueue2Test
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Sequenceable'!

!SharedQueue2Test methodsFor: 'testing' stamp: 'ls 6/25/2005 13:49'!
testBasics
	| q |
	q := SharedQueue2 new.

	self should: [ q nextOrNil = nil ].

	q nextPut: 5.
	self should: [ q nextOrNil = 5 ].
	self should: [ q nextOrNil = nil ].

! !

!SharedQueue2Test methodsFor: 'testing' stamp: 'stephaneducasse 2/4/2006 20:10'!
testContention1
	"here is a test case that breaks the standard SharedQueue from Squeak 3.8"

	| q r1 r2 |
	q := SharedQueue2 new.
	q nextPut: 5.
	q nextPut: 10.

	self should: [ q nextOrNil = 5 ].

	[ r1 := q next ] fork.
	[ r2 := q next ] fork.
	Processor  yield.   "let the above two threads block"
	
	q nextPut: 10.
	Processor yield.

	self should: [ r1 = 10 ].
	self should: [ r2 = 10 ].
	self should: [ q nextOrNil = nil ].
! !

!SharedQueue2Test methodsFor: 'testing' stamp: 'ls 6/25/2005 14:15'!
testNextOrNilSuchThat
	| q item |
	q := SharedQueue2 new.
	q nextPut: 5.
	q nextPut: 6.

	item := q nextOrNilSuchThat: [ :x | x even ].
	self should: [ item = 6 ].

	self should: [ q nextOrNil = 5 ].
	self should: [ q nextOrNil = nil ].
! !

TestCase subclass: #SkipListTest
	uses: TEmptyTest
	instanceVariableNames: 'empty nonEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-SkipLists'!

!SkipListTest commentStamp: 'tlk 4/30/2006 21:49' prior: 0!
A SkipListTest is tests basic functionality of SkipList

Instance Variables: none
!

SkipListTest class
	uses: TEmptyTest classTrait
	instanceVariableNames: ''!

!SkipListTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 12/5/2008 16:13'!
empty
	
	^ empty! !

!SkipListTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 12/5/2008 16:13'!
nonEmpty
	
	^ nonEmpty! !

!SkipListTest methodsFor: 'initialize-release' stamp: 'stephane.ducasse 12/5/2008 16:17'!
setUp
	
	super initialize.
	empty := SkipList new.
	nonEmpty := SkipList new.
	nonEmpty	at: #a put: 20.
	nonEmpty	at: #b put: 30. 
	nonEmpty	at: #c put: 40.
	nonEmpty	at: #d put: 30.! !

!SkipListTest methodsFor: 'implementationtests' stamp: 'sd 3/25/2006 14:32'!
testAdd
	"tests size after adding element"
	
	"self run:#testAdd"
	| s |
	s := SkipList new.
	s add: 1.
	self assert: s size = 1
! !

!SkipListTest methodsFor: 'implementationtests' stamp: 'sd 3/25/2006 14:32'!
testAddIfPresent
	"adds an already existing element. Decides to add another one in the
	ifPresent block"
	
	| s |
	s := SkipList new.
	s add: 1.
	self
		shouldnt: [s
					add: 1
					ifPresent: [s add: 2]]
		raise: Exception.
	self assert: s size = 2
! !

!SkipListTest methodsFor: 'public' stamp: 'sd 5/2/2006 20:45'!
testAt
	"self run: #testAt"
	"self debug: #testAt"

	| sk |
	sk := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	sk at: 3 put: '666'.
	self assert: (sk at: 7) = '77777'.
	self assert: (sk includes: 7). 
	self assert: (sk at: 3) = '3333'.
	self assert: (sk includes: 3). 
	self assert: (sk size) = 3

	! !

!SkipListTest methodsFor: 'public' stamp: 'YE 3/8/2006 10:00'!
testAtPutAlreadyPresent
	"self run: #testAtPutAlreadyPresent"
	"self debug: #testAtPutAlreadyPresent"

	| sk |
	sk := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	sk at: 3 put: '666'.
	
	self assert: (sk at: 7) = '77777'.
	self assert: (sk includes: 7). 
	
	self assert: (sk at: 3) = '3333'.
	
	self assert: (sk includes: 3). 
	self assert: (sk size) = 3

	! !

!SkipListTest methodsFor: 'public' stamp: 'YE 3/8/2006 09:56'!
testCreation
	"self run: #testCreation"
	"self debug: #testCreation"

	| sk |
	sk := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	self assert: (sk at: 7) = '77777'.
	self assert: (sk includes: 7). 

	! !

!SkipListTest methodsFor: 'public' stamp: 'YE 3/16/2006 10:50'!
testIfAbsent
	"self run: #testIfAbsent"
	"self debug: #testIfAbsent"

	| sk temp |
	sk := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	sk add: 7 ifPresent: [sk at: 8 put: '88'].
	temp := sk at: 9 ifAbsent: [sk at: 8].
	self assert: (temp = '88')
	
	

	! !

!SkipListTest methodsFor: 'public' stamp: 'YE 3/16/2006 10:49'!
testIfPresent
	"self run: #testIfPresent"
	"self debug: #testIfPresent"

	| sk |
	sk := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	sk add: 7 ifPresent: [sk at: 8 put: '88'].
	self assert: (sk at: 7) = '77777'.
	self assert: (sk at: 8) = '88'.
	

	! !

!SkipListTest methodsFor: 'public' stamp: 'YE 3/16/2006 11:47'!
testIsEqualTo
	"self run: #testIsEqualTo"
	"self debug: #testIsEqualTo"

	| sk sk2 |
	sk := SkipList new.
	sk2 := SkipList new.
	sk at: 11 put: '111111'.
	sk at: 3 put: '3333'.
	sk at: 7 put: '77777'.
	sk at: 9 put: '3333'.
	
	sk2 at: 3 put: '3333'.
	sk2 at: 5 put: '3333'.
	self assert: (sk is: (sk at: 3) equalTo: (sk at: 9)).
	self assert: (sk is: (sk at: 3) equalTo: (sk2 at: 3)).
	self assert: (sk is: (sk at: 3) equalTo: (sk2 at: 5))
	
	
	

	! !

!SkipListTest methodsFor: 'implementationtests' stamp: 'tlk 4/30/2006 21:37'!
testMaxLevel
	"No node should have a level greater than the skiplist maxLevel"
	"self run: #testMaxLevel"
	| s |
	s := SkipList new.
	s add: 12.
	s add: 53.
	s add: 14.
	s
		nodesDo: [:n | 
			      self
   					assert: n level <= s maxLevel]! !

!SkipListTest methodsFor: 'implementationtests' stamp: 'sd 5/2/2006 20:45'!
testNavigation
	"self run: #testNavigation"
	| node1 node2 node3 node4 skip |
	node1 := SkipListNode
				key: 1
				value: 3
				level: 1.
	node2 := SkipListNode
				key: 2
				value: 7
				level: 2.
	node3 := SkipListNode
				key: 3
				value: 11
				level: 1.
	node4 := SkipListNode
				key: 4
				value: 23
				level: 2.
	node1 atForward: 1 put: node2.
	node2 atForward: 1 put: node3.
	node2 atForward: 2 put: node4.
	node4 atForward: 1 put: nil.
	skip := SkipList new.
	skip atForward: 1 put: node1.
	skip atForward: 2 put: node2.
	self assert: skip first = node1.
	self
		assert: (skip at: 2) = node2 value! !

!SkipListTest methodsFor: 'implementationtests' stamp: 'YE 2/28/2006 09:45'!
testRandomLevel
	"a randomLevel should not be greater than maxLevel"
	"self run: #testRandomLevel"
	| s |
	s := SkipList new.
	s maxLevel: 5.
	self assert: s randomLevel <= 5! !

!SkipListTest methodsFor: 'public' stamp: 'sd 3/25/2006 14:36'!
testRemove
	"tests size after removing element"
	"self run:#testRemove"
	
	| s |
	s := SkipList new.
	s add: 1.
	self assert: s size = 1.
	self
		should: [s remove: 2]
		raise: Exception.
	s remove: 1.
	self assert: s size = 0
! !

TestCase subclass: #StackTest
	uses: TEmptyTest + TCloneTest - {#testCopyNonEmpty}
	instanceVariableNames: 'empty nonEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Stack'!

StackTest class
	uses: TEmptyTest classTrait + TCloneTest classTrait
	instanceVariableNames: 'testSize'!

StackTest class
	uses: TEmptyTest classTrait + TCloneTest classTrait
	instanceVariableNames: 'testSize'!

!StackTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 12/5/2008 17:45'!
empty
	
 	^ empty! !

!StackTest methodsFor: 'tests - empty' stamp: 'stephane.ducasse 12/5/2008 17:45'!
nonEmpty
	
 	^ nonEmpty! !

!StackTest methodsFor: 'setup' stamp: 'stephane.ducasse 12/5/2008 17:42'!
setUp

	empty := Stack new.
	nonEmpty := Stack new.
	nonEmpty push: 1.
	nonEmpty push: -2.
	nonEmpty push: 3.
	nonEmpty push: 1.
	! !

!StackTest methodsFor: 'test' stamp: 'dc 6/21/2006 10:55'!
testEmptyError

	| aStack |
	aStack := Stack new.
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.
	
	aStack push: 'element'.
	
	self shouldnt: [ aStack top ] raise: Error.
	self shouldnt: [ aStack pop] raise: Error.
	
	
	"The stack is empty again due to previous pop"
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testPop

	| aStack res elem |
	elem := 'anElement'.	
	aStack := Stack new.
	self assert: aStack isEmpty.
	
	aStack push: 'a'.
	aStack push: elem.
	res := aStack pop.	
	self assert: res = elem.
	self assert: res == elem.
	
	self assert: aStack size = 1.
	aStack pop.
	self assert: aStack isEmpty.

! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testPush
	
	| aStack |
	aStack := Stack new.
	aStack push: 'a'.
	self assert: aStack size = 1.	
	aStack push: 'b'.
	self assert: aStack size = 2.
	! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testSize
	
	| aStack |
	aStack := Stack new.
	self assert: aStack size = 0.
	aStack push: 'a'.
	self assert: aStack size = 1.
	aStack push: 'b'.
	self assert: aStack size = 2.
	aStack pop.
	self assert: aStack size = 1.
	aStack pop.
	self assert: aStack size = 0.

 
	


! !

!StackTest methodsFor: 'test' stamp: 'sd 3/21/2006 22:13'!
testTop

	| aStack |
	aStack := Stack new.
	self assert: aStack isEmpty.
	aStack push: 'a'.
	aStack push: 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack size = 2.! !

TestCase subclass: #TextAndTextStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextAndTextStreamTest commentStamp: '<historical>' prior: 0!
At May 09, 2003 Tim Olson sent a bug report to the Squeak developers list that inspired me to examine the protocol of TextStream in greater detail.  (The bug that Tim reported was present in Squeak 3.4, it is shown in testExample1.) In a discussion that followed,  Daniel Vainsencher proposed that we should have tests for Text and TextStreams. This class is an attempt to implement that proposal. For Squeak 3.4, some of the test examples fail.!

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/10/2003 20:17'!
example1: size

   | ts text |

  ts := TextStream on: (Text new: size).
  ts  nextPutAll: 'xxxxx' asText.
  ts nextPutAll: ('yyyyy' asText allBold, 'zzzzzzz' asText).
  text := ts contents.
  ^text
  ! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/10/2003 20:26'!
example2

      | ts text |

  ts := TextStream on: (Text new: 50).
  ts  nextPutAll: 'abc' asText.
  ts nextPutAll: 'def' asText allBold.
  ts nextPutAll: 'ghijk' asText.
  text := ts contents.
  ^text
  ! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/11/2003 13:09'!
replacementAtStartExample3

   | text1  replacement  length  |

   text1 := 'This is a simple text' copy asText.
    " without the copy, we would modify a constant that the compiler attached at the compiled method. "
   length  := 'This' size.
   replacement := 'Tht' asText.
   text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
! !

!TextAndTextStreamTest methodsFor: 'examples' stamp: 'BG 6/11/2003 13:05'!
replacementExample3

  " for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
    This test examines the preservation of this assertion for in-place replacement 
 Here, the replacement text is shorteer than the text that is shall replace. "


   | text1 string replacement startPos length startPosInRep string2 |

   text1 := (string := 'This is again simple text' copy) asText.
     " without the copy, we would modify a constant that the compiler attached at the compiled method. "
   startPos := string findString: 'simple'. 
   length  := 'simple' size.
   replacement := (string2 := 'both simple and short') asText.
   startPosInRep :=  string2 findString: 'short'.
   text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
   
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:45'!
testAddStringToTextStream

	"It is possible to add a string into a TextStream.
	This test verifies that the created text has text attributes for all its characters. "

	| ts text |
	ts := TextStream on: (Text new: 50).
	ts nextPutAll: 'abc' asText.
	ts nextPutAll: 'def' asText allBold.
	ts nextPutAll: 'ghijk'.
	text := ts contents.
     " now, check the fundamental invariant of a text: "
	self assert: text string size = text runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:40'!
testExampleText1
	"self run: #testExampleText1"
	"inspired by a bug report from Tim Olson.
	Text attributes are lost when the stream collection is expanded.
	Documented BUG!!!!!!"

    | text1 text2 atts1 atts2 |
	text1 := self example1: 10. " here we will loose the attribute bold "
	text2 := self example1: 50. " here we have a larger buffer and will not loose text attributes "
	atts1 := text1 runs copyFrom: 6 to: 10. 
	atts2 := text2 runs copyFrom: 6 to: 10. 

	self assert: atts1 = atts2.
      ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:40'!
testExampleText2
	"a Text looses its attributes when it is reversed "
	
	| text1 text2 |
	text1 := self example2.
	text2 := text1 reversed reversed.
	self assert: text1 runs = text2 runs.
 
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:35'!
testRunArrayAdjacentMerge

	"this demonstrates that adjancent runs with equal attributes are merged. "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis bold times: 5.
	self assert: (runArray runs size = 2). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:37'!
testRunArrayInvariant

	"this verifies that the fundamental invariant of a RunArray is always satisfied. "
	"see comment below"
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert:
       ((1 to: runArray size) allSatisfy:
           [:idx |  | lastIndex lastOffset lastRun lengthOfPreviousRuns |
               runArray at: idx.  " updates the cached values "
               lastIndex := runArray instVarNamed: 'lastIndex'.
               lastRun := runArray instVarNamed: 'lastRun'.
               lastOffset := runArray instVarNamed: 'lastOffset'.
               lengthOfPreviousRuns 
                   := (1 to: lastRun - 1)
                      inject: 0
                       into: [:sum :idx2 | sum + (runArray runs at: idx2)].
               lastIndex = (lastOffset + lengthOfPreviousRuns + 1) 
           ]
       ). 

" This method is a bit tricky. First, it uses Object>>instVarNamed: to access instance variables for which no accessors are defined. The same method is used by the debuggers and by various inspectors.
The assertion itself explains the meaning of the cached values."! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:36'!
testRunArrayReversal

  	"this tests the reversal of a  RunArray "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray reversed runs size = 3). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:32'!
testRunArrayRunsAreNotMerged

	" this demonstrates that different runs are not merged "
	| runArray |
	runArray := RunArray new.
	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray runs size = 3). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:33'!
testRunArrayRunsSize

 	"this demonstrates that the size of a run array is the sum of the sizes of its runs. "
	| runArray |
	runArray := RunArray new.
  	runArray 
		addLast: TextEmphasis normal times: 5;
		addLast: TextEmphasis bold times: 5;
		addLast: TextEmphasis normal times: 5.
	self assert: (runArray size = 15). ! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:49'!
testTextEmphasisRangeDetection1

	"this tests the detection of the range of a text attribute. "
	| text startPos boldStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	boldStyle := TextEmphasis bold.

  " uncomment the following statement for examine failures: "
  " -----------------
       (1 to: text size) do:
           [:idx | | range |
              range := text rangeOf: boldStyle startingAt: idx.
             Transcript show: startPos; show: ' -- '; show: idx printString; show: '  '; show: range printString; show: range size printString; show: ((idx between: startPos and: startPos + 3)
                  ifTrue:
                    [range first = startPos & (range size = 4)]
                  ifFalse:
                    [range first = idx & (range size = 0)]) printString; cr.
           ].
    ------------- "

	self assert: 
		((1 to: text size) allSatisfy:
			[:idx | | range |
				range := text rangeOf: boldStyle startingAt: idx.
				(idx between: startPos and: startPos + 3)
					ifTrue: [range first = startPos & (range size = 4)]
					ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:51'!
testTextEmphasisRangeDetection2

	"this tests the detection of the range of a text attribute.
	Here the searched attribute spans three runs. The objective of the test is whether the entire range is always found."
	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	text addAttribute: TextEmphasis italic from: startPos - 2 to: startPos + 5.
	searchedStyle := TextEmphasis italic.

  " uncomment the following statement for examine failures: " 
  " -----------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos; show: ' -- '; show: idx printString; show: '  '; show: range printString; show: range size printString; show: ((idx between: startPos - 2 and: startPos -2 + 7)
                  ifTrue:
                    [range first = (startPos - 2) & (range size = 8)]
                  ifFalse:
                    [range first = idx & (range size = 0)]) printString; cr.
           ].
   ----------------------- "
	self assert: 
			((1 to: text size) allSatisfy:
				[:idx | | range |
					range := text rangeOf: searchedStyle startingAt: idx.
					(idx between: startPos - 2 and: startPos -2 + 7)
						ifTrue: [range first = (startPos - 2) & (range size = 8)]
						ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:53'!
testTextEmphasisRangeDetection3

	"this tests the detection of the range of a text attribute.
    Here the searched attribute spans three runs. The the range to be detected begins at text position 1. The objective of the test is whether the entire range is always found."

	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.
	text addAttribute: TextEmphasis italic from: 1 to: startPos + 5.
	searchedStyle := TextEmphasis italic.

	" uncomment the following statement to examine failures: "
 	" -----------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos;
					show: ' -- ';
					show: idx printString;
					show: '  ';
					show: range printString;
					show: range size printString;
                        show: ' ';
					 show: ((idx between: 1 and: startPos + 5)
                  					ifTrue:
                  					  [range first = 1 & (range size = (startPos + 5))]
                					ifFalse:
                   					 [range first = idx & (range size = 0)]) printString; cr.
           ].
   ----------------------- "
	self assert: 
       ((1 to: text size) allSatisfy:
           [:idx | | range |
              range := text rangeOf: searchedStyle startingAt: idx.
              (idx between: 1 and: startPos + 5)
                  ifTrue:
                    [range first = 1 & (range size = (startPos + 5))]
                  ifFalse:
                    [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:55'!
testTextEmphasisRangeDetection4

	"this tests the detection of the range of a text attribute.
	Here the searched attribute spans three runs. The the range to be detected extends to the end of the text . The objective of the test is whether the 
	entire range is always found."

	| text startPos searchedStyle |
	text := 'This is a text with attriute bold for some characters' asText.
	startPos := text findString: 'bold' startingAt: 1.
	text addAttribute: TextEmphasis bold from: startPos to: startPos + 3.	
	text addAttribute: TextEmphasis italic from: startPos - 2 to: text size.
	searchedStyle := TextEmphasis italic.

	" uncomment the following statement to examine failures: "
 
 	" -----------------------------------------
       (1 to: text size) do:
           [:idx | | range | 
              range := text rangeOf: searchedStyle startingAt: idx.
             Transcript show: startPos;
					show: ' -- ';
					show: idx printString;
					show: '  ';
					show: range printString;
					show: range size printString;
                        show: ' ';
					 show: ((idx between: startPos - 2 and: text size)
                  			ifTrue:
   			                 [range first = (startPos - 2) & (range size = (text size - (startPos - 2) + 1))]
                  			ifFalse:
 			                 [range first = idx & (range size = 0)]) printString;
					cr.
           ].
   -------------------------------"

	self assert: 
       ((1 to: text size) allSatisfy:
           [:idx | | range |
              range := text rangeOf: searchedStyle startingAt: idx.
              (idx between: startPos - 2 and: text size)
                  ifTrue: [range first = (startPos - 2) & (range size = (text size - (startPos - 2) + 1))]
                  ifFalse: [range first = idx & (range size = 0)]])! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:56'!
testTextReplacement1

	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

   | text1 string replacement startPos length startPosInRep string2 |
   text1 := (string := 'This is a simple text' copy) asText.
   "without the copy, we would modify a constant that the compiler attached at the compiled method. "
   startPos := string findString: 'simple'. 
   length  := 'simple' size.
   replacement := (string2 := 'both simple and short*') asText.
   startPosInRep :=  string2 findString: 'short'.
   text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
   self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:57'!
testTextReplacement2


  	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
    	This test examines the preservation of this assertion for in-place replacement.
    	Here, the replacement text has trailing characters. "

	| text1 string replacement startPos length startPosInRep string2 |
	text1 := (string := 'This is simple text' copy) asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	startPos := string findString: 'simple'. 
	length := 'simple' size.
	replacement := (string2 := 'both simple and short*************') asText.
	startPosInRep :=  string2 findString: 'short'.
	text1 replaceFrom: startPos 
        to: startPos + length - 1
        with: replacement
        startingAt: startPosInRep.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:58'!
testTextReplacement3

	"for a Text  t,
	the following assertion should always hold:
	t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement 
	Here, the replacement text is shorteer than the text that is shall replace. "

	self should: [self replacementExample3]  raise: Error! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:59'!
testTextReplacementAtStartPos1

	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

	| text1  replacement  length  |
	text1 := 'This is a simple text' copy asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	length  := 'This' size.
	replacement := 'That' asText.
	text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:00'!
testTextReplacementAtStartPos2

  	"for a Text  t,
     the following assertion should always hold:
     t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement "

	| text1  replacement  length  |
	text1 := 'This is a simple text' copy asText.
	"without the copy, we would modify a constant that the compiler attached at the compiled method. "
	length  := 'This' size.
	replacement := 'That********' asText.
	text1 replaceFrom: 1 
        to:   length
        with: replacement
        startingAt: 1.
	self assert: text1 string size = text1 runs size.
! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:00'!
testTextReplacementAtStartPos3

	"for a Text  t,
	the following assertion should always hold:
	t string size = t run size 
	This test examines the preservation of this assertion for in-place replacement 
	Here, the replacement text is shorteer than the text that is shall replace. "

	self should: [self replacementAtStartExample3] raise: Error! !

!TextAndTextStreamTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:44'!
testTextStreamAdjacentRunsWithIdentitcalAttributes

	"This test verifies that adjacent runs with identical attributes are coalesced."

	| ts text rangeOfBold |
	ts := TextStream on: (Text new: 50).
	ts nextPutAll: 'abc' asText.
	ts nextPutAll: 'def' asText allBold.
	ts nextPutAll: 'ghijk'.
	text := ts contents.
	rangeOfBold := text find:  TextEmphasis bold.
	text removeAttribute: TextEmphasis bold from: rangeOfBold first to: rangeOfBold last. 
     "now, check that only one run is left and that it has the correct size "
	self assert: text runs runs size = 1 & (text runs size = text string size).
! !

TestCase subclass: #TextLineEndingsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Text'!

!TextLineEndingsTest commentStamp: 'nk 11/1/2003 07:55' prior: 0!
This is a test case for Text>>withSqueakLineEndings and String>>withSqueakLineEndings.

The main problem we've seen with the Text version is that it doesn't preserve formatting correctly.!

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:06'!
testDecoratedTextConversionCrLF

	| text newText |
	text := ('123456', String crlf, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size - 1) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 9).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:06'!
testDecoratedTextConversionJustLF

	| text newText |
	text := ('123456', String lf, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 8).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testDecoratedTextConversionNoLF

	| text newText |
	text := ('123456', String cr, '901234') asText.
	text addAttribute: TextColor blue from: 4 to: 10.
	text addAttribute: TextColor red from: 6 to: 9.
	text addAttribute: TextEmphasis bold.
	newText := text withSqueakLineEndings.
	self assert: ((text size) = newText size).
	self assert: (newText size = newText runs size).
	self assert: (newText attributesAt: 6) = (text attributesAt: 6).
	self assert: (newText attributesAt: 8) = (text attributesAt: 8).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionCrLF

	| string newText |
	string := 'This is a test', String crlf, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size - 1) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionJustCR

	| string newText |
	string := 'This is a test', String cr, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testSimpleTextConversionJustLF

	| string newText |
	string := 'This is a test', String lf, 'of the conversion'.
	newText := string asText withSqueakLineEndings.
	self assert: ((string size) = newText size).
	self assert: (newText size = newText runs size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionCrLF

	| string newString |
	string := 'This is a test', String crlf, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: ((string size - 1) = newString size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionJustLF

	| string newString |
	string := 'This is a test', String lf, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: (string size = newString size).! !

!TextLineEndingsTest methodsFor: 'tests' stamp: 'sd 6/5/2005 10:07'!
testStringConversionNoLF

	| string newString |
	string := 'This is a test', String cr, 'of the conversion'.
	newString := string withSqueakLineEndings.
	self assert: (string = newString).! !

TestCase subclass: #TranscriptTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Streams'!

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 8/3/2008 15:53'!
testCloseCauseFlushing
	|transcriptStr|
	transcriptStr := TranscriptStream new.
	transcriptStr cr; space; tab; nextPut: $A; nextPutAll: 'B'.
	self assert: transcriptStr size = 5.
	transcriptStr close.
	self assert: transcriptStr isEmpty! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 7/19/2008 00:26'!
testOtherInstancesOfTranscriptStreamAsString
	self deny: TranscriptStream new asString = 'Transcript'! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 7/19/2008 00:25'!
testOtherInstancesOfTranscriptStreamPrintString
	self deny: TranscriptStream new printString = 'Transcript'! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 8/26/2008 08:56'!
testOtherInstancesOfTranscriptStreamSelfEvaluating
	self deny: TranscriptStream new isSelfEvaluating! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 7/19/2008 00:26'!
testTranscriptAsString
	self assert: Transcript asString = 'Transcript'! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 7/19/2008 00:24'!
testTranscriptPrintString
	self assert: Transcript printString = 'Transcript'! !

!TranscriptTest methodsFor: 'testing' stamp: 'Noury 8/26/2008 08:55'!
testTranscriptisSelfEvaluating
	self assert: Transcript isSelfEvaluating! !

TestCase subclass: #WideCharacterSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CollectionsTests-Support'!

!WideCharacterSetTest commentStamp: 'nice 11/19/2007 22:45' prior: 0!
WideCharacterSetTest holds tests for WideCharacterSet!

!WideCharacterSetTest methodsFor: 'testing' stamp: 'nice 11/19/2007 23:36'!
testAddingToCharacterSet

	| cs wcs wc |
	cs := CharacterSet newFrom: 'aeiouy'.
	wcs := cs copy.
	wc := 4452 asCharacter.
	
	self shouldnt: [wcs add: wc] raise: Error description: 'adding a WideCharacter to an ordinary CharacterSet should turn it into a WideCharacterSet'.

	self should: [wcs size = (cs size + 1)] description: 'We just added a Character, size should be increased by one'.
	self shouldnt: [wcs = cs] description: 'We just added a Character, sets should not be equal'.
	self shouldnt: [cs = wcs] description: 'We just added a Character, sets should not be equal'.
	self should: [cs allSatisfy: [:char | wcs includes: char]] description: 'Each character of the original CharacterSet should be included in the WideCharacterSet'.
	self should: [wcs hasWideCharacters] description: 'We just added a WideCharacter, so this WideCharacterSet definitely has one'.
	self should: [wcs includes: wc] description: 'We just added this WideCharacter, so this WideCharacterSet should include it'.
	
	wcs add: wc.
	self should: [wcs size = (cs size + 1)] description: 'We just added a Character already included in the set, size should be unchanged'.
	
	wcs remove: wc.
	self should: [wcs size = cs size] description: 'We added then removed a Character, now size should be equal to original'.
	self shouldnt: [wcs hasWideCharacters] description: 'We just removed the only WideCharacter, so this WideCharacterSet definitely has no WideCharacter'.
	
	self should: [wcs = cs] description: 'A WideCharacterSet can be equal to an Ordinary CharacterSet'.
	self should: [cs = wcs] description: 'An ordinary CharacterSet can be equal to a WideCharacterSet'.
	self should: [cs hash = wcs hash] description: 'If some objects are equal, then they should have same hash code'.
	
	! !

!WideCharacterSetTest methodsFor: 'testing' stamp: 'nice 11/20/2007 00:52'!
testCopy
    | theOriginal theCopy |
    theOriginal := WideCharacterSet newFrom: ('abc' copyWith: 300 asCharacter).
    theCopy := theOriginal copy.
    theCopy remove: $a.
    ^self should: [theOriginal includes: $a] description: 'Changing the copy should not change the original'.! !

!WideCharacterSetTest methodsFor: 'testing' stamp: 'nice 11/19/2007 23:34'!
testCreation
	"By now, only creation method is newFrom:"

	| cs1 wcs1 cs2 wcs2 byteString wideString |
	byteString := 'aeiouy'.
	wideString := 'aeiouy' copyWith: 340 asCharacter.

	cs1 := CharacterSet newFrom: byteString.
	wcs1 := WideCharacterSet newFrom: byteString.
	self should: [wcs1 = cs1].
	self should: [wcs1 size = byteString "asSet" size].
	
	cs2 := CharacterSet newFrom: wideString.
	wcs2 := WideCharacterSet newFrom: wideString.
	self should: [wcs2 = cs2].
	self should: [wcs2 size = wideString "asSet" size].
	
	self should: [(byteString indexOfAnyOf: wcs1) = 1] description: 'This should used optimized byteArrayMap method'.
	self should: [(byteString indexOfAnyOf: wcs2) = 1] description: 'This should used optimized byteArrayMap method'.
	
	self should: [('bcd' indexOfAnyOf: wcs1) = 0] description: 'This should used optimized byteArrayMap method'.
	self should: [('bcd' indexOfAnyOf: wcs2) = 0] description: 'This should used optimized byteArrayMap method'.! !
